/* Generated by Cython 0.27.3 */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_27_3"
#define CYTHON_FUTURE_DIVISION 1
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (0 && PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#if PY_VERSION_HEX < 0x030700A0 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject **args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject **args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif


#define __PYX_ERR(f_index, lineno, Ln_error) \
{ \
  __pyx_filename = __pyx_f[f_index]; __pyx_lineno = lineno; __pyx_clineno = __LINE__; goto Ln_error; \
}

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__multifruits
#define __PYX_HAVE_API__multifruits
#include <string.h>
#include <stdio.h>
#include "pythread.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "multifruits.pyx",
  "stringsource",
  "type.pxd",
  "bool.pxd",
  "complex.pxd",
};

/*--- Type declarations ---*/
struct __pyx_obj_11multifruits_Parser;

/* "multifruits.pyx":6
 * 
 * 
 * cdef enum state:             # <<<<<<<<<<<<<<
 *     PREAMBLE,  # 0
 *     PREAMBLE_HY,
 */
enum __pyx_t_11multifruits_state {
  __pyx_e_11multifruits_PREAMBLE,
  __pyx_e_11multifruits_PREAMBLE_HY,
  __pyx_e_11multifruits_FIRST_BOUNDARY,
  __pyx_e_11multifruits_FIRST_BOUNDARY_DONE,
  __pyx_e_11multifruits_HEADER_NAME_START,
  __pyx_e_11multifruits_HEADER_NAME,
  __pyx_e_11multifruits_HEADER_VALUE_START,
  __pyx_e_11multifruits_HEADER_VALUE,
  __pyx_e_11multifruits_HEADER_VALUE_CR,
  __pyx_e_11multifruits_HEADERS_DONE,
  __pyx_e_11multifruits_DATA,
  __pyx_e_11multifruits_DATA_CR,
  __pyx_e_11multifruits_DATA_CR_LF,
  __pyx_e_11multifruits_DATA_CR_LF_HY,
  __pyx_e_11multifruits_DATA_BOUNDARY,
  __pyx_e_11multifruits_DATA_BOUNDARY_DONE,
  __pyx_e_11multifruits_DATA_BOUNDARY_DONE_CR_LF,
  __pyx_e_11multifruits_DATA_BOUNDARY_DONE_HY_HY,
  __pyx_e_11multifruits_EPILOGUE
};

/* "multifruits.pyx":27
 *     EPILOGUE,
 * 
 * cdef class Parser:             # <<<<<<<<<<<<<<
 * 
 *     cdef:
 */
struct __pyx_obj_11multifruits_Parser {
  PyObject_HEAD
  struct __pyx_vtabstruct_11multifruits_Parser *__pyx_vtab;
  PyObject *_boundary;
  unsigned int _boundary_index;
  unsigned int _boundary_length;
  unsigned char _state;
  PyObject *_current_header_name;
  PyObject *_current_header_value;
  PyObject *_on_body_begin;
  PyObject *_on_part_begin;
  PyObject *_on_header;
  PyObject *_on_headers_complete;
  PyObject *_on_data;
  PyObject *_on_part_complete;
  PyObject *_on_body_complete;
};



struct __pyx_vtabstruct_11multifruits_Parser {
  PyObject *(*_feed_data)(struct __pyx_obj_11multifruits_Parser *, char *);
};
static struct __pyx_vtabstruct_11multifruits_Parser *__pyx_vtabptr_11multifruits_Parser;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* DictGetItem.proto */
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
static PyObject *__Pyx_PyDict_GetItem(PyObject *d, PyObject* key) {
    PyObject *value;
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (!PyErr_Occurred()) {
            PyObject* args = PyTuple_Pack(1, key);
            if (likely(args))
                PyErr_SetObject(PyExc_KeyError, args);
            Py_XDECREF(args);
        }
        return NULL;
    }
    Py_INCREF(value);
    return value;
}
#else
    #define __Pyx_PyDict_GetItem(d, key) PyObject_GetItem(d, key)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* IncludeStringH.proto */
#include <string.h>

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* GetModuleGlobalName.proto */
static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name);

/* bytes_index.proto */
static CYTHON_INLINE char __Pyx_PyBytes_GetItemInt(PyObject* bytes, Py_ssize_t index, int check_bounds);

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* PyDictContains.proto */
static CYTHON_INLINE int __Pyx_PyDict_ContainsTF(PyObject* item, PyObject* dict, int eq) {
    int result = PyDict_Contains(dict, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* dict_getitem_default.proto */
static PyObject* __Pyx_PyDict_GetItemDefault(PyObject* d, PyObject* key, PyObject* default_value);

/* PySequenceContains.proto */
static CYTHON_INLINE int __Pyx_PySequence_ContainsTF(PyObject* item, PyObject* seq, int eq) {
    int result = PySequence_Contains(seq, item);
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_int(unsigned int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_char(unsigned char value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_char(char value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE unsigned int __Pyx_PyInt_As_unsigned_int(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE unsigned char __Pyx_PyInt_As_unsigned_char(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE char __Pyx_PyInt_As_char(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* PyIdentifierFromString.proto */
#if !defined(__Pyx_PyIdentifier_FromString)
#if PY_MAJOR_VERSION < 3
  #define __Pyx_PyIdentifier_FromString(s) PyString_FromString(s)
#else
  #define __Pyx_PyIdentifier_FromString(s) PyUnicode_FromString(s)
#endif
#endif

/* ModuleImport.proto */
static PyObject *__Pyx_ImportModule(const char *name);

/* TypeImport.proto */
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name, size_t size, int strict);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static PyObject *__pyx_f_11multifruits_6Parser__feed_data(struct __pyx_obj_11multifruits_Parser *__pyx_v_self, char *__pyx_v_data); /* proto*/

/* Module declarations from 'cpython.version' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'cpython.exc' */

/* Module declarations from 'cpython.module' */

/* Module declarations from 'cpython.mem' */

/* Module declarations from 'cpython.tuple' */

/* Module declarations from 'cpython.list' */

/* Module declarations from 'cpython.sequence' */

/* Module declarations from 'cpython.mapping' */

/* Module declarations from 'cpython.iterator' */

/* Module declarations from 'cpython.number' */

/* Module declarations from 'cpython.int' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.bool' */
static PyTypeObject *__pyx_ptype_7cpython_4bool_bool = 0;

/* Module declarations from 'cpython.long' */

/* Module declarations from 'cpython.float' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.complex' */
static PyTypeObject *__pyx_ptype_7cpython_7complex_complex = 0;

/* Module declarations from 'cpython.string' */

/* Module declarations from 'cpython.unicode' */

/* Module declarations from 'cpython.dict' */

/* Module declarations from 'cpython.instance' */

/* Module declarations from 'cpython.function' */

/* Module declarations from 'cpython.method' */

/* Module declarations from 'cpython.weakref' */

/* Module declarations from 'cpython.getargs' */

/* Module declarations from 'cpython.pythread' */

/* Module declarations from 'cpython.pystate' */

/* Module declarations from 'cpython.cobject' */

/* Module declarations from 'cpython.oldbuffer' */

/* Module declarations from 'cpython.set' */

/* Module declarations from 'cpython.buffer' */

/* Module declarations from 'cpython.bytes' */

/* Module declarations from 'cpython.pycapsule' */

/* Module declarations from 'cpython' */

/* Module declarations from 'multifruits' */
static PyTypeObject *__pyx_ptype_11multifruits_Parser = 0;
static PyObject *__pyx_f_11multifruits_parse_content_disposition(PyObject *, int __pyx_skip_dispatch); /*proto*/
static PyObject *__pyx_f_11multifruits___pyx_unpickle_Parser__set_state(struct __pyx_obj_11multifruits_Parser *, PyObject *); /*proto*/
#define __Pyx_MODULE_NAME "multifruits"
extern int __pyx_module_is_main_multifruits;
int __pyx_module_is_main_multifruits = 0;

/* Implementation of 'multifruits' */
static PyObject *__pyx_builtin_KeyError;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_LookupError;
static PyObject *__pyx_builtin_UnicodeDecodeError;
static const char __pyx_k__8[] = "\r";
static const char __pyx_k__10[] = "\r\n";
static const char __pyx_k__12[] = "\r\n-";
static const char __pyx_k__14[] = "\r\n--";
static const char __pyx_k__19[] = "";
static const char __pyx_k__20[] = "\\";
static const char __pyx_k__23[] = "''";
static const char __pyx_k_get[] = "get";
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_lower[] = "lower";
static const char __pyx_k_split[] = "split";
static const char __pyx_k_decode[] = "decode";
static const char __pyx_k_errors[] = "errors";
static const char __pyx_k_ignore[] = "ignore";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_params[] = "params";
static const char __pyx_k_pickle[] = "pickle";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_handler[] = "handler";
static const char __pyx_k_on_data[] = "on_data";
static const char __pyx_k_replace[] = "replace";
static const char __pyx_k_KeyError[] = "KeyError";
static const char __pyx_k_boundary[] = "boundary";
static const char __pyx_k_encoding[] = "encoding";
static const char __pyx_k_filename[] = "filename*";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_pyx_type[] = "__pyx_type";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_on_header[] = "on_header";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_filename_2[] = "filename";
static const char __pyx_k_pyx_result[] = "__pyx_result";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_HEADER_NAME[] = "HEADER_NAME";
static const char __pyx_k_LookupError[] = "LookupError";
static const char __pyx_k_PickleError[] = "PickleError";
static const char __pyx_k_multifruits[] = "multifruits";
static const char __pyx_k_HEADERS_DONE[] = "HEADERS_DONE";
static const char __pyx_k_content_type[] = "content_type";
static const char __pyx_k_pyx_checksum[] = "__pyx_checksum";
static const char __pyx_k_stringsource[] = "stringsource";
static const char __pyx_k_on_body_begin[] = "on_body_begin";
static const char __pyx_k_on_part_begin[] = "on_part_begin";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_FIRST_BOUNDARY[] = "FIRST_BOUNDARY";
static const char __pyx_k_on_header_name[] = "on_header_name";
static const char __pyx_k_HEADER_VALUE_CR[] = "HEADER_VALUE_CR";
static const char __pyx_k_multifruits_pyx[] = "multifruits.pyx";
static const char __pyx_k_on_header_value[] = "on_header_value";
static const char __pyx_k_pyx_PickleError[] = "__pyx_PickleError";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_FIRST_BOUNDARY_r[] = "FIRST_BOUNDARY: \\r";
static const char __pyx_k_extract_filename[] = "extract_filename";
static const char __pyx_k_on_body_complete[] = "on_body_complete";
static const char __pyx_k_on_part_complete[] = "on_part_complete";
static const char __pyx_k_DATA_BOUNDARY_DONE[] = "DATA_BOUNDARY_DONE";
static const char __pyx_k_UnicodeDecodeError[] = "UnicodeDecodeError";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_FIRST_BOUNDARY_DONE[] = "FIRST_BOUNDARY_DONE";
static const char __pyx_k_on_headers_complete[] = "on_headers_complete";
static const char __pyx_k_pyx_unpickle_Parser[] = "__pyx_unpickle_Parser";
static const char __pyx_k_maybe_call_on_header[] = "_maybe_call_on_header";
static const char __pyx_k_DATA_BOUNDARY_DONE_CR_LF[] = "DATA_BOUNDARY_DONE_CR_LF";
static const char __pyx_k_DATA_BOUNDARY_DONE_HY_HY[] = "DATA_BOUNDARY_DONE_HY_HY";
static const char __pyx_k_Incompatible_checksums_s_vs_0x7a[] = "Incompatible checksums (%s vs 0x7a58f64 = (_boundary, _boundary_index, _boundary_length, _current_header_name, _current_header_value, _on_body_begin, _on_body_complete, _on_data, _on_header, _on_headers_complete, _on_part_begin, _on_part_complete, _state))";
static const char __pyx_k_Missing_boundary_in_Content_Type[] = "Missing boundary in Content-Type.";
static PyObject *__pyx_n_u_DATA_BOUNDARY_DONE;
static PyObject *__pyx_n_u_DATA_BOUNDARY_DONE_CR_LF;
static PyObject *__pyx_n_u_DATA_BOUNDARY_DONE_HY_HY;
static PyObject *__pyx_n_u_FIRST_BOUNDARY;
static PyObject *__pyx_n_u_FIRST_BOUNDARY_DONE;
static PyObject *__pyx_kp_u_FIRST_BOUNDARY_r;
static PyObject *__pyx_n_u_HEADERS_DONE;
static PyObject *__pyx_n_u_HEADER_NAME;
static PyObject *__pyx_n_u_HEADER_VALUE_CR;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x7a;
static PyObject *__pyx_n_s_KeyError;
static PyObject *__pyx_n_s_LookupError;
static PyObject *__pyx_kp_u_Missing_boundary_in_Content_Type;
static PyObject *__pyx_n_s_PickleError;
static PyObject *__pyx_n_s_UnicodeDecodeError;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_kp_b__10;
static PyObject *__pyx_kp_b__12;
static PyObject *__pyx_kp_b__14;
static PyObject *__pyx_kp_b__19;
static PyObject *__pyx_kp_b__20;
static PyObject *__pyx_kp_b__23;
static PyObject *__pyx_kp_b__8;
static PyObject *__pyx_n_b_boundary;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_content_type;
static PyObject *__pyx_n_s_decode;
static PyObject *__pyx_n_s_dict;
static PyObject *__pyx_n_s_encoding;
static PyObject *__pyx_n_s_errors;
static PyObject *__pyx_n_s_extract_filename;
static PyObject *__pyx_kp_b_filename;
static PyObject *__pyx_n_b_filename_2;
static PyObject *__pyx_n_s_filename_2;
static PyObject *__pyx_n_s_get;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_handler;
static PyObject *__pyx_n_u_ignore;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_lower;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_maybe_call_on_header;
static PyObject *__pyx_n_s_multifruits;
static PyObject *__pyx_kp_s_multifruits_pyx;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_new;
static PyObject *__pyx_n_u_on_body_begin;
static PyObject *__pyx_n_u_on_body_complete;
static PyObject *__pyx_n_u_on_data;
static PyObject *__pyx_n_u_on_header;
static PyObject *__pyx_n_s_on_header_name;
static PyObject *__pyx_n_s_on_header_value;
static PyObject *__pyx_n_s_on_headers_complete;
static PyObject *__pyx_n_u_on_headers_complete;
static PyObject *__pyx_n_u_on_part_begin;
static PyObject *__pyx_n_u_on_part_complete;
static PyObject *__pyx_n_s_params;
static PyObject *__pyx_n_s_pickle;
static PyObject *__pyx_n_s_pyx_PickleError;
static PyObject *__pyx_n_s_pyx_checksum;
static PyObject *__pyx_n_s_pyx_result;
static PyObject *__pyx_n_s_pyx_state;
static PyObject *__pyx_n_s_pyx_type;
static PyObject *__pyx_n_s_pyx_unpickle_Parser;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_replace;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_split;
static PyObject *__pyx_kp_s_stringsource;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_update;
static int __pyx_pf_11multifruits_6Parser___init__(struct __pyx_obj_11multifruits_Parser *__pyx_v_self, PyObject *__pyx_v_handler, PyObject *__pyx_v_content_type); /* proto */
static PyObject *__pyx_pf_11multifruits_6Parser_2_maybe_call_on_header(struct __pyx_obj_11multifruits_Parser *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11multifruits_6Parser_4on_header_name(struct __pyx_obj_11multifruits_Parser *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_11multifruits_6Parser_6on_header_value(struct __pyx_obj_11multifruits_Parser *__pyx_v_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_11multifruits_6Parser_8on_headers_complete(struct __pyx_obj_11multifruits_Parser *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11multifruits_6Parser_10feed_data(struct __pyx_obj_11multifruits_Parser *__pyx_v_self, char *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_11multifruits_6Parser_12__reduce_cython__(struct __pyx_obj_11multifruits_Parser *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_11multifruits_6Parser_14__setstate_cython__(struct __pyx_obj_11multifruits_Parser *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_11multifruits_parse_content_disposition(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_data); /* proto */
static PyObject *__pyx_pf_11multifruits_2extract_filename(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_params); /* proto */
static PyObject *__pyx_pf_11multifruits_4__pyx_unpickle_Parser(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_11multifruits_Parser(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_int_128290660;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__16;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__18;
static PyObject *__pyx_tuple__21;
static PyObject *__pyx_tuple__22;
static PyObject *__pyx_tuple__24;
static PyObject *__pyx_tuple__25;
static PyObject *__pyx_tuple__27;
static PyObject *__pyx_codeobj__26;
static PyObject *__pyx_codeobj__28;

/* "multifruits.pyx":39
 *         _on_data, _on_part_complete, _on_body_complete
 * 
 *     def __init__(self, handler, bytes content_type):             # <<<<<<<<<<<<<<
 *         cdef dict params
 *         cdef bytes _
 */

/* Python wrapper */
static int __pyx_pw_11multifruits_6Parser_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_11multifruits_6Parser_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_handler = 0;
  PyObject *__pyx_v_content_type = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_handler,&__pyx_n_s_content_type,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_handler)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_content_type)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 39, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 39, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_handler = values[0];
    __pyx_v_content_type = ((PyObject*)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 39, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("multifruits.Parser.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_content_type), (&PyBytes_Type), 1, "content_type", 1))) __PYX_ERR(0, 39, __pyx_L1_error)
  __pyx_r = __pyx_pf_11multifruits_6Parser___init__(((struct __pyx_obj_11multifruits_Parser *)__pyx_v_self), __pyx_v_handler, __pyx_v_content_type);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_11multifruits_6Parser___init__(struct __pyx_obj_11multifruits_Parser *__pyx_v_self, PyObject *__pyx_v_handler, PyObject *__pyx_v_content_type) {
  PyObject *__pyx_v_params = 0;
  CYTHON_UNUSED PyObject *__pyx_v__ = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *(*__pyx_t_5)(PyObject *);
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "multifruits.pyx":42
 *         cdef dict params
 *         cdef bytes _
 *         _, params = parse_content_disposition(content_type)             # <<<<<<<<<<<<<<
 *         try:
 *             self._boundary = params[b'boundary']
 */
  __pyx_t_1 = __pyx_f_11multifruits_parse_content_disposition(__pyx_v_content_type, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
    PyObject* sequence = __pyx_t_1;
    #if !CYTHON_COMPILING_IN_PYPY
    Py_ssize_t size = Py_SIZE(sequence);
    #else
    Py_ssize_t size = PySequence_Size(sequence);
    #endif
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 42, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1); 
    } else {
      __pyx_t_2 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyList_GET_ITEM(sequence, 1); 
    }
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_3);
    #else
    __pyx_t_2 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 42, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 42, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    #endif
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  } else {
    Py_ssize_t index = -1;
    __pyx_t_4 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 42, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_5 = Py_TYPE(__pyx_t_4)->tp_iternext;
    index = 0; __pyx_t_2 = __pyx_t_5(__pyx_t_4); if (unlikely(!__pyx_t_2)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_2);
    index = 1; __pyx_t_3 = __pyx_t_5(__pyx_t_4); if (unlikely(!__pyx_t_3)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_3);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_5(__pyx_t_4), 2) < 0) __PYX_ERR(0, 42, __pyx_L1_error)
    __pyx_t_5 = NULL;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    goto __pyx_L4_unpacking_done;
    __pyx_L3_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_5 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 42, __pyx_L1_error)
    __pyx_L4_unpacking_done:;
  }
  if (!(likely(PyBytes_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 42, __pyx_L1_error)
  if (!(likely(PyDict_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "dict", Py_TYPE(__pyx_t_3)->tp_name), 0))) __PYX_ERR(0, 42, __pyx_L1_error)
  __pyx_v__ = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_v_params = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "multifruits.pyx":43
 *         cdef bytes _
 *         _, params = parse_content_disposition(content_type)
 *         try:             # <<<<<<<<<<<<<<
 *             self._boundary = params[b'boundary']
 *         except KeyError:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8);
    __Pyx_XGOTREF(__pyx_t_6);
    __Pyx_XGOTREF(__pyx_t_7);
    __Pyx_XGOTREF(__pyx_t_8);
    /*try:*/ {

      /* "multifruits.pyx":44
 *         _, params = parse_content_disposition(content_type)
 *         try:
 *             self._boundary = params[b'boundary']             # <<<<<<<<<<<<<<
 *         except KeyError:
 *             raise ValueError('Missing boundary in Content-Type.')
 */
      if (unlikely(__pyx_v_params == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 44, __pyx_L5_error)
      }
      __pyx_t_1 = __Pyx_PyDict_GetItem(__pyx_v_params, __pyx_n_b_boundary); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 44, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 44, __pyx_L5_error)
      __Pyx_GIVEREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_v_self->_boundary);
      __Pyx_DECREF(__pyx_v_self->_boundary);
      __pyx_v_self->_boundary = ((PyObject*)__pyx_t_1);
      __pyx_t_1 = 0;

      /* "multifruits.pyx":43
 *         cdef bytes _
 *         _, params = parse_content_disposition(content_type)
 *         try:             # <<<<<<<<<<<<<<
 *             self._boundary = params[b'boundary']
 *         except KeyError:
 */
    }
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    goto __pyx_L10_try_end;
    __pyx_L5_error:;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "multifruits.pyx":45
 *         try:
 *             self._boundary = params[b'boundary']
 *         except KeyError:             # <<<<<<<<<<<<<<
 *             raise ValueError('Missing boundary in Content-Type.')
 *         self._boundary_length = len(self._boundary)
 */
    __pyx_t_9 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_KeyError);
    if (__pyx_t_9) {
      __Pyx_AddTraceback("multifruits.Parser.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_1, &__pyx_t_3, &__pyx_t_2) < 0) __PYX_ERR(0, 45, __pyx_L7_except_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_GOTREF(__pyx_t_2);

      /* "multifruits.pyx":46
 *             self._boundary = params[b'boundary']
 *         except KeyError:
 *             raise ValueError('Missing boundary in Content-Type.')             # <<<<<<<<<<<<<<
 *         self._boundary_length = len(self._boundary)
 *         self._current_header_name = None
 */
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 46, __pyx_L7_except_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_Raise(__pyx_t_4, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __PYX_ERR(0, 46, __pyx_L7_except_error)
    }
    goto __pyx_L7_except_error;
    __pyx_L7_except_error:;

    /* "multifruits.pyx":43
 *         cdef bytes _
 *         _, params = parse_content_disposition(content_type)
 *         try:             # <<<<<<<<<<<<<<
 *             self._boundary = params[b'boundary']
 *         except KeyError:
 */
    __Pyx_XGIVEREF(__pyx_t_6);
    __Pyx_XGIVEREF(__pyx_t_7);
    __Pyx_XGIVEREF(__pyx_t_8);
    __Pyx_ExceptionReset(__pyx_t_6, __pyx_t_7, __pyx_t_8);
    goto __pyx_L1_error;
    __pyx_L10_try_end:;
  }

  /* "multifruits.pyx":47
 *         except KeyError:
 *             raise ValueError('Missing boundary in Content-Type.')
 *         self._boundary_length = len(self._boundary)             # <<<<<<<<<<<<<<
 *         self._current_header_name = None
 *         self._current_header_value = None
 */
  __pyx_t_2 = __pyx_v_self->_boundary;
  __Pyx_INCREF(__pyx_t_2);
  if (unlikely(__pyx_t_2 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 47, __pyx_L1_error)
  }
  __pyx_t_10 = PyBytes_GET_SIZE(__pyx_t_2); if (unlikely(__pyx_t_10 == ((Py_ssize_t)-1))) __PYX_ERR(0, 47, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_self->_boundary_length = __pyx_t_10;

  /* "multifruits.pyx":48
 *             raise ValueError('Missing boundary in Content-Type.')
 *         self._boundary_length = len(self._boundary)
 *         self._current_header_name = None             # <<<<<<<<<<<<<<
 *         self._current_header_value = None
 *         self._state = 0
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_current_header_name);
  __Pyx_DECREF(__pyx_v_self->_current_header_name);
  __pyx_v_self->_current_header_name = ((PyObject*)Py_None);

  /* "multifruits.pyx":49
 *         self._boundary_length = len(self._boundary)
 *         self._current_header_name = None
 *         self._current_header_value = None             # <<<<<<<<<<<<<<
 *         self._state = 0
 *         self._boundary_index = 0
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_current_header_value);
  __Pyx_DECREF(__pyx_v_self->_current_header_value);
  __pyx_v_self->_current_header_value = ((PyObject*)Py_None);

  /* "multifruits.pyx":50
 *         self._current_header_name = None
 *         self._current_header_value = None
 *         self._state = 0             # <<<<<<<<<<<<<<
 *         self._boundary_index = 0
 *         self._on_body_begin = getattr(handler, 'on_body_begin', None)
 */
  __pyx_v_self->_state = 0;

  /* "multifruits.pyx":51
 *         self._current_header_value = None
 *         self._state = 0
 *         self._boundary_index = 0             # <<<<<<<<<<<<<<
 *         self._on_body_begin = getattr(handler, 'on_body_begin', None)
 *         self._on_part_begin = getattr(handler, 'on_part_begin', None)
 */
  __pyx_v_self->_boundary_index = 0;

  /* "multifruits.pyx":52
 *         self._state = 0
 *         self._boundary_index = 0
 *         self._on_body_begin = getattr(handler, 'on_body_begin', None)             # <<<<<<<<<<<<<<
 *         self._on_part_begin = getattr(handler, 'on_part_begin', None)
 *         self._on_header = getattr(handler, 'on_header', None)
 */
  __pyx_t_2 = __Pyx_GetAttr3(__pyx_v_handler, __pyx_n_u_on_body_begin, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 52, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->_on_body_begin);
  __Pyx_DECREF(__pyx_v_self->_on_body_begin);
  __pyx_v_self->_on_body_begin = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "multifruits.pyx":53
 *         self._boundary_index = 0
 *         self._on_body_begin = getattr(handler, 'on_body_begin', None)
 *         self._on_part_begin = getattr(handler, 'on_part_begin', None)             # <<<<<<<<<<<<<<
 *         self._on_header = getattr(handler, 'on_header', None)
 *         self._on_headers_complete = getattr(handler, 'on_headers_complete', None)
 */
  __pyx_t_2 = __Pyx_GetAttr3(__pyx_v_handler, __pyx_n_u_on_part_begin, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->_on_part_begin);
  __Pyx_DECREF(__pyx_v_self->_on_part_begin);
  __pyx_v_self->_on_part_begin = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "multifruits.pyx":54
 *         self._on_body_begin = getattr(handler, 'on_body_begin', None)
 *         self._on_part_begin = getattr(handler, 'on_part_begin', None)
 *         self._on_header = getattr(handler, 'on_header', None)             # <<<<<<<<<<<<<<
 *         self._on_headers_complete = getattr(handler, 'on_headers_complete', None)
 *         self._on_data = getattr(handler, 'on_data', None)
 */
  __pyx_t_2 = __Pyx_GetAttr3(__pyx_v_handler, __pyx_n_u_on_header, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 54, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->_on_header);
  __Pyx_DECREF(__pyx_v_self->_on_header);
  __pyx_v_self->_on_header = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "multifruits.pyx":55
 *         self._on_part_begin = getattr(handler, 'on_part_begin', None)
 *         self._on_header = getattr(handler, 'on_header', None)
 *         self._on_headers_complete = getattr(handler, 'on_headers_complete', None)             # <<<<<<<<<<<<<<
 *         self._on_data = getattr(handler, 'on_data', None)
 *         self._on_part_complete = getattr(handler, 'on_part_complete', None)
 */
  __pyx_t_2 = __Pyx_GetAttr3(__pyx_v_handler, __pyx_n_u_on_headers_complete, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 55, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->_on_headers_complete);
  __Pyx_DECREF(__pyx_v_self->_on_headers_complete);
  __pyx_v_self->_on_headers_complete = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "multifruits.pyx":56
 *         self._on_header = getattr(handler, 'on_header', None)
 *         self._on_headers_complete = getattr(handler, 'on_headers_complete', None)
 *         self._on_data = getattr(handler, 'on_data', None)             # <<<<<<<<<<<<<<
 *         self._on_part_complete = getattr(handler, 'on_part_complete', None)
 *         self._on_body_complete = getattr(handler, 'on_body_complete', None)
 */
  __pyx_t_2 = __Pyx_GetAttr3(__pyx_v_handler, __pyx_n_u_on_data, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->_on_data);
  __Pyx_DECREF(__pyx_v_self->_on_data);
  __pyx_v_self->_on_data = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "multifruits.pyx":57
 *         self._on_headers_complete = getattr(handler, 'on_headers_complete', None)
 *         self._on_data = getattr(handler, 'on_data', None)
 *         self._on_part_complete = getattr(handler, 'on_part_complete', None)             # <<<<<<<<<<<<<<
 *         self._on_body_complete = getattr(handler, 'on_body_complete', None)
 * 
 */
  __pyx_t_2 = __Pyx_GetAttr3(__pyx_v_handler, __pyx_n_u_on_part_complete, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 57, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->_on_part_complete);
  __Pyx_DECREF(__pyx_v_self->_on_part_complete);
  __pyx_v_self->_on_part_complete = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "multifruits.pyx":58
 *         self._on_data = getattr(handler, 'on_data', None)
 *         self._on_part_complete = getattr(handler, 'on_part_complete', None)
 *         self._on_body_complete = getattr(handler, 'on_body_complete', None)             # <<<<<<<<<<<<<<
 * 
 *     def _maybe_call_on_header(self):
 */
  __pyx_t_2 = __Pyx_GetAttr3(__pyx_v_handler, __pyx_n_u_on_body_complete, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->_on_body_complete);
  __Pyx_DECREF(__pyx_v_self->_on_body_complete);
  __pyx_v_self->_on_body_complete = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "multifruits.pyx":39
 *         _on_data, _on_part_complete, _on_body_complete
 * 
 *     def __init__(self, handler, bytes content_type):             # <<<<<<<<<<<<<<
 *         cdef dict params
 *         cdef bytes _
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("multifruits.Parser.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_params);
  __Pyx_XDECREF(__pyx_v__);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "multifruits.pyx":60
 *         self._on_body_complete = getattr(handler, 'on_body_complete', None)
 * 
 *     def _maybe_call_on_header(self):             # <<<<<<<<<<<<<<
 *         if self._current_header_value is not None:
 *             if self._on_header is not None:
 */

/* Python wrapper */
static PyObject *__pyx_pw_11multifruits_6Parser_3_maybe_call_on_header(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_11multifruits_6Parser_3_maybe_call_on_header(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_maybe_call_on_header (wrapper)", 0);
  __pyx_r = __pyx_pf_11multifruits_6Parser_2_maybe_call_on_header(((struct __pyx_obj_11multifruits_Parser *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11multifruits_6Parser_2_maybe_call_on_header(struct __pyx_obj_11multifruits_Parser *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  __Pyx_RefNannySetupContext("_maybe_call_on_header", 0);

  /* "multifruits.pyx":61
 * 
 *     def _maybe_call_on_header(self):
 *         if self._current_header_value is not None:             # <<<<<<<<<<<<<<
 *             if self._on_header is not None:
 *                 self._on_header(self._current_header_name, self._current_header_value)
 */
  __pyx_t_1 = (__pyx_v_self->_current_header_value != ((PyObject*)Py_None));
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "multifruits.pyx":62
 *     def _maybe_call_on_header(self):
 *         if self._current_header_value is not None:
 *             if self._on_header is not None:             # <<<<<<<<<<<<<<
 *                 self._on_header(self._current_header_name, self._current_header_value)
 *             self._current_header_name = self._current_header_value = None
 */
    __pyx_t_2 = (__pyx_v_self->_on_header != Py_None);
    __pyx_t_1 = (__pyx_t_2 != 0);
    if (__pyx_t_1) {

      /* "multifruits.pyx":63
 *         if self._current_header_value is not None:
 *             if self._on_header is not None:
 *                 self._on_header(self._current_header_name, self._current_header_value)             # <<<<<<<<<<<<<<
 *             self._current_header_name = self._current_header_value = None
 * 
 */
      __Pyx_INCREF(__pyx_v_self->_on_header);
      __pyx_t_4 = __pyx_v_self->_on_header; __pyx_t_5 = NULL;
      __pyx_t_6 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
          __pyx_t_6 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_self->_current_header_name, __pyx_v_self->_current_header_value};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 63, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
        PyObject *__pyx_temp[3] = {__pyx_t_5, __pyx_v_self->_current_header_name, __pyx_v_self->_current_header_value};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 63, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
      } else
      #endif
      {
        __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 63, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        if (__pyx_t_5) {
          __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
        }
        __Pyx_INCREF(__pyx_v_self->_current_header_name);
        __Pyx_GIVEREF(__pyx_v_self->_current_header_name);
        PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, __pyx_v_self->_current_header_name);
        __Pyx_INCREF(__pyx_v_self->_current_header_value);
        __Pyx_GIVEREF(__pyx_v_self->_current_header_value);
        PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_v_self->_current_header_value);
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_7, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 63, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

      /* "multifruits.pyx":62
 *     def _maybe_call_on_header(self):
 *         if self._current_header_value is not None:
 *             if self._on_header is not None:             # <<<<<<<<<<<<<<
 *                 self._on_header(self._current_header_name, self._current_header_value)
 *             self._current_header_name = self._current_header_value = None
 */
    }

    /* "multifruits.pyx":64
 *             if self._on_header is not None:
 *                 self._on_header(self._current_header_name, self._current_header_value)
 *             self._current_header_name = self._current_header_value = None             # <<<<<<<<<<<<<<
 * 
 *     def on_header_name(self, data):
 */
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_self->_current_header_name);
    __Pyx_DECREF(__pyx_v_self->_current_header_name);
    __pyx_v_self->_current_header_name = ((PyObject*)Py_None);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    __Pyx_GOTREF(__pyx_v_self->_current_header_value);
    __Pyx_DECREF(__pyx_v_self->_current_header_value);
    __pyx_v_self->_current_header_value = ((PyObject*)Py_None);

    /* "multifruits.pyx":61
 * 
 *     def _maybe_call_on_header(self):
 *         if self._current_header_value is not None:             # <<<<<<<<<<<<<<
 *             if self._on_header is not None:
 *                 self._on_header(self._current_header_name, self._current_header_value)
 */
  }

  /* "multifruits.pyx":60
 *         self._on_body_complete = getattr(handler, 'on_body_complete', None)
 * 
 *     def _maybe_call_on_header(self):             # <<<<<<<<<<<<<<
 *         if self._current_header_value is not None:
 *             if self._on_header is not None:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("multifruits.Parser._maybe_call_on_header", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "multifruits.pyx":66
 *             self._current_header_name = self._current_header_value = None
 * 
 *     def on_header_name(self, data):             # <<<<<<<<<<<<<<
 *         self._maybe_call_on_header()
 *         if self._current_header_name is None:
 */

/* Python wrapper */
static PyObject *__pyx_pw_11multifruits_6Parser_5on_header_name(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_pw_11multifruits_6Parser_5on_header_name(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("on_header_name (wrapper)", 0);
  __pyx_r = __pyx_pf_11multifruits_6Parser_4on_header_name(((struct __pyx_obj_11multifruits_Parser *)__pyx_v_self), ((PyObject *)__pyx_v_data));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11multifruits_6Parser_4on_header_name(struct __pyx_obj_11multifruits_Parser *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("on_header_name", 0);

  /* "multifruits.pyx":67
 * 
 *     def on_header_name(self, data):
 *         self._maybe_call_on_header()             # <<<<<<<<<<<<<<
 *         if self._current_header_name is None:
 *             self._current_header_name = data
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_maybe_call_on_header); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 67, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 67, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 67, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "multifruits.pyx":68
 *     def on_header_name(self, data):
 *         self._maybe_call_on_header()
 *         if self._current_header_name is None:             # <<<<<<<<<<<<<<
 *             self._current_header_name = data
 *         else:
 */
  __pyx_t_4 = (__pyx_v_self->_current_header_name == ((PyObject*)Py_None));
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "multifruits.pyx":69
 *         self._maybe_call_on_header()
 *         if self._current_header_name is None:
 *             self._current_header_name = data             # <<<<<<<<<<<<<<
 *         else:
 *             self._current_header_name += data
 */
    if (!(likely(PyBytes_CheckExact(__pyx_v_data))||((__pyx_v_data) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_v_data)->tp_name), 0))) __PYX_ERR(0, 69, __pyx_L1_error)
    __pyx_t_1 = __pyx_v_data;
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->_current_header_name);
    __Pyx_DECREF(__pyx_v_self->_current_header_name);
    __pyx_v_self->_current_header_name = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "multifruits.pyx":68
 *     def on_header_name(self, data):
 *         self._maybe_call_on_header()
 *         if self._current_header_name is None:             # <<<<<<<<<<<<<<
 *             self._current_header_name = data
 *         else:
 */
    goto __pyx_L3;
  }

  /* "multifruits.pyx":71
 *             self._current_header_name = data
 *         else:
 *             self._current_header_name += data             # <<<<<<<<<<<<<<
 * 
 *     def on_header_value(self, data):
 */
  /*else*/ {
    __pyx_t_1 = PyNumber_InPlaceAdd(__pyx_v_self->_current_header_name, __pyx_v_data); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 71, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 71, __pyx_L1_error)
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->_current_header_name);
    __Pyx_DECREF(__pyx_v_self->_current_header_name);
    __pyx_v_self->_current_header_name = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;
  }
  __pyx_L3:;

  /* "multifruits.pyx":66
 *             self._current_header_name = self._current_header_value = None
 * 
 *     def on_header_name(self, data):             # <<<<<<<<<<<<<<
 *         self._maybe_call_on_header()
 *         if self._current_header_name is None:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("multifruits.Parser.on_header_name", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "multifruits.pyx":73
 *             self._current_header_name += data
 * 
 *     def on_header_value(self, data):             # <<<<<<<<<<<<<<
 *         if self._current_header_value is None:
 *             self._current_header_value = data
 */

/* Python wrapper */
static PyObject *__pyx_pw_11multifruits_6Parser_7on_header_value(PyObject *__pyx_v_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_pw_11multifruits_6Parser_7on_header_value(PyObject *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("on_header_value (wrapper)", 0);
  __pyx_r = __pyx_pf_11multifruits_6Parser_6on_header_value(((struct __pyx_obj_11multifruits_Parser *)__pyx_v_self), ((PyObject *)__pyx_v_data));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11multifruits_6Parser_6on_header_value(struct __pyx_obj_11multifruits_Parser *__pyx_v_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("on_header_value", 0);

  /* "multifruits.pyx":74
 * 
 *     def on_header_value(self, data):
 *         if self._current_header_value is None:             # <<<<<<<<<<<<<<
 *             self._current_header_value = data
 *         else:
 */
  __pyx_t_1 = (__pyx_v_self->_current_header_value == ((PyObject*)Py_None));
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "multifruits.pyx":75
 *     def on_header_value(self, data):
 *         if self._current_header_value is None:
 *             self._current_header_value = data             # <<<<<<<<<<<<<<
 *         else:
 *             self._current_header_value += data
 */
    if (!(likely(PyBytes_CheckExact(__pyx_v_data))||((__pyx_v_data) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_v_data)->tp_name), 0))) __PYX_ERR(0, 75, __pyx_L1_error)
    __pyx_t_3 = __pyx_v_data;
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->_current_header_value);
    __Pyx_DECREF(__pyx_v_self->_current_header_value);
    __pyx_v_self->_current_header_value = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "multifruits.pyx":74
 * 
 *     def on_header_value(self, data):
 *         if self._current_header_value is None:             # <<<<<<<<<<<<<<
 *             self._current_header_value = data
 *         else:
 */
    goto __pyx_L3;
  }

  /* "multifruits.pyx":77
 *             self._current_header_value = data
 *         else:
 *             self._current_header_value += data             # <<<<<<<<<<<<<<
 * 
 *     def on_headers_complete(self):
 */
  /*else*/ {
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_self->_current_header_value, __pyx_v_data); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 77, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    if (!(likely(PyBytes_CheckExact(__pyx_t_3))||((__pyx_t_3) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_3)->tp_name), 0))) __PYX_ERR(0, 77, __pyx_L1_error)
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->_current_header_value);
    __Pyx_DECREF(__pyx_v_self->_current_header_value);
    __pyx_v_self->_current_header_value = ((PyObject*)__pyx_t_3);
    __pyx_t_3 = 0;
  }
  __pyx_L3:;

  /* "multifruits.pyx":73
 *             self._current_header_name += data
 * 
 *     def on_header_value(self, data):             # <<<<<<<<<<<<<<
 *         if self._current_header_value is None:
 *             self._current_header_value = data
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("multifruits.Parser.on_header_value", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "multifruits.pyx":79
 *             self._current_header_value += data
 * 
 *     def on_headers_complete(self):             # <<<<<<<<<<<<<<
 *         self._maybe_call_on_header()
 *         if self._on_headers_complete is not None:
 */

/* Python wrapper */
static PyObject *__pyx_pw_11multifruits_6Parser_9on_headers_complete(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_11multifruits_6Parser_9on_headers_complete(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("on_headers_complete (wrapper)", 0);
  __pyx_r = __pyx_pf_11multifruits_6Parser_8on_headers_complete(((struct __pyx_obj_11multifruits_Parser *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11multifruits_6Parser_8on_headers_complete(struct __pyx_obj_11multifruits_Parser *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("on_headers_complete", 0);

  /* "multifruits.pyx":80
 * 
 *     def on_headers_complete(self):
 *         self._maybe_call_on_header()             # <<<<<<<<<<<<<<
 *         if self._on_headers_complete is not None:
 *             self._on_headers_complete()
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_maybe_call_on_header); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 80, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 80, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 80, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "multifruits.pyx":81
 *     def on_headers_complete(self):
 *         self._maybe_call_on_header()
 *         if self._on_headers_complete is not None:             # <<<<<<<<<<<<<<
 *             self._on_headers_complete()
 * 
 */
  __pyx_t_4 = (__pyx_v_self->_on_headers_complete != Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "multifruits.pyx":82
 *         self._maybe_call_on_header()
 *         if self._on_headers_complete is not None:
 *             self._on_headers_complete()             # <<<<<<<<<<<<<<
 * 
 *     cdef _feed_data(self, char* data):
 */
    __Pyx_INCREF(__pyx_v_self->_on_headers_complete);
    __pyx_t_2 = __pyx_v_self->_on_headers_complete; __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (__pyx_t_3) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 82, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    } else {
      __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 82, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "multifruits.pyx":81
 *     def on_headers_complete(self):
 *         self._maybe_call_on_header()
 *         if self._on_headers_complete is not None:             # <<<<<<<<<<<<<<
 *             self._on_headers_complete()
 * 
 */
  }

  /* "multifruits.pyx":79
 *             self._current_header_value += data
 * 
 *     def on_headers_complete(self):             # <<<<<<<<<<<<<<
 *         self._maybe_call_on_header()
 *         if self._on_headers_complete is not None:
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("multifruits.Parser.on_headers_complete", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "multifruits.pyx":84
 *             self._on_headers_complete()
 * 
 *     cdef _feed_data(self, char* data):             # <<<<<<<<<<<<<<
 *         cdef:
 *             unsigned int i = 0
 */

static PyObject *__pyx_f_11multifruits_6Parser__feed_data(struct __pyx_obj_11multifruits_Parser *__pyx_v_self, char *__pyx_v_data) {
  unsigned int __pyx_v_i;
  unsigned int __pyx_v_mark;
  char __pyx_v_c;
  unsigned int __pyx_v_length;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  char __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  __Pyx_RefNannySetupContext("_feed_data", 0);

  /* "multifruits.pyx":86
 *     cdef _feed_data(self, char* data):
 *         cdef:
 *             unsigned int i = 0             # <<<<<<<<<<<<<<
 *             unsigned int mark = 0
 *             char c
 */
  __pyx_v_i = 0;

  /* "multifruits.pyx":87
 *         cdef:
 *             unsigned int i = 0
 *             unsigned int mark = 0             # <<<<<<<<<<<<<<
 *             char c
 *             unsigned int length = len(data)
 */
  __pyx_v_mark = 0;

  /* "multifruits.pyx":89
 *             unsigned int mark = 0
 *             char c
 *             unsigned int length = len(data)             # <<<<<<<<<<<<<<
 * 
 *         while i < length:
 */
  __pyx_t_1 = strlen(__pyx_v_data); 
  __pyx_v_length = __pyx_t_1;

  /* "multifruits.pyx":91
 *             unsigned int length = len(data)
 * 
 *         while i < length:             # <<<<<<<<<<<<<<
 *             c = data[i]
 *             if self._state == PREAMBLE:
 */
  while (1) {
    __pyx_t_2 = ((__pyx_v_i < __pyx_v_length) != 0);
    if (!__pyx_t_2) break;

    /* "multifruits.pyx":92
 * 
 *         while i < length:
 *             c = data[i]             # <<<<<<<<<<<<<<
 *             if self._state == PREAMBLE:
 *                 if c == b'-':
 */
    __pyx_v_c = (__pyx_v_data[__pyx_v_i]);

    /* "multifruits.pyx":93
 *         while i < length:
 *             c = data[i]
 *             if self._state == PREAMBLE:             # <<<<<<<<<<<<<<
 *                 if c == b'-':
 *                     self._state = PREAMBLE_HY
 */
    switch (__pyx_v_self->_state) {
      case __pyx_e_11multifruits_PREAMBLE:

      /* "multifruits.pyx":94
 *             c = data[i]
 *             if self._state == PREAMBLE:
 *                 if c == b'-':             # <<<<<<<<<<<<<<
 *                     self._state = PREAMBLE_HY
 *                 i += 1
 */
      __pyx_t_2 = ((__pyx_v_c == '-') != 0);
      if (__pyx_t_2) {

        /* "multifruits.pyx":95
 *             if self._state == PREAMBLE:
 *                 if c == b'-':
 *                     self._state = PREAMBLE_HY             # <<<<<<<<<<<<<<
 *                 i += 1
 *             elif self._state == PREAMBLE_HY:
 */
        __pyx_v_self->_state = __pyx_e_11multifruits_PREAMBLE_HY;

        /* "multifruits.pyx":94
 *             c = data[i]
 *             if self._state == PREAMBLE:
 *                 if c == b'-':             # <<<<<<<<<<<<<<
 *                     self._state = PREAMBLE_HY
 *                 i += 1
 */
      }

      /* "multifruits.pyx":96
 *                 if c == b'-':
 *                     self._state = PREAMBLE_HY
 *                 i += 1             # <<<<<<<<<<<<<<
 *             elif self._state == PREAMBLE_HY:
 *                 if c == b'-':
 */
      __pyx_v_i = (__pyx_v_i + 1);

      /* "multifruits.pyx":93
 *         while i < length:
 *             c = data[i]
 *             if self._state == PREAMBLE:             # <<<<<<<<<<<<<<
 *                 if c == b'-':
 *                     self._state = PREAMBLE_HY
 */
      break;

      /* "multifruits.pyx":97
 *                     self._state = PREAMBLE_HY
 *                 i += 1
 *             elif self._state == PREAMBLE_HY:             # <<<<<<<<<<<<<<
 *                 if c == b'-':
 *                     self._state = FIRST_BOUNDARY
 */
      case __pyx_e_11multifruits_PREAMBLE_HY:

      /* "multifruits.pyx":98
 *                 i += 1
 *             elif self._state == PREAMBLE_HY:
 *                 if c == b'-':             # <<<<<<<<<<<<<<
 *                     self._state = FIRST_BOUNDARY
 *                 else:
 */
      __pyx_t_2 = ((__pyx_v_c == '-') != 0);
      if (__pyx_t_2) {

        /* "multifruits.pyx":99
 *             elif self._state == PREAMBLE_HY:
 *                 if c == b'-':
 *                     self._state = FIRST_BOUNDARY             # <<<<<<<<<<<<<<
 *                 else:
 *                     self._state = PREAMBLE
 */
        __pyx_v_self->_state = __pyx_e_11multifruits_FIRST_BOUNDARY;

        /* "multifruits.pyx":98
 *                 i += 1
 *             elif self._state == PREAMBLE_HY:
 *                 if c == b'-':             # <<<<<<<<<<<<<<
 *                     self._state = FIRST_BOUNDARY
 *                 else:
 */
        goto __pyx_L6;
      }

      /* "multifruits.pyx":101
 *                     self._state = FIRST_BOUNDARY
 *                 else:
 *                     self._state = PREAMBLE             # <<<<<<<<<<<<<<
 *                 i += 1
 *             elif self._state == FIRST_BOUNDARY:
 */
      /*else*/ {
        __pyx_v_self->_state = __pyx_e_11multifruits_PREAMBLE;
      }
      __pyx_L6:;

      /* "multifruits.pyx":102
 *                 else:
 *                     self._state = PREAMBLE
 *                 i += 1             # <<<<<<<<<<<<<<
 *             elif self._state == FIRST_BOUNDARY:
 *                 self._boundary_index = 0
 */
      __pyx_v_i = (__pyx_v_i + 1);

      /* "multifruits.pyx":97
 *                     self._state = PREAMBLE_HY
 *                 i += 1
 *             elif self._state == PREAMBLE_HY:             # <<<<<<<<<<<<<<
 *                 if c == b'-':
 *                     self._state = FIRST_BOUNDARY
 */
      break;

      /* "multifruits.pyx":103
 *                     self._state = PREAMBLE
 *                 i += 1
 *             elif self._state == FIRST_BOUNDARY:             # <<<<<<<<<<<<<<
 *                 self._boundary_index = 0
 *                 while i < length:
 */
      case __pyx_e_11multifruits_FIRST_BOUNDARY:

      /* "multifruits.pyx":104
 *                 i += 1
 *             elif self._state == FIRST_BOUNDARY:
 *                 self._boundary_index = 0             # <<<<<<<<<<<<<<
 *                 while i < length:
 *                     c = data[i]
 */
      __pyx_v_self->_boundary_index = 0;

      /* "multifruits.pyx":105
 *             elif self._state == FIRST_BOUNDARY:
 *                 self._boundary_index = 0
 *                 while i < length:             # <<<<<<<<<<<<<<
 *                     c = data[i]
 *                     if self._boundary_index == self._boundary_length:
 */
      while (1) {
        __pyx_t_2 = ((__pyx_v_i < __pyx_v_length) != 0);
        if (!__pyx_t_2) break;

        /* "multifruits.pyx":106
 *                 self._boundary_index = 0
 *                 while i < length:
 *                     c = data[i]             # <<<<<<<<<<<<<<
 *                     if self._boundary_index == self._boundary_length:
 *                         if c != b'\r':
 */
        __pyx_v_c = (__pyx_v_data[__pyx_v_i]);

        /* "multifruits.pyx":107
 *                 while i < length:
 *                     c = data[i]
 *                     if self._boundary_index == self._boundary_length:             # <<<<<<<<<<<<<<
 *                         if c != b'\r':
 *                             raise ValueError('FIRST_BOUNDARY: \\r')
 */
        __pyx_t_2 = ((__pyx_v_self->_boundary_index == __pyx_v_self->_boundary_length) != 0);
        if (__pyx_t_2) {

          /* "multifruits.pyx":108
 *                     c = data[i]
 *                     if self._boundary_index == self._boundary_length:
 *                         if c != b'\r':             # <<<<<<<<<<<<<<
 *                             raise ValueError('FIRST_BOUNDARY: \\r')
 *                         self._state = FIRST_BOUNDARY_DONE
 */
          __pyx_t_2 = ((__pyx_v_c != '\r') != 0);
          if (__pyx_t_2) {

            /* "multifruits.pyx":109
 *                     if self._boundary_index == self._boundary_length:
 *                         if c != b'\r':
 *                             raise ValueError('FIRST_BOUNDARY: \\r')             # <<<<<<<<<<<<<<
 *                         self._state = FIRST_BOUNDARY_DONE
 *                         i += 1
 */
            __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 109, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_3);
            __Pyx_Raise(__pyx_t_3, 0, 0, 0);
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            __PYX_ERR(0, 109, __pyx_L1_error)

            /* "multifruits.pyx":108
 *                     c = data[i]
 *                     if self._boundary_index == self._boundary_length:
 *                         if c != b'\r':             # <<<<<<<<<<<<<<
 *                             raise ValueError('FIRST_BOUNDARY: \\r')
 *                         self._state = FIRST_BOUNDARY_DONE
 */
          }

          /* "multifruits.pyx":110
 *                         if c != b'\r':
 *                             raise ValueError('FIRST_BOUNDARY: \\r')
 *                         self._state = FIRST_BOUNDARY_DONE             # <<<<<<<<<<<<<<
 *                         i += 1
 *                         break
 */
          __pyx_v_self->_state = __pyx_e_11multifruits_FIRST_BOUNDARY_DONE;

          /* "multifruits.pyx":111
 *                             raise ValueError('FIRST_BOUNDARY: \\r')
 *                         self._state = FIRST_BOUNDARY_DONE
 *                         i += 1             # <<<<<<<<<<<<<<
 *                         break
 *                     elif c == self._boundary[self._boundary_index]:
 */
          __pyx_v_i = (__pyx_v_i + 1);

          /* "multifruits.pyx":112
 *                         self._state = FIRST_BOUNDARY_DONE
 *                         i += 1
 *                         break             # <<<<<<<<<<<<<<
 *                     elif c == self._boundary[self._boundary_index]:
 *                         self._boundary_index += 1
 */
          goto __pyx_L8_break;

          /* "multifruits.pyx":107
 *                 while i < length:
 *                     c = data[i]
 *                     if self._boundary_index == self._boundary_length:             # <<<<<<<<<<<<<<
 *                         if c != b'\r':
 *                             raise ValueError('FIRST_BOUNDARY: \\r')
 */
        }

        /* "multifruits.pyx":113
 *                         i += 1
 *                         break
 *                     elif c == self._boundary[self._boundary_index]:             # <<<<<<<<<<<<<<
 *                         self._boundary_index += 1
 *                     else:
 */
        __pyx_t_3 = __Pyx_PyInt_From_char(__pyx_v_c); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 113, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = __Pyx_GetItemInt(__pyx_v_self->_boundary, __pyx_v_self->_boundary_index, unsigned int, 0, __Pyx_PyInt_From_unsigned_int, 0, 0, 1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 113, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_5 = PyObject_RichCompare(__pyx_t_3, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 113, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 113, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (__pyx_t_2) {

          /* "multifruits.pyx":114
 *                         break
 *                     elif c == self._boundary[self._boundary_index]:
 *                         self._boundary_index += 1             # <<<<<<<<<<<<<<
 *                     else:
 *                         raise ValueError('FIRST_BOUNDARY')
 */
          __pyx_v_self->_boundary_index = (__pyx_v_self->_boundary_index + 1);

          /* "multifruits.pyx":113
 *                         i += 1
 *                         break
 *                     elif c == self._boundary[self._boundary_index]:             # <<<<<<<<<<<<<<
 *                         self._boundary_index += 1
 *                     else:
 */
          goto __pyx_L9;
        }

        /* "multifruits.pyx":116
 *                         self._boundary_index += 1
 *                     else:
 *                         raise ValueError('FIRST_BOUNDARY')             # <<<<<<<<<<<<<<
 *                     i += 1
 *             elif self._state == FIRST_BOUNDARY_DONE:
 */
        /*else*/ {
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 116, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_Raise(__pyx_t_5, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __PYX_ERR(0, 116, __pyx_L1_error)
        }
        __pyx_L9:;

        /* "multifruits.pyx":117
 *                     else:
 *                         raise ValueError('FIRST_BOUNDARY')
 *                     i += 1             # <<<<<<<<<<<<<<
 *             elif self._state == FIRST_BOUNDARY_DONE:
 *                 if c != b'\n':
 */
        __pyx_v_i = (__pyx_v_i + 1);
      }
      __pyx_L8_break:;

      /* "multifruits.pyx":103
 *                     self._state = PREAMBLE
 *                 i += 1
 *             elif self._state == FIRST_BOUNDARY:             # <<<<<<<<<<<<<<
 *                 self._boundary_index = 0
 *                 while i < length:
 */
      break;

      /* "multifruits.pyx":118
 *                         raise ValueError('FIRST_BOUNDARY')
 *                     i += 1
 *             elif self._state == FIRST_BOUNDARY_DONE:             # <<<<<<<<<<<<<<
 *                 if c != b'\n':
 *                     raise ValueError('FIRST_BOUNDARY_DONE')
 */
      case __pyx_e_11multifruits_FIRST_BOUNDARY_DONE:

      /* "multifruits.pyx":119
 *                     i += 1
 *             elif self._state == FIRST_BOUNDARY_DONE:
 *                 if c != b'\n':             # <<<<<<<<<<<<<<
 *                     raise ValueError('FIRST_BOUNDARY_DONE')
 *                 if self._on_body_begin is not None:
 */
      __pyx_t_2 = ((__pyx_v_c != '\n') != 0);
      if (__pyx_t_2) {

        /* "multifruits.pyx":120
 *             elif self._state == FIRST_BOUNDARY_DONE:
 *                 if c != b'\n':
 *                     raise ValueError('FIRST_BOUNDARY_DONE')             # <<<<<<<<<<<<<<
 *                 if self._on_body_begin is not None:
 *                     self._on_body_begin()
 */
        __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 120, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_Raise(__pyx_t_5, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __PYX_ERR(0, 120, __pyx_L1_error)

        /* "multifruits.pyx":119
 *                     i += 1
 *             elif self._state == FIRST_BOUNDARY_DONE:
 *                 if c != b'\n':             # <<<<<<<<<<<<<<
 *                     raise ValueError('FIRST_BOUNDARY_DONE')
 *                 if self._on_body_begin is not None:
 */
      }

      /* "multifruits.pyx":121
 *                 if c != b'\n':
 *                     raise ValueError('FIRST_BOUNDARY_DONE')
 *                 if self._on_body_begin is not None:             # <<<<<<<<<<<<<<
 *                     self._on_body_begin()
 *                 if self._on_part_begin is not None:
 */
      __pyx_t_2 = (__pyx_v_self->_on_body_begin != Py_None);
      __pyx_t_6 = (__pyx_t_2 != 0);
      if (__pyx_t_6) {

        /* "multifruits.pyx":122
 *                     raise ValueError('FIRST_BOUNDARY_DONE')
 *                 if self._on_body_begin is not None:
 *                     self._on_body_begin()             # <<<<<<<<<<<<<<
 *                 if self._on_part_begin is not None:
 *                     self._on_part_begin()
 */
        __Pyx_INCREF(__pyx_v_self->_on_body_begin);
        __pyx_t_4 = __pyx_v_self->_on_body_begin; __pyx_t_3 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_3)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_3);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
          }
        }
        if (__pyx_t_3) {
          __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 122, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        } else {
          __pyx_t_5 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 122, __pyx_L1_error)
        }
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

        /* "multifruits.pyx":121
 *                 if c != b'\n':
 *                     raise ValueError('FIRST_BOUNDARY_DONE')
 *                 if self._on_body_begin is not None:             # <<<<<<<<<<<<<<
 *                     self._on_body_begin()
 *                 if self._on_part_begin is not None:
 */
      }

      /* "multifruits.pyx":123
 *                 if self._on_body_begin is not None:
 *                     self._on_body_begin()
 *                 if self._on_part_begin is not None:             # <<<<<<<<<<<<<<
 *                     self._on_part_begin()
 *                 self._state = HEADER_NAME_START
 */
      __pyx_t_6 = (__pyx_v_self->_on_part_begin != Py_None);
      __pyx_t_2 = (__pyx_t_6 != 0);
      if (__pyx_t_2) {

        /* "multifruits.pyx":124
 *                     self._on_body_begin()
 *                 if self._on_part_begin is not None:
 *                     self._on_part_begin()             # <<<<<<<<<<<<<<
 *                 self._state = HEADER_NAME_START
 *                 i += 1
 */
        __Pyx_INCREF(__pyx_v_self->_on_part_begin);
        __pyx_t_4 = __pyx_v_self->_on_part_begin; __pyx_t_3 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_3)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_3);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
          }
        }
        if (__pyx_t_3) {
          __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 124, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        } else {
          __pyx_t_5 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 124, __pyx_L1_error)
        }
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

        /* "multifruits.pyx":123
 *                 if self._on_body_begin is not None:
 *                     self._on_body_begin()
 *                 if self._on_part_begin is not None:             # <<<<<<<<<<<<<<
 *                     self._on_part_begin()
 *                 self._state = HEADER_NAME_START
 */
      }

      /* "multifruits.pyx":125
 *                 if self._on_part_begin is not None:
 *                     self._on_part_begin()
 *                 self._state = HEADER_NAME_START             # <<<<<<<<<<<<<<
 *                 i += 1
 *             elif self._state == HEADER_NAME_START:
 */
      __pyx_v_self->_state = __pyx_e_11multifruits_HEADER_NAME_START;

      /* "multifruits.pyx":126
 *                     self._on_part_begin()
 *                 self._state = HEADER_NAME_START
 *                 i += 1             # <<<<<<<<<<<<<<
 *             elif self._state == HEADER_NAME_START:
 *                 if c == b'\r':
 */
      __pyx_v_i = (__pyx_v_i + 1);

      /* "multifruits.pyx":118
 *                         raise ValueError('FIRST_BOUNDARY')
 *                     i += 1
 *             elif self._state == FIRST_BOUNDARY_DONE:             # <<<<<<<<<<<<<<
 *                 if c != b'\n':
 *                     raise ValueError('FIRST_BOUNDARY_DONE')
 */
      break;

      /* "multifruits.pyx":127
 *                 self._state = HEADER_NAME_START
 *                 i += 1
 *             elif self._state == HEADER_NAME_START:             # <<<<<<<<<<<<<<
 *                 if c == b'\r':
 *                     self._state = HEADERS_DONE
 */
      case __pyx_e_11multifruits_HEADER_NAME_START:

      /* "multifruits.pyx":128
 *                 i += 1
 *             elif self._state == HEADER_NAME_START:
 *                 if c == b'\r':             # <<<<<<<<<<<<<<
 *                     self._state = HEADERS_DONE
 *                     i += 1
 */
      __pyx_t_2 = ((__pyx_v_c == '\r') != 0);
      if (__pyx_t_2) {

        /* "multifruits.pyx":129
 *             elif self._state == HEADER_NAME_START:
 *                 if c == b'\r':
 *                     self._state = HEADERS_DONE             # <<<<<<<<<<<<<<
 *                     i += 1
 *                 else:
 */
        __pyx_v_self->_state = __pyx_e_11multifruits_HEADERS_DONE;

        /* "multifruits.pyx":130
 *                 if c == b'\r':
 *                     self._state = HEADERS_DONE
 *                     i += 1             # <<<<<<<<<<<<<<
 *                 else:
 *                     self._state = HEADER_NAME
 */
        __pyx_v_i = (__pyx_v_i + 1);

        /* "multifruits.pyx":128
 *                 i += 1
 *             elif self._state == HEADER_NAME_START:
 *                 if c == b'\r':             # <<<<<<<<<<<<<<
 *                     self._state = HEADERS_DONE
 *                     i += 1
 */
        goto __pyx_L14;
      }

      /* "multifruits.pyx":132
 *                     i += 1
 *                 else:
 *                     self._state = HEADER_NAME             # <<<<<<<<<<<<<<
 *             elif self._state == HEADER_NAME:
 *                 mark = i
 */
      /*else*/ {
        __pyx_v_self->_state = __pyx_e_11multifruits_HEADER_NAME;
      }
      __pyx_L14:;

      /* "multifruits.pyx":127
 *                 self._state = HEADER_NAME_START
 *                 i += 1
 *             elif self._state == HEADER_NAME_START:             # <<<<<<<<<<<<<<
 *                 if c == b'\r':
 *                     self._state = HEADERS_DONE
 */
      break;

      /* "multifruits.pyx":133
 *                 else:
 *                     self._state = HEADER_NAME
 *             elif self._state == HEADER_NAME:             # <<<<<<<<<<<<<<
 *                 mark = i
 *                 while i < length:
 */
      case __pyx_e_11multifruits_HEADER_NAME:

      /* "multifruits.pyx":134
 *                     self._state = HEADER_NAME
 *             elif self._state == HEADER_NAME:
 *                 mark = i             # <<<<<<<<<<<<<<
 *                 while i < length:
 *                     c = data[i]
 */
      __pyx_v_mark = __pyx_v_i;

      /* "multifruits.pyx":135
 *             elif self._state == HEADER_NAME:
 *                 mark = i
 *                 while i < length:             # <<<<<<<<<<<<<<
 *                     c = data[i]
 *                     if c == b':':
 */
      while (1) {
        __pyx_t_2 = ((__pyx_v_i < __pyx_v_length) != 0);
        if (!__pyx_t_2) break;

        /* "multifruits.pyx":136
 *                 mark = i
 *                 while i < length:
 *                     c = data[i]             # <<<<<<<<<<<<<<
 *                     if c == b':':
 *                         self._state = HEADER_VALUE_START
 */
        __pyx_v_c = (__pyx_v_data[__pyx_v_i]);

        /* "multifruits.pyx":137
 *                 while i < length:
 *                     c = data[i]
 *                     if c == b':':             # <<<<<<<<<<<<<<
 *                         self._state = HEADER_VALUE_START
 *                         break
 */
        __pyx_t_2 = ((__pyx_v_c == ':') != 0);
        if (__pyx_t_2) {

          /* "multifruits.pyx":138
 *                     c = data[i]
 *                     if c == b':':
 *                         self._state = HEADER_VALUE_START             # <<<<<<<<<<<<<<
 *                         break
 *                     # TODO: try with a dict access when benchmarked.
 */
          __pyx_v_self->_state = __pyx_e_11multifruits_HEADER_VALUE_START;

          /* "multifruits.pyx":139
 *                     if c == b':':
 *                         self._state = HEADER_VALUE_START
 *                         break             # <<<<<<<<<<<<<<
 *                     # TODO: try with a dict access when benchmarked.
 *                     elif c in (b'(', b')', b'<', b'>', b'@', b',', b';',
 */
          goto __pyx_L16_break;

          /* "multifruits.pyx":137
 *                 while i < length:
 *                     c = data[i]
 *                     if c == b':':             # <<<<<<<<<<<<<<
 *                         self._state = HEADER_VALUE_START
 *                         break
 */
        }

        /* "multifruits.pyx":141
 *                         break
 *                     # TODO: try with a dict access when benchmarked.
 *                     elif c in (b'(', b')', b'<', b'>', b'@', b',', b';',             # <<<<<<<<<<<<<<
 *                                b'\\', b'"', b'/', b'[', b']', b'?', b'=', b'{',
 *                                b'}', b' ', b'\t'):
 */
        __pyx_t_7 = __pyx_v_c;
        __pyx_t_6 = ((__pyx_t_7 == '(') != 0);
        if (!__pyx_t_6) {
        } else {
          __pyx_t_2 = __pyx_t_6;
          goto __pyx_L18_bool_binop_done;
        }
        __pyx_t_6 = ((__pyx_t_7 == ')') != 0);
        if (!__pyx_t_6) {
        } else {
          __pyx_t_2 = __pyx_t_6;
          goto __pyx_L18_bool_binop_done;
        }
        __pyx_t_6 = ((__pyx_t_7 == '<') != 0);
        if (!__pyx_t_6) {
        } else {
          __pyx_t_2 = __pyx_t_6;
          goto __pyx_L18_bool_binop_done;
        }
        __pyx_t_6 = ((__pyx_t_7 == '>') != 0);
        if (!__pyx_t_6) {
        } else {
          __pyx_t_2 = __pyx_t_6;
          goto __pyx_L18_bool_binop_done;
        }
        __pyx_t_6 = ((__pyx_t_7 == '@') != 0);
        if (!__pyx_t_6) {
        } else {
          __pyx_t_2 = __pyx_t_6;
          goto __pyx_L18_bool_binop_done;
        }
        __pyx_t_6 = ((__pyx_t_7 == ',') != 0);
        if (!__pyx_t_6) {
        } else {
          __pyx_t_2 = __pyx_t_6;
          goto __pyx_L18_bool_binop_done;
        }
        __pyx_t_6 = ((__pyx_t_7 == ';') != 0);
        if (!__pyx_t_6) {
        } else {
          __pyx_t_2 = __pyx_t_6;
          goto __pyx_L18_bool_binop_done;
        }

        /* "multifruits.pyx":142
 *                     # TODO: try with a dict access when benchmarked.
 *                     elif c in (b'(', b')', b'<', b'>', b'@', b',', b';',
 *                                b'\\', b'"', b'/', b'[', b']', b'?', b'=', b'{',             # <<<<<<<<<<<<<<
 *                                b'}', b' ', b'\t'):
 *                         raise ValueError('HEADER_NAME')
 */
        __pyx_t_6 = ((__pyx_t_7 == '\\') != 0);
        if (!__pyx_t_6) {
        } else {
          __pyx_t_2 = __pyx_t_6;
          goto __pyx_L18_bool_binop_done;
        }

        /* "multifruits.pyx":141
 *                         break
 *                     # TODO: try with a dict access when benchmarked.
 *                     elif c in (b'(', b')', b'<', b'>', b'@', b',', b';',             # <<<<<<<<<<<<<<
 *                                b'\\', b'"', b'/', b'[', b']', b'?', b'=', b'{',
 *                                b'}', b' ', b'\t'):
 */
        __pyx_t_6 = ((__pyx_t_7 == '"') != 0);
        if (!__pyx_t_6) {
        } else {
          __pyx_t_2 = __pyx_t_6;
          goto __pyx_L18_bool_binop_done;
        }

        /* "multifruits.pyx":142
 *                     # TODO: try with a dict access when benchmarked.
 *                     elif c in (b'(', b')', b'<', b'>', b'@', b',', b';',
 *                                b'\\', b'"', b'/', b'[', b']', b'?', b'=', b'{',             # <<<<<<<<<<<<<<
 *                                b'}', b' ', b'\t'):
 *                         raise ValueError('HEADER_NAME')
 */
        __pyx_t_6 = ((__pyx_t_7 == '/') != 0);
        if (!__pyx_t_6) {
        } else {
          __pyx_t_2 = __pyx_t_6;
          goto __pyx_L18_bool_binop_done;
        }

        /* "multifruits.pyx":141
 *                         break
 *                     # TODO: try with a dict access when benchmarked.
 *                     elif c in (b'(', b')', b'<', b'>', b'@', b',', b';',             # <<<<<<<<<<<<<<
 *                                b'\\', b'"', b'/', b'[', b']', b'?', b'=', b'{',
 *                                b'}', b' ', b'\t'):
 */
        __pyx_t_6 = ((__pyx_t_7 == '[') != 0);
        if (!__pyx_t_6) {
        } else {
          __pyx_t_2 = __pyx_t_6;
          goto __pyx_L18_bool_binop_done;
        }

        /* "multifruits.pyx":142
 *                     # TODO: try with a dict access when benchmarked.
 *                     elif c in (b'(', b')', b'<', b'>', b'@', b',', b';',
 *                                b'\\', b'"', b'/', b'[', b']', b'?', b'=', b'{',             # <<<<<<<<<<<<<<
 *                                b'}', b' ', b'\t'):
 *                         raise ValueError('HEADER_NAME')
 */
        __pyx_t_6 = ((__pyx_t_7 == ']') != 0);
        if (!__pyx_t_6) {
        } else {
          __pyx_t_2 = __pyx_t_6;
          goto __pyx_L18_bool_binop_done;
        }

        /* "multifruits.pyx":141
 *                         break
 *                     # TODO: try with a dict access when benchmarked.
 *                     elif c in (b'(', b')', b'<', b'>', b'@', b',', b';',             # <<<<<<<<<<<<<<
 *                                b'\\', b'"', b'/', b'[', b']', b'?', b'=', b'{',
 *                                b'}', b' ', b'\t'):
 */
        __pyx_t_6 = ((__pyx_t_7 == '?') != 0);
        if (!__pyx_t_6) {
        } else {
          __pyx_t_2 = __pyx_t_6;
          goto __pyx_L18_bool_binop_done;
        }

        /* "multifruits.pyx":142
 *                     # TODO: try with a dict access when benchmarked.
 *                     elif c in (b'(', b')', b'<', b'>', b'@', b',', b';',
 *                                b'\\', b'"', b'/', b'[', b']', b'?', b'=', b'{',             # <<<<<<<<<<<<<<
 *                                b'}', b' ', b'\t'):
 *                         raise ValueError('HEADER_NAME')
 */
        __pyx_t_6 = ((__pyx_t_7 == '=') != 0);
        if (!__pyx_t_6) {
        } else {
          __pyx_t_2 = __pyx_t_6;
          goto __pyx_L18_bool_binop_done;
        }

        /* "multifruits.pyx":141
 *                         break
 *                     # TODO: try with a dict access when benchmarked.
 *                     elif c in (b'(', b')', b'<', b'>', b'@', b',', b';',             # <<<<<<<<<<<<<<
 *                                b'\\', b'"', b'/', b'[', b']', b'?', b'=', b'{',
 *                                b'}', b' ', b'\t'):
 */
        __pyx_t_6 = ((__pyx_t_7 == '{') != 0);
          if (!__pyx_t_6) {
          } else {
            __pyx_t_2 = __pyx_t_6;
            goto __pyx_L18_bool_binop_done;
          }

          /* "multifruits.pyx":143
 *                     elif c in (b'(', b')', b'<', b'>', b'@', b',', b';',
 *                                b'\\', b'"', b'/', b'[', b']', b'?', b'=', b'{',
 *                                b'}', b' ', b'\t'):             # <<<<<<<<<<<<<<
 *                         raise ValueError('HEADER_NAME')
 *                     i += 1
 */
        __pyx_t_6 = ((__pyx_t_7 == '}') != 0);
        if (!__pyx_t_6) {
        } else {
          __pyx_t_2 = __pyx_t_6;
          goto __pyx_L18_bool_binop_done;
        }

        /* "multifruits.pyx":141
 *                         break
 *                     # TODO: try with a dict access when benchmarked.
 *                     elif c in (b'(', b')', b'<', b'>', b'@', b',', b';',             # <<<<<<<<<<<<<<
 *                                b'\\', b'"', b'/', b'[', b']', b'?', b'=', b'{',
 *                                b'}', b' ', b'\t'):
 */
        __pyx_t_6 = ((__pyx_t_7 == ' ') != 0);
        if (!__pyx_t_6) {
        } else {
          __pyx_t_2 = __pyx_t_6;
          goto __pyx_L18_bool_binop_done;
        }

        /* "multifruits.pyx":143
 *                     elif c in (b'(', b')', b'<', b'>', b'@', b',', b';',
 *                                b'\\', b'"', b'/', b'[', b']', b'?', b'=', b'{',
 *                                b'}', b' ', b'\t'):             # <<<<<<<<<<<<<<
 *                         raise ValueError('HEADER_NAME')
 *                     i += 1
 */
        __pyx_t_6 = ((__pyx_t_7 == '\t') != 0);
        __pyx_t_2 = __pyx_t_6;
        __pyx_L18_bool_binop_done:;

        /* "multifruits.pyx":141
 *                         break
 *                     # TODO: try with a dict access when benchmarked.
 *                     elif c in (b'(', b')', b'<', b'>', b'@', b',', b';',             # <<<<<<<<<<<<<<
 *                                b'\\', b'"', b'/', b'[', b']', b'?', b'=', b'{',
 *                                b'}', b' ', b'\t'):
 */
        __pyx_t_6 = (__pyx_t_2 != 0);
        if (__pyx_t_6) {

          /* "multifruits.pyx":144
 *                                b'\\', b'"', b'/', b'[', b']', b'?', b'=', b'{',
 *                                b'}', b' ', b'\t'):
 *                         raise ValueError('HEADER_NAME')             # <<<<<<<<<<<<<<
 *                     i += 1
 *                 if i > mark:
 */
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 144, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_Raise(__pyx_t_5, 0, 0, 0);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __PYX_ERR(0, 144, __pyx_L1_error)

          /* "multifruits.pyx":141
 *                         break
 *                     # TODO: try with a dict access when benchmarked.
 *                     elif c in (b'(', b')', b'<', b'>', b'@', b',', b';',             # <<<<<<<<<<<<<<
 *                                b'\\', b'"', b'/', b'[', b']', b'?', b'=', b'{',
 *                                b'}', b' ', b'\t'):
 */
        }

        /* "multifruits.pyx":145
 *                                b'}', b' ', b'\t'):
 *                         raise ValueError('HEADER_NAME')
 *                     i += 1             # <<<<<<<<<<<<<<
 *                 if i > mark:
 *                     self.on_header_name(data[mark:i])
 */
        __pyx_v_i = (__pyx_v_i + 1);
      }
      __pyx_L16_break:;

      /* "multifruits.pyx":146
 *                         raise ValueError('HEADER_NAME')
 *                     i += 1
 *                 if i > mark:             # <<<<<<<<<<<<<<
 *                     self.on_header_name(data[mark:i])
 *                 i += 1
 */
      __pyx_t_6 = ((__pyx_v_i > __pyx_v_mark) != 0);
      if (__pyx_t_6) {

        /* "multifruits.pyx":147
 *                     i += 1
 *                 if i > mark:
 *                     self.on_header_name(data[mark:i])             # <<<<<<<<<<<<<<
 *                 i += 1
 *             elif self._state == HEADER_VALUE_START:
 */
        __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_on_header_name); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 147, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_3 = __Pyx_PyBytes_FromStringAndSize(__pyx_v_data + __pyx_v_mark, __pyx_v_i - __pyx_v_mark); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 147, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_8 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_8)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_8);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
          }
        }
        if (!__pyx_t_8) {
          __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 147, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_GOTREF(__pyx_t_5);
        } else {
          #if CYTHON_FAST_PYCALL
          if (PyFunction_Check(__pyx_t_4)) {
            PyObject *__pyx_temp[2] = {__pyx_t_8, __pyx_t_3};
            __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 147, __pyx_L1_error)
            __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
            __Pyx_GOTREF(__pyx_t_5);
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          } else
          #endif
          #if CYTHON_FAST_PYCCALL
          if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
            PyObject *__pyx_temp[2] = {__pyx_t_8, __pyx_t_3};
            __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 147, __pyx_L1_error)
            __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
            __Pyx_GOTREF(__pyx_t_5);
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          } else
          #endif
          {
            __pyx_t_9 = PyTuple_New(1+1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 147, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_9);
            __Pyx_GIVEREF(__pyx_t_8); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_8); __pyx_t_8 = NULL;
            __Pyx_GIVEREF(__pyx_t_3);
            PyTuple_SET_ITEM(__pyx_t_9, 0+1, __pyx_t_3);
            __pyx_t_3 = 0;
            __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_9, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 147, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_5);
            __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          }
        }
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

        /* "multifruits.pyx":146
 *                         raise ValueError('HEADER_NAME')
 *                     i += 1
 *                 if i > mark:             # <<<<<<<<<<<<<<
 *                     self.on_header_name(data[mark:i])
 *                 i += 1
 */
      }

      /* "multifruits.pyx":148
 *                 if i > mark:
 *                     self.on_header_name(data[mark:i])
 *                 i += 1             # <<<<<<<<<<<<<<
 *             elif self._state == HEADER_VALUE_START:
 *                 if c != b' ' and c != b'\t':
 */
      __pyx_v_i = (__pyx_v_i + 1);

      /* "multifruits.pyx":133
 *                 else:
 *                     self._state = HEADER_NAME
 *             elif self._state == HEADER_NAME:             # <<<<<<<<<<<<<<
 *                 mark = i
 *                 while i < length:
 */
      break;

      /* "multifruits.pyx":149
 *                     self.on_header_name(data[mark:i])
 *                 i += 1
 *             elif self._state == HEADER_VALUE_START:             # <<<<<<<<<<<<<<
 *                 if c != b' ' and c != b'\t':
 *                     self._state = HEADER_VALUE
 */
      case __pyx_e_11multifruits_HEADER_VALUE_START:

      /* "multifruits.pyx":150
 *                 i += 1
 *             elif self._state == HEADER_VALUE_START:
 *                 if c != b' ' and c != b'\t':             # <<<<<<<<<<<<<<
 *                     self._state = HEADER_VALUE
 *                 else:
 */
      __pyx_t_2 = ((__pyx_v_c != ' ') != 0);
      if (__pyx_t_2) {
      } else {
        __pyx_t_6 = __pyx_t_2;
        goto __pyx_L38_bool_binop_done;
      }
      __pyx_t_2 = ((__pyx_v_c != '\t') != 0);
      __pyx_t_6 = __pyx_t_2;
      __pyx_L38_bool_binop_done:;
      if (__pyx_t_6) {

        /* "multifruits.pyx":151
 *             elif self._state == HEADER_VALUE_START:
 *                 if c != b' ' and c != b'\t':
 *                     self._state = HEADER_VALUE             # <<<<<<<<<<<<<<
 *                 else:
 *                     i += 1
 */
        __pyx_v_self->_state = __pyx_e_11multifruits_HEADER_VALUE;

        /* "multifruits.pyx":150
 *                 i += 1
 *             elif self._state == HEADER_VALUE_START:
 *                 if c != b' ' and c != b'\t':             # <<<<<<<<<<<<<<
 *                     self._state = HEADER_VALUE
 *                 else:
 */
        goto __pyx_L37;
      }

      /* "multifruits.pyx":153
 *                     self._state = HEADER_VALUE
 *                 else:
 *                     i += 1             # <<<<<<<<<<<<<<
 *             elif self._state == HEADER_VALUE:
 *                 mark = i
 */
      /*else*/ {
        __pyx_v_i = (__pyx_v_i + 1);
      }
      __pyx_L37:;

      /* "multifruits.pyx":149
 *                     self.on_header_name(data[mark:i])
 *                 i += 1
 *             elif self._state == HEADER_VALUE_START:             # <<<<<<<<<<<<<<
 *                 if c != b' ' and c != b'\t':
 *                     self._state = HEADER_VALUE
 */
      break;

      /* "multifruits.pyx":154
 *                 else:
 *                     i += 1
 *             elif self._state == HEADER_VALUE:             # <<<<<<<<<<<<<<
 *                 mark = i
 *                 while i < length:
 */
      case __pyx_e_11multifruits_HEADER_VALUE:

      /* "multifruits.pyx":155
 *                     i += 1
 *             elif self._state == HEADER_VALUE:
 *                 mark = i             # <<<<<<<<<<<<<<
 *                 while i < length:
 *                     c = data[i]
 */
      __pyx_v_mark = __pyx_v_i;

      /* "multifruits.pyx":156
 *             elif self._state == HEADER_VALUE:
 *                 mark = i
 *                 while i < length:             # <<<<<<<<<<<<<<
 *                     c = data[i]
 *                     if c == b'\r':
 */
      while (1) {
        __pyx_t_6 = ((__pyx_v_i < __pyx_v_length) != 0);
        if (!__pyx_t_6) break;

        /* "multifruits.pyx":157
 *                 mark = i
 *                 while i < length:
 *                     c = data[i]             # <<<<<<<<<<<<<<
 *                     if c == b'\r':
 *                         self._state = HEADER_VALUE_CR
 */
        __pyx_v_c = (__pyx_v_data[__pyx_v_i]);

        /* "multifruits.pyx":158
 *                 while i < length:
 *                     c = data[i]
 *                     if c == b'\r':             # <<<<<<<<<<<<<<
 *                         self._state = HEADER_VALUE_CR
 *                         break
 */
        __pyx_t_6 = ((__pyx_v_c == '\r') != 0);
        if (__pyx_t_6) {

          /* "multifruits.pyx":159
 *                     c = data[i]
 *                     if c == b'\r':
 *                         self._state = HEADER_VALUE_CR             # <<<<<<<<<<<<<<
 *                         break
 *                     i += 1
 */
          __pyx_v_self->_state = __pyx_e_11multifruits_HEADER_VALUE_CR;

          /* "multifruits.pyx":160
 *                     if c == b'\r':
 *                         self._state = HEADER_VALUE_CR
 *                         break             # <<<<<<<<<<<<<<
 *                     i += 1
 *                 if i > mark:
 */
          goto __pyx_L41_break;

          /* "multifruits.pyx":158
 *                 while i < length:
 *                     c = data[i]
 *                     if c == b'\r':             # <<<<<<<<<<<<<<
 *                         self._state = HEADER_VALUE_CR
 *                         break
 */
        }

        /* "multifruits.pyx":161
 *                         self._state = HEADER_VALUE_CR
 *                         break
 *                     i += 1             # <<<<<<<<<<<<<<
 *                 if i > mark:
 *                     self.on_header_value(data[mark:i])
 */
        __pyx_v_i = (__pyx_v_i + 1);
      }
      __pyx_L41_break:;

      /* "multifruits.pyx":162
 *                         break
 *                     i += 1
 *                 if i > mark:             # <<<<<<<<<<<<<<
 *                     self.on_header_value(data[mark:i])
 *                 i += 1
 */
      __pyx_t_6 = ((__pyx_v_i > __pyx_v_mark) != 0);
      if (__pyx_t_6) {

        /* "multifruits.pyx":163
 *                     i += 1
 *                 if i > mark:
 *                     self.on_header_value(data[mark:i])             # <<<<<<<<<<<<<<
 *                 i += 1
 *             elif self._state == HEADER_VALUE_CR:
 */
        __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_on_header_value); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 163, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __pyx_t_9 = __Pyx_PyBytes_FromStringAndSize(__pyx_v_data + __pyx_v_mark, __pyx_v_i - __pyx_v_mark); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 163, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_9);
        __pyx_t_3 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_3)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_3);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
          }
        }
        if (!__pyx_t_3) {
          __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_9); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 163, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          __Pyx_GOTREF(__pyx_t_5);
        } else {
          #if CYTHON_FAST_PYCALL
          if (PyFunction_Check(__pyx_t_4)) {
            PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_t_9};
            __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 163, __pyx_L1_error)
            __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
            __Pyx_GOTREF(__pyx_t_5);
            __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          } else
          #endif
          #if CYTHON_FAST_PYCCALL
          if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
            PyObject *__pyx_temp[2] = {__pyx_t_3, __pyx_t_9};
            __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 163, __pyx_L1_error)
            __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
            __Pyx_GOTREF(__pyx_t_5);
            __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          } else
          #endif
          {
            __pyx_t_8 = PyTuple_New(1+1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 163, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_8);
            __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_3); __pyx_t_3 = NULL;
            __Pyx_GIVEREF(__pyx_t_9);
            PyTuple_SET_ITEM(__pyx_t_8, 0+1, __pyx_t_9);
            __pyx_t_9 = 0;
            __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_8, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 163, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_5);
            __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          }
        }
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

        /* "multifruits.pyx":162
 *                         break
 *                     i += 1
 *                 if i > mark:             # <<<<<<<<<<<<<<
 *                     self.on_header_value(data[mark:i])
 *                 i += 1
 */
      }

      /* "multifruits.pyx":164
 *                 if i > mark:
 *                     self.on_header_value(data[mark:i])
 *                 i += 1             # <<<<<<<<<<<<<<
 *             elif self._state == HEADER_VALUE_CR:
 *                 if c != b'\n':
 */
      __pyx_v_i = (__pyx_v_i + 1);

      /* "multifruits.pyx":154
 *                 else:
 *                     i += 1
 *             elif self._state == HEADER_VALUE:             # <<<<<<<<<<<<<<
 *                 mark = i
 *                 while i < length:
 */
      break;

      /* "multifruits.pyx":165
 *                     self.on_header_value(data[mark:i])
 *                 i += 1
 *             elif self._state == HEADER_VALUE_CR:             # <<<<<<<<<<<<<<
 *                 if c != b'\n':
 *                     raise ValueError('HEADER_VALUE_CR')
 */
      case __pyx_e_11multifruits_HEADER_VALUE_CR:

      /* "multifruits.pyx":166
 *                 i += 1
 *             elif self._state == HEADER_VALUE_CR:
 *                 if c != b'\n':             # <<<<<<<<<<<<<<
 *                     raise ValueError('HEADER_VALUE_CR')
 *                 self._state = HEADER_NAME_START
 */
      __pyx_t_6 = ((__pyx_v_c != '\n') != 0);
      if (__pyx_t_6) {

        /* "multifruits.pyx":167
 *             elif self._state == HEADER_VALUE_CR:
 *                 if c != b'\n':
 *                     raise ValueError('HEADER_VALUE_CR')             # <<<<<<<<<<<<<<
 *                 self._state = HEADER_NAME_START
 *                 i += 1
 */
        __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 167, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_Raise(__pyx_t_5, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __PYX_ERR(0, 167, __pyx_L1_error)

        /* "multifruits.pyx":166
 *                 i += 1
 *             elif self._state == HEADER_VALUE_CR:
 *                 if c != b'\n':             # <<<<<<<<<<<<<<
 *                     raise ValueError('HEADER_VALUE_CR')
 *                 self._state = HEADER_NAME_START
 */
      }

      /* "multifruits.pyx":168
 *                 if c != b'\n':
 *                     raise ValueError('HEADER_VALUE_CR')
 *                 self._state = HEADER_NAME_START             # <<<<<<<<<<<<<<
 *                 i += 1
 *             elif self._state == HEADERS_DONE:
 */
      __pyx_v_self->_state = __pyx_e_11multifruits_HEADER_NAME_START;

      /* "multifruits.pyx":169
 *                     raise ValueError('HEADER_VALUE_CR')
 *                 self._state = HEADER_NAME_START
 *                 i += 1             # <<<<<<<<<<<<<<
 *             elif self._state == HEADERS_DONE:
 *                 if c != b'\n':
 */
      __pyx_v_i = (__pyx_v_i + 1);

      /* "multifruits.pyx":165
 *                     self.on_header_value(data[mark:i])
 *                 i += 1
 *             elif self._state == HEADER_VALUE_CR:             # <<<<<<<<<<<<<<
 *                 if c != b'\n':
 *                     raise ValueError('HEADER_VALUE_CR')
 */
      break;

      /* "multifruits.pyx":170
 *                 self._state = HEADER_NAME_START
 *                 i += 1
 *             elif self._state == HEADERS_DONE:             # <<<<<<<<<<<<<<
 *                 if c != b'\n':
 *                     raise ValueError('HEADERS_DONE')
 */
      case __pyx_e_11multifruits_HEADERS_DONE:

      /* "multifruits.pyx":171
 *                 i += 1
 *             elif self._state == HEADERS_DONE:
 *                 if c != b'\n':             # <<<<<<<<<<<<<<
 *                     raise ValueError('HEADERS_DONE')
 *                 self.on_headers_complete()
 */
      __pyx_t_6 = ((__pyx_v_c != '\n') != 0);
      if (__pyx_t_6) {

        /* "multifruits.pyx":172
 *             elif self._state == HEADERS_DONE:
 *                 if c != b'\n':
 *                     raise ValueError('HEADERS_DONE')             # <<<<<<<<<<<<<<
 *                 self.on_headers_complete()
 *                 self._state = DATA
 */
        __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__7, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 172, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_Raise(__pyx_t_5, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __PYX_ERR(0, 172, __pyx_L1_error)

        /* "multifruits.pyx":171
 *                 i += 1
 *             elif self._state == HEADERS_DONE:
 *                 if c != b'\n':             # <<<<<<<<<<<<<<
 *                     raise ValueError('HEADERS_DONE')
 *                 self.on_headers_complete()
 */
      }

      /* "multifruits.pyx":173
 *                 if c != b'\n':
 *                     raise ValueError('HEADERS_DONE')
 *                 self.on_headers_complete()             # <<<<<<<<<<<<<<
 *                 self._state = DATA
 *                 i += 1
 */
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_on_headers_complete); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 173, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_8 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
        __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_4);
        if (likely(__pyx_t_8)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_8);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_4, function);
        }
      }
      if (__pyx_t_8) {
        __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_8); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 173, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      } else {
        __pyx_t_5 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 173, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "multifruits.pyx":174
 *                     raise ValueError('HEADERS_DONE')
 *                 self.on_headers_complete()
 *                 self._state = DATA             # <<<<<<<<<<<<<<
 *                 i += 1
 *             elif self._state == DATA:
 */
      __pyx_v_self->_state = __pyx_e_11multifruits_DATA;

      /* "multifruits.pyx":175
 *                 self.on_headers_complete()
 *                 self._state = DATA
 *                 i += 1             # <<<<<<<<<<<<<<
 *             elif self._state == DATA:
 *                 mark = i
 */
      __pyx_v_i = (__pyx_v_i + 1);

      /* "multifruits.pyx":170
 *                 self._state = HEADER_NAME_START
 *                 i += 1
 *             elif self._state == HEADERS_DONE:             # <<<<<<<<<<<<<<
 *                 if c != b'\n':
 *                     raise ValueError('HEADERS_DONE')
 */
      break;

      /* "multifruits.pyx":176
 *                 self._state = DATA
 *                 i += 1
 *             elif self._state == DATA:             # <<<<<<<<<<<<<<
 *                 mark = i
 *                 while i < length:
 */
      case __pyx_e_11multifruits_DATA:

      /* "multifruits.pyx":177
 *                 i += 1
 *             elif self._state == DATA:
 *                 mark = i             # <<<<<<<<<<<<<<
 *                 while i < length:
 *                     c = data[i]
 */
      __pyx_v_mark = __pyx_v_i;

      /* "multifruits.pyx":178
 *             elif self._state == DATA:
 *                 mark = i
 *                 while i < length:             # <<<<<<<<<<<<<<
 *                     c = data[i]
 *                     if c == b'\r':
 */
      while (1) {
        __pyx_t_6 = ((__pyx_v_i < __pyx_v_length) != 0);
        if (!__pyx_t_6) break;

        /* "multifruits.pyx":179
 *                 mark = i
 *                 while i < length:
 *                     c = data[i]             # <<<<<<<<<<<<<<
 *                     if c == b'\r':
 *                         self._state = DATA_CR
 */
        __pyx_v_c = (__pyx_v_data[__pyx_v_i]);

        /* "multifruits.pyx":180
 *                 while i < length:
 *                     c = data[i]
 *                     if c == b'\r':             # <<<<<<<<<<<<<<
 *                         self._state = DATA_CR
 *                         break
 */
        __pyx_t_6 = ((__pyx_v_c == '\r') != 0);
        if (__pyx_t_6) {

          /* "multifruits.pyx":181
 *                     c = data[i]
 *                     if c == b'\r':
 *                         self._state = DATA_CR             # <<<<<<<<<<<<<<
 *                         break
 *                     i += 1
 */
          __pyx_v_self->_state = __pyx_e_11multifruits_DATA_CR;

          /* "multifruits.pyx":182
 *                     if c == b'\r':
 *                         self._state = DATA_CR
 *                         break             # <<<<<<<<<<<<<<
 *                     i += 1
 *                 if i > mark:
 */
          goto __pyx_L47_break;

          /* "multifruits.pyx":180
 *                 while i < length:
 *                     c = data[i]
 *                     if c == b'\r':             # <<<<<<<<<<<<<<
 *                         self._state = DATA_CR
 *                         break
 */
        }

        /* "multifruits.pyx":183
 *                         self._state = DATA_CR
 *                         break
 *                     i += 1             # <<<<<<<<<<<<<<
 *                 if i > mark:
 *                     if self._on_data is not None:
 */
        __pyx_v_i = (__pyx_v_i + 1);
      }
      __pyx_L47_break:;

      /* "multifruits.pyx":184
 *                         break
 *                     i += 1
 *                 if i > mark:             # <<<<<<<<<<<<<<
 *                     if self._on_data is not None:
 *                         self._on_data(data[mark:i])
 */
      __pyx_t_6 = ((__pyx_v_i > __pyx_v_mark) != 0);
      if (__pyx_t_6) {

        /* "multifruits.pyx":185
 *                     i += 1
 *                 if i > mark:
 *                     if self._on_data is not None:             # <<<<<<<<<<<<<<
 *                         self._on_data(data[mark:i])
 *                 i += 1
 */
        __pyx_t_6 = (__pyx_v_self->_on_data != Py_None);
        __pyx_t_2 = (__pyx_t_6 != 0);
        if (__pyx_t_2) {

          /* "multifruits.pyx":186
 *                 if i > mark:
 *                     if self._on_data is not None:
 *                         self._on_data(data[mark:i])             # <<<<<<<<<<<<<<
 *                 i += 1
 *             elif self._state == DATA_CR:
 */
          __pyx_t_4 = __Pyx_PyBytes_FromStringAndSize(__pyx_v_data + __pyx_v_mark, __pyx_v_i - __pyx_v_mark); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 186, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_INCREF(__pyx_v_self->_on_data);
          __pyx_t_8 = __pyx_v_self->_on_data; __pyx_t_9 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
            __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
            if (likely(__pyx_t_9)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
              __Pyx_INCREF(__pyx_t_9);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_8, function);
            }
          }
          if (!__pyx_t_9) {
            __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 186, __pyx_L1_error)
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            __Pyx_GOTREF(__pyx_t_5);
          } else {
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(__pyx_t_8)) {
              PyObject *__pyx_temp[2] = {__pyx_t_9, __pyx_t_4};
              __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_8, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 186, __pyx_L1_error)
              __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
              __Pyx_GOTREF(__pyx_t_5);
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            } else
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(__pyx_t_8)) {
              PyObject *__pyx_temp[2] = {__pyx_t_9, __pyx_t_4};
              __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_8, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 186, __pyx_L1_error)
              __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
              __Pyx_GOTREF(__pyx_t_5);
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            } else
            #endif
            {
              __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 186, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_3);
              __Pyx_GIVEREF(__pyx_t_9); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_9); __pyx_t_9 = NULL;
              __Pyx_GIVEREF(__pyx_t_4);
              PyTuple_SET_ITEM(__pyx_t_3, 0+1, __pyx_t_4);
              __pyx_t_4 = 0;
              __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_t_3, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 186, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_5);
              __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            }
          }
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

          /* "multifruits.pyx":185
 *                     i += 1
 *                 if i > mark:
 *                     if self._on_data is not None:             # <<<<<<<<<<<<<<
 *                         self._on_data(data[mark:i])
 *                 i += 1
 */
        }

        /* "multifruits.pyx":184
 *                         break
 *                     i += 1
 *                 if i > mark:             # <<<<<<<<<<<<<<
 *                     if self._on_data is not None:
 *                         self._on_data(data[mark:i])
 */
      }

      /* "multifruits.pyx":187
 *                     if self._on_data is not None:
 *                         self._on_data(data[mark:i])
 *                 i += 1             # <<<<<<<<<<<<<<
 *             elif self._state == DATA_CR:
 *                 if c == b'\n':
 */
      __pyx_v_i = (__pyx_v_i + 1);

      /* "multifruits.pyx":176
 *                 self._state = DATA
 *                 i += 1
 *             elif self._state == DATA:             # <<<<<<<<<<<<<<
 *                 mark = i
 *                 while i < length:
 */
      break;

      /* "multifruits.pyx":188
 *                         self._on_data(data[mark:i])
 *                 i += 1
 *             elif self._state == DATA_CR:             # <<<<<<<<<<<<<<
 *                 if c == b'\n':
 *                     self._state = DATA_CR_LF
 */
      case __pyx_e_11multifruits_DATA_CR:

      /* "multifruits.pyx":189
 *                 i += 1
 *             elif self._state == DATA_CR:
 *                 if c == b'\n':             # <<<<<<<<<<<<<<
 *                     self._state = DATA_CR_LF
 *                     i += 1
 */
      __pyx_t_2 = ((__pyx_v_c == '\n') != 0);
      if (__pyx_t_2) {

        /* "multifruits.pyx":190
 *             elif self._state == DATA_CR:
 *                 if c == b'\n':
 *                     self._state = DATA_CR_LF             # <<<<<<<<<<<<<<
 *                     i += 1
 *                 else:
 */
        __pyx_v_self->_state = __pyx_e_11multifruits_DATA_CR_LF;

        /* "multifruits.pyx":191
 *                 if c == b'\n':
 *                     self._state = DATA_CR_LF
 *                     i += 1             # <<<<<<<<<<<<<<
 *                 else:
 *                     if self._on_data is not None:
 */
        __pyx_v_i = (__pyx_v_i + 1);

        /* "multifruits.pyx":189
 *                 i += 1
 *             elif self._state == DATA_CR:
 *                 if c == b'\n':             # <<<<<<<<<<<<<<
 *                     self._state = DATA_CR_LF
 *                     i += 1
 */
        goto __pyx_L51;
      }

      /* "multifruits.pyx":193
 *                     i += 1
 *                 else:
 *                     if self._on_data is not None:             # <<<<<<<<<<<<<<
 *                         self._on_data(b'\r')
 *                     self._state = DATA
 */
      /*else*/ {
        __pyx_t_2 = (__pyx_v_self->_on_data != Py_None);
        __pyx_t_6 = (__pyx_t_2 != 0);
        if (__pyx_t_6) {

          /* "multifruits.pyx":194
 *                 else:
 *                     if self._on_data is not None:
 *                         self._on_data(b'\r')             # <<<<<<<<<<<<<<
 *                     self._state = DATA
 *             elif self._state == DATA_CR_LF:
 */
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_v_self->_on_data, __pyx_tuple__9, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 194, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

          /* "multifruits.pyx":193
 *                     i += 1
 *                 else:
 *                     if self._on_data is not None:             # <<<<<<<<<<<<<<
 *                         self._on_data(b'\r')
 *                     self._state = DATA
 */
        }

        /* "multifruits.pyx":195
 *                     if self._on_data is not None:
 *                         self._on_data(b'\r')
 *                     self._state = DATA             # <<<<<<<<<<<<<<
 *             elif self._state == DATA_CR_LF:
 *                 if c == b'-':
 */
        __pyx_v_self->_state = __pyx_e_11multifruits_DATA;
      }
      __pyx_L51:;

      /* "multifruits.pyx":188
 *                         self._on_data(data[mark:i])
 *                 i += 1
 *             elif self._state == DATA_CR:             # <<<<<<<<<<<<<<
 *                 if c == b'\n':
 *                     self._state = DATA_CR_LF
 */
      break;

      /* "multifruits.pyx":196
 *                         self._on_data(b'\r')
 *                     self._state = DATA
 *             elif self._state == DATA_CR_LF:             # <<<<<<<<<<<<<<
 *                 if c == b'-':
 *                     self._state = DATA_CR_LF_HY
 */
      case __pyx_e_11multifruits_DATA_CR_LF:

      /* "multifruits.pyx":197
 *                     self._state = DATA
 *             elif self._state == DATA_CR_LF:
 *                 if c == b'-':             # <<<<<<<<<<<<<<
 *                     self._state = DATA_CR_LF_HY
 *                     i += 1
 */
      __pyx_t_6 = ((__pyx_v_c == '-') != 0);
      if (__pyx_t_6) {

        /* "multifruits.pyx":198
 *             elif self._state == DATA_CR_LF:
 *                 if c == b'-':
 *                     self._state = DATA_CR_LF_HY             # <<<<<<<<<<<<<<
 *                     i += 1
 *                 else:
 */
        __pyx_v_self->_state = __pyx_e_11multifruits_DATA_CR_LF_HY;

        /* "multifruits.pyx":199
 *                 if c == b'-':
 *                     self._state = DATA_CR_LF_HY
 *                     i += 1             # <<<<<<<<<<<<<<
 *                 else:
 *                     if self._on_data is not None:
 */
        __pyx_v_i = (__pyx_v_i + 1);

        /* "multifruits.pyx":197
 *                     self._state = DATA
 *             elif self._state == DATA_CR_LF:
 *                 if c == b'-':             # <<<<<<<<<<<<<<
 *                     self._state = DATA_CR_LF_HY
 *                     i += 1
 */
        goto __pyx_L53;
      }

      /* "multifruits.pyx":201
 *                     i += 1
 *                 else:
 *                     if self._on_data is not None:             # <<<<<<<<<<<<<<
 *                         self._on_data(b'\r\n')
 *                     self._state = DATA
 */
      /*else*/ {
        __pyx_t_6 = (__pyx_v_self->_on_data != Py_None);
        __pyx_t_2 = (__pyx_t_6 != 0);
        if (__pyx_t_2) {

          /* "multifruits.pyx":202
 *                 else:
 *                     if self._on_data is not None:
 *                         self._on_data(b'\r\n')             # <<<<<<<<<<<<<<
 *                     self._state = DATA
 *             elif self._state == DATA_CR_LF_HY:
 */
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_v_self->_on_data, __pyx_tuple__11, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 202, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

          /* "multifruits.pyx":201
 *                     i += 1
 *                 else:
 *                     if self._on_data is not None:             # <<<<<<<<<<<<<<
 *                         self._on_data(b'\r\n')
 *                     self._state = DATA
 */
        }

        /* "multifruits.pyx":203
 *                     if self._on_data is not None:
 *                         self._on_data(b'\r\n')
 *                     self._state = DATA             # <<<<<<<<<<<<<<
 *             elif self._state == DATA_CR_LF_HY:
 *                 if c == b'-':
 */
        __pyx_v_self->_state = __pyx_e_11multifruits_DATA;
      }
      __pyx_L53:;

      /* "multifruits.pyx":196
 *                         self._on_data(b'\r')
 *                     self._state = DATA
 *             elif self._state == DATA_CR_LF:             # <<<<<<<<<<<<<<
 *                 if c == b'-':
 *                     self._state = DATA_CR_LF_HY
 */
      break;

      /* "multifruits.pyx":204
 *                         self._on_data(b'\r\n')
 *                     self._state = DATA
 *             elif self._state == DATA_CR_LF_HY:             # <<<<<<<<<<<<<<
 *                 if c == b'-':
 *                     self._state = DATA_BOUNDARY
 */
      case __pyx_e_11multifruits_DATA_CR_LF_HY:

      /* "multifruits.pyx":205
 *                     self._state = DATA
 *             elif self._state == DATA_CR_LF_HY:
 *                 if c == b'-':             # <<<<<<<<<<<<<<
 *                     self._state = DATA_BOUNDARY
 *                     self._boundary_index = 0
 */
      __pyx_t_2 = ((__pyx_v_c == '-') != 0);
      if (__pyx_t_2) {

        /* "multifruits.pyx":206
 *             elif self._state == DATA_CR_LF_HY:
 *                 if c == b'-':
 *                     self._state = DATA_BOUNDARY             # <<<<<<<<<<<<<<
 *                     self._boundary_index = 0
 *                     i += 1
 */
        __pyx_v_self->_state = __pyx_e_11multifruits_DATA_BOUNDARY;

        /* "multifruits.pyx":207
 *                 if c == b'-':
 *                     self._state = DATA_BOUNDARY
 *                     self._boundary_index = 0             # <<<<<<<<<<<<<<
 *                     i += 1
 *                 else:
 */
        __pyx_v_self->_boundary_index = 0;

        /* "multifruits.pyx":208
 *                     self._state = DATA_BOUNDARY
 *                     self._boundary_index = 0
 *                     i += 1             # <<<<<<<<<<<<<<
 *                 else:
 *                     if self._on_data is not None:
 */
        __pyx_v_i = (__pyx_v_i + 1);

        /* "multifruits.pyx":205
 *                     self._state = DATA
 *             elif self._state == DATA_CR_LF_HY:
 *                 if c == b'-':             # <<<<<<<<<<<<<<
 *                     self._state = DATA_BOUNDARY
 *                     self._boundary_index = 0
 */
        goto __pyx_L55;
      }

      /* "multifruits.pyx":210
 *                     i += 1
 *                 else:
 *                     if self._on_data is not None:             # <<<<<<<<<<<<<<
 *                         self._on_data(b'\r\n-')
 *                     self._state = DATA
 */
      /*else*/ {
        __pyx_t_2 = (__pyx_v_self->_on_data != Py_None);
        __pyx_t_6 = (__pyx_t_2 != 0);
        if (__pyx_t_6) {

          /* "multifruits.pyx":211
 *                 else:
 *                     if self._on_data is not None:
 *                         self._on_data(b'\r\n-')             # <<<<<<<<<<<<<<
 *                     self._state = DATA
 *             elif self._state == DATA_BOUNDARY:
 */
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_v_self->_on_data, __pyx_tuple__13, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 211, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

          /* "multifruits.pyx":210
 *                     i += 1
 *                 else:
 *                     if self._on_data is not None:             # <<<<<<<<<<<<<<
 *                         self._on_data(b'\r\n-')
 *                     self._state = DATA
 */
        }

        /* "multifruits.pyx":212
 *                     if self._on_data is not None:
 *                         self._on_data(b'\r\n-')
 *                     self._state = DATA             # <<<<<<<<<<<<<<
 *             elif self._state == DATA_BOUNDARY:
 *                 while i < length:
 */
        __pyx_v_self->_state = __pyx_e_11multifruits_DATA;
      }
      __pyx_L55:;

      /* "multifruits.pyx":204
 *                         self._on_data(b'\r\n')
 *                     self._state = DATA
 *             elif self._state == DATA_CR_LF_HY:             # <<<<<<<<<<<<<<
 *                 if c == b'-':
 *                     self._state = DATA_BOUNDARY
 */
      break;

      /* "multifruits.pyx":213
 *                         self._on_data(b'\r\n-')
 *                     self._state = DATA
 *             elif self._state == DATA_BOUNDARY:             # <<<<<<<<<<<<<<
 *                 while i < length:
 *                     c = data[i]
 */
      case __pyx_e_11multifruits_DATA_BOUNDARY:

      /* "multifruits.pyx":214
 *                     self._state = DATA
 *             elif self._state == DATA_BOUNDARY:
 *                 while i < length:             # <<<<<<<<<<<<<<
 *                     c = data[i]
 *                     if self._boundary_index == self._boundary_length:
 */
      while (1) {
        __pyx_t_6 = ((__pyx_v_i < __pyx_v_length) != 0);
        if (!__pyx_t_6) break;

        /* "multifruits.pyx":215
 *             elif self._state == DATA_BOUNDARY:
 *                 while i < length:
 *                     c = data[i]             # <<<<<<<<<<<<<<
 *                     if self._boundary_index == self._boundary_length:
 *                         self._state = DATA_BOUNDARY_DONE
 */
        __pyx_v_c = (__pyx_v_data[__pyx_v_i]);

        /* "multifruits.pyx":216
 *                 while i < length:
 *                     c = data[i]
 *                     if self._boundary_index == self._boundary_length:             # <<<<<<<<<<<<<<
 *                         self._state = DATA_BOUNDARY_DONE
 *                         break
 */
        __pyx_t_6 = ((__pyx_v_self->_boundary_index == __pyx_v_self->_boundary_length) != 0);
        if (__pyx_t_6) {

          /* "multifruits.pyx":217
 *                     c = data[i]
 *                     if self._boundary_index == self._boundary_length:
 *                         self._state = DATA_BOUNDARY_DONE             # <<<<<<<<<<<<<<
 *                         break
 *                     elif c == self._boundary[self._boundary_index]:
 */
          __pyx_v_self->_state = __pyx_e_11multifruits_DATA_BOUNDARY_DONE;

          /* "multifruits.pyx":218
 *                     if self._boundary_index == self._boundary_length:
 *                         self._state = DATA_BOUNDARY_DONE
 *                         break             # <<<<<<<<<<<<<<
 *                     elif c == self._boundary[self._boundary_index]:
 *                         self._boundary_index += 1
 */
          goto __pyx_L58_break;

          /* "multifruits.pyx":216
 *                 while i < length:
 *                     c = data[i]
 *                     if self._boundary_index == self._boundary_length:             # <<<<<<<<<<<<<<
 *                         self._state = DATA_BOUNDARY_DONE
 *                         break
 */
        }

        /* "multifruits.pyx":219
 *                         self._state = DATA_BOUNDARY_DONE
 *                         break
 *                     elif c == self._boundary[self._boundary_index]:             # <<<<<<<<<<<<<<
 *                         self._boundary_index += 1
 *                         i += 1
 */
        __pyx_t_5 = __Pyx_PyInt_From_char(__pyx_v_c); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 219, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_8 = __Pyx_GetItemInt(__pyx_v_self->_boundary, __pyx_v_self->_boundary_index, unsigned int, 0, __Pyx_PyInt_From_unsigned_int, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 219, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_3 = PyObject_RichCompare(__pyx_t_5, __pyx_t_8, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 219, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 219, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (__pyx_t_6) {

          /* "multifruits.pyx":220
 *                         break
 *                     elif c == self._boundary[self._boundary_index]:
 *                         self._boundary_index += 1             # <<<<<<<<<<<<<<
 *                         i += 1
 *                     else:
 */
          __pyx_v_self->_boundary_index = (__pyx_v_self->_boundary_index + 1);

          /* "multifruits.pyx":221
 *                     elif c == self._boundary[self._boundary_index]:
 *                         self._boundary_index += 1
 *                         i += 1             # <<<<<<<<<<<<<<
 *                     else:
 *                         if self._on_data is not None:
 */
          __pyx_v_i = (__pyx_v_i + 1);

          /* "multifruits.pyx":219
 *                         self._state = DATA_BOUNDARY_DONE
 *                         break
 *                     elif c == self._boundary[self._boundary_index]:             # <<<<<<<<<<<<<<
 *                         self._boundary_index += 1
 *                         i += 1
 */
          goto __pyx_L59;
        }

        /* "multifruits.pyx":223
 *                         i += 1
 *                     else:
 *                         if self._on_data is not None:             # <<<<<<<<<<<<<<
 *                             self._on_data(b'\r\n--')
 *                             self._on_data(self._boundary[:self._boundary_index])
 */
        /*else*/ {
          __pyx_t_6 = (__pyx_v_self->_on_data != Py_None);
          __pyx_t_2 = (__pyx_t_6 != 0);
          if (__pyx_t_2) {

            /* "multifruits.pyx":224
 *                     else:
 *                         if self._on_data is not None:
 *                             self._on_data(b'\r\n--')             # <<<<<<<<<<<<<<
 *                             self._on_data(self._boundary[:self._boundary_index])
 *                         self._state = DATA
 */
            __pyx_t_3 = __Pyx_PyObject_Call(__pyx_v_self->_on_data, __pyx_tuple__15, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 224, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_3);
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

            /* "multifruits.pyx":225
 *                         if self._on_data is not None:
 *                             self._on_data(b'\r\n--')
 *                             self._on_data(self._boundary[:self._boundary_index])             # <<<<<<<<<<<<<<
 *                         self._state = DATA
 *                         break
 */
            if (unlikely(__pyx_v_self->_boundary == Py_None)) {
              PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
              __PYX_ERR(0, 225, __pyx_L1_error)
            }
            __pyx_t_8 = PySequence_GetSlice(__pyx_v_self->_boundary, 0, __pyx_v_self->_boundary_index); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 225, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_8);
            __Pyx_INCREF(__pyx_v_self->_on_data);
            __pyx_t_5 = __pyx_v_self->_on_data; __pyx_t_4 = NULL;
            if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
              __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
              if (likely(__pyx_t_4)) {
                PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
                __Pyx_INCREF(__pyx_t_4);
                __Pyx_INCREF(function);
                __Pyx_DECREF_SET(__pyx_t_5, function);
              }
            }
            if (!__pyx_t_4) {
              __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 225, __pyx_L1_error)
              __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
              __Pyx_GOTREF(__pyx_t_3);
            } else {
              #if CYTHON_FAST_PYCALL
              if (PyFunction_Check(__pyx_t_5)) {
                PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_t_8};
                __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 225, __pyx_L1_error)
                __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
                __Pyx_GOTREF(__pyx_t_3);
                __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
              } else
              #endif
              #if CYTHON_FAST_PYCCALL
              if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
                PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_t_8};
                __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 225, __pyx_L1_error)
                __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
                __Pyx_GOTREF(__pyx_t_3);
                __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
              } else
              #endif
              {
                __pyx_t_9 = PyTuple_New(1+1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 225, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_9);
                __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_9, 0, __pyx_t_4); __pyx_t_4 = NULL;
                __Pyx_GIVEREF(__pyx_t_8);
                PyTuple_SET_ITEM(__pyx_t_9, 0+1, __pyx_t_8);
                __pyx_t_8 = 0;
                __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_9, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 225, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_3);
                __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
              }
            }
            __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

            /* "multifruits.pyx":223
 *                         i += 1
 *                     else:
 *                         if self._on_data is not None:             # <<<<<<<<<<<<<<
 *                             self._on_data(b'\r\n--')
 *                             self._on_data(self._boundary[:self._boundary_index])
 */
          }

          /* "multifruits.pyx":226
 *                             self._on_data(b'\r\n--')
 *                             self._on_data(self._boundary[:self._boundary_index])
 *                         self._state = DATA             # <<<<<<<<<<<<<<
 *                         break
 *             elif self._state == DATA_BOUNDARY_DONE:
 */
          __pyx_v_self->_state = __pyx_e_11multifruits_DATA;

          /* "multifruits.pyx":227
 *                             self._on_data(self._boundary[:self._boundary_index])
 *                         self._state = DATA
 *                         break             # <<<<<<<<<<<<<<
 *             elif self._state == DATA_BOUNDARY_DONE:
 *                 if c == b'\r':
 */
          goto __pyx_L58_break;
        }
        __pyx_L59:;
      }
      __pyx_L58_break:;

      /* "multifruits.pyx":213
 *                         self._on_data(b'\r\n-')
 *                     self._state = DATA
 *             elif self._state == DATA_BOUNDARY:             # <<<<<<<<<<<<<<
 *                 while i < length:
 *                     c = data[i]
 */
      break;

      /* "multifruits.pyx":228
 *                         self._state = DATA
 *                         break
 *             elif self._state == DATA_BOUNDARY_DONE:             # <<<<<<<<<<<<<<
 *                 if c == b'\r':
 *                     self._state = DATA_BOUNDARY_DONE_CR_LF
 */
      case __pyx_e_11multifruits_DATA_BOUNDARY_DONE:

      /* "multifruits.pyx":229
 *                         break
 *             elif self._state == DATA_BOUNDARY_DONE:
 *                 if c == b'\r':             # <<<<<<<<<<<<<<
 *                     self._state = DATA_BOUNDARY_DONE_CR_LF
 *                 elif c == b'-':
 */
      __pyx_t_2 = ((__pyx_v_c == '\r') != 0);
      if (__pyx_t_2) {

        /* "multifruits.pyx":230
 *             elif self._state == DATA_BOUNDARY_DONE:
 *                 if c == b'\r':
 *                     self._state = DATA_BOUNDARY_DONE_CR_LF             # <<<<<<<<<<<<<<
 *                 elif c == b'-':
 *                     self._state = DATA_BOUNDARY_DONE_HY_HY
 */
        __pyx_v_self->_state = __pyx_e_11multifruits_DATA_BOUNDARY_DONE_CR_LF;

        /* "multifruits.pyx":229
 *                         break
 *             elif self._state == DATA_BOUNDARY_DONE:
 *                 if c == b'\r':             # <<<<<<<<<<<<<<
 *                     self._state = DATA_BOUNDARY_DONE_CR_LF
 *                 elif c == b'-':
 */
        goto __pyx_L61;
      }

      /* "multifruits.pyx":231
 *                 if c == b'\r':
 *                     self._state = DATA_BOUNDARY_DONE_CR_LF
 *                 elif c == b'-':             # <<<<<<<<<<<<<<
 *                     self._state = DATA_BOUNDARY_DONE_HY_HY
 *                 else:
 */
      __pyx_t_2 = ((__pyx_v_c == '-') != 0);
      if (__pyx_t_2) {

        /* "multifruits.pyx":232
 *                     self._state = DATA_BOUNDARY_DONE_CR_LF
 *                 elif c == b'-':
 *                     self._state = DATA_BOUNDARY_DONE_HY_HY             # <<<<<<<<<<<<<<
 *                 else:
 *                     raise ValueError('DATA_BOUNDARY_DONE')
 */
        __pyx_v_self->_state = __pyx_e_11multifruits_DATA_BOUNDARY_DONE_HY_HY;

        /* "multifruits.pyx":231
 *                 if c == b'\r':
 *                     self._state = DATA_BOUNDARY_DONE_CR_LF
 *                 elif c == b'-':             # <<<<<<<<<<<<<<
 *                     self._state = DATA_BOUNDARY_DONE_HY_HY
 *                 else:
 */
        goto __pyx_L61;
      }

      /* "multifruits.pyx":234
 *                     self._state = DATA_BOUNDARY_DONE_HY_HY
 *                 else:
 *                     raise ValueError('DATA_BOUNDARY_DONE')             # <<<<<<<<<<<<<<
 *                 i += 1
 *             elif self._state == DATA_BOUNDARY_DONE_CR_LF:
 */
      /*else*/ {
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__16, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 234, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_Raise(__pyx_t_3, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __PYX_ERR(0, 234, __pyx_L1_error)
      }
      __pyx_L61:;

      /* "multifruits.pyx":235
 *                 else:
 *                     raise ValueError('DATA_BOUNDARY_DONE')
 *                 i += 1             # <<<<<<<<<<<<<<
 *             elif self._state == DATA_BOUNDARY_DONE_CR_LF:
 *                 if c != b'\n':
 */
      __pyx_v_i = (__pyx_v_i + 1);

      /* "multifruits.pyx":228
 *                         self._state = DATA
 *                         break
 *             elif self._state == DATA_BOUNDARY_DONE:             # <<<<<<<<<<<<<<
 *                 if c == b'\r':
 *                     self._state = DATA_BOUNDARY_DONE_CR_LF
 */
      break;

      /* "multifruits.pyx":236
 *                     raise ValueError('DATA_BOUNDARY_DONE')
 *                 i += 1
 *             elif self._state == DATA_BOUNDARY_DONE_CR_LF:             # <<<<<<<<<<<<<<
 *                 if c != b'\n':
 *                     raise ValueError('DATA_BOUNDARY_DONE_CR_LF')
 */
      case __pyx_e_11multifruits_DATA_BOUNDARY_DONE_CR_LF:

      /* "multifruits.pyx":237
 *                 i += 1
 *             elif self._state == DATA_BOUNDARY_DONE_CR_LF:
 *                 if c != b'\n':             # <<<<<<<<<<<<<<
 *                     raise ValueError('DATA_BOUNDARY_DONE_CR_LF')
 *                 if self._on_part_complete is not None:
 */
      __pyx_t_2 = ((__pyx_v_c != '\n') != 0);
      if (__pyx_t_2) {

        /* "multifruits.pyx":238
 *             elif self._state == DATA_BOUNDARY_DONE_CR_LF:
 *                 if c != b'\n':
 *                     raise ValueError('DATA_BOUNDARY_DONE_CR_LF')             # <<<<<<<<<<<<<<
 *                 if self._on_part_complete is not None:
 *                     self._on_part_complete()
 */
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__17, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 238, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_Raise(__pyx_t_3, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __PYX_ERR(0, 238, __pyx_L1_error)

        /* "multifruits.pyx":237
 *                 i += 1
 *             elif self._state == DATA_BOUNDARY_DONE_CR_LF:
 *                 if c != b'\n':             # <<<<<<<<<<<<<<
 *                     raise ValueError('DATA_BOUNDARY_DONE_CR_LF')
 *                 if self._on_part_complete is not None:
 */
      }

      /* "multifruits.pyx":239
 *                 if c != b'\n':
 *                     raise ValueError('DATA_BOUNDARY_DONE_CR_LF')
 *                 if self._on_part_complete is not None:             # <<<<<<<<<<<<<<
 *                     self._on_part_complete()
 *                 if self._on_part_begin is not None:
 */
      __pyx_t_2 = (__pyx_v_self->_on_part_complete != Py_None);
      __pyx_t_6 = (__pyx_t_2 != 0);
      if (__pyx_t_6) {

        /* "multifruits.pyx":240
 *                     raise ValueError('DATA_BOUNDARY_DONE_CR_LF')
 *                 if self._on_part_complete is not None:
 *                     self._on_part_complete()             # <<<<<<<<<<<<<<
 *                 if self._on_part_begin is not None:
 *                     self._on_part_begin()
 */
        __Pyx_INCREF(__pyx_v_self->_on_part_complete);
        __pyx_t_5 = __pyx_v_self->_on_part_complete; __pyx_t_9 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_9)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_9);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
          }
        }
        if (__pyx_t_9) {
          __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_9); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 240, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        } else {
          __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 240, __pyx_L1_error)
        }
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

        /* "multifruits.pyx":239
 *                 if c != b'\n':
 *                     raise ValueError('DATA_BOUNDARY_DONE_CR_LF')
 *                 if self._on_part_complete is not None:             # <<<<<<<<<<<<<<
 *                     self._on_part_complete()
 *                 if self._on_part_begin is not None:
 */
      }

      /* "multifruits.pyx":241
 *                 if self._on_part_complete is not None:
 *                     self._on_part_complete()
 *                 if self._on_part_begin is not None:             # <<<<<<<<<<<<<<
 *                     self._on_part_begin()
 *                 self._state = HEADER_NAME_START
 */
      __pyx_t_6 = (__pyx_v_self->_on_part_begin != Py_None);
      __pyx_t_2 = (__pyx_t_6 != 0);
      if (__pyx_t_2) {

        /* "multifruits.pyx":242
 *                     self._on_part_complete()
 *                 if self._on_part_begin is not None:
 *                     self._on_part_begin()             # <<<<<<<<<<<<<<
 *                 self._state = HEADER_NAME_START
 *                 i += 1
 */
        __Pyx_INCREF(__pyx_v_self->_on_part_begin);
        __pyx_t_5 = __pyx_v_self->_on_part_begin; __pyx_t_9 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_9)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_9);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
          }
        }
        if (__pyx_t_9) {
          __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_9); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 242, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        } else {
          __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 242, __pyx_L1_error)
        }
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

        /* "multifruits.pyx":241
 *                 if self._on_part_complete is not None:
 *                     self._on_part_complete()
 *                 if self._on_part_begin is not None:             # <<<<<<<<<<<<<<
 *                     self._on_part_begin()
 *                 self._state = HEADER_NAME_START
 */
      }

      /* "multifruits.pyx":243
 *                 if self._on_part_begin is not None:
 *                     self._on_part_begin()
 *                 self._state = HEADER_NAME_START             # <<<<<<<<<<<<<<
 *                 i += 1
 *             elif self._state == DATA_BOUNDARY_DONE_HY_HY:
 */
      __pyx_v_self->_state = __pyx_e_11multifruits_HEADER_NAME_START;

      /* "multifruits.pyx":244
 *                     self._on_part_begin()
 *                 self._state = HEADER_NAME_START
 *                 i += 1             # <<<<<<<<<<<<<<
 *             elif self._state == DATA_BOUNDARY_DONE_HY_HY:
 *                 if c != b'-':
 */
      __pyx_v_i = (__pyx_v_i + 1);

      /* "multifruits.pyx":236
 *                     raise ValueError('DATA_BOUNDARY_DONE')
 *                 i += 1
 *             elif self._state == DATA_BOUNDARY_DONE_CR_LF:             # <<<<<<<<<<<<<<
 *                 if c != b'\n':
 *                     raise ValueError('DATA_BOUNDARY_DONE_CR_LF')
 */
      break;

      /* "multifruits.pyx":245
 *                 self._state = HEADER_NAME_START
 *                 i += 1
 *             elif self._state == DATA_BOUNDARY_DONE_HY_HY:             # <<<<<<<<<<<<<<
 *                 if c != b'-':
 *                     raise ValueError('DATA_BOUNDARY_DONE_HY_HY')
 */
      case __pyx_e_11multifruits_DATA_BOUNDARY_DONE_HY_HY:

      /* "multifruits.pyx":246
 *                 i += 1
 *             elif self._state == DATA_BOUNDARY_DONE_HY_HY:
 *                 if c != b'-':             # <<<<<<<<<<<<<<
 *                     raise ValueError('DATA_BOUNDARY_DONE_HY_HY')
 *                 if self._on_part_complete is not None:
 */
      __pyx_t_2 = ((__pyx_v_c != '-') != 0);
      if (__pyx_t_2) {

        /* "multifruits.pyx":247
 *             elif self._state == DATA_BOUNDARY_DONE_HY_HY:
 *                 if c != b'-':
 *                     raise ValueError('DATA_BOUNDARY_DONE_HY_HY')             # <<<<<<<<<<<<<<
 *                 if self._on_part_complete is not None:
 *                     self._on_part_complete()
 */
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__18, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 247, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_Raise(__pyx_t_3, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __PYX_ERR(0, 247, __pyx_L1_error)

        /* "multifruits.pyx":246
 *                 i += 1
 *             elif self._state == DATA_BOUNDARY_DONE_HY_HY:
 *                 if c != b'-':             # <<<<<<<<<<<<<<
 *                     raise ValueError('DATA_BOUNDARY_DONE_HY_HY')
 *                 if self._on_part_complete is not None:
 */
      }

      /* "multifruits.pyx":248
 *                 if c != b'-':
 *                     raise ValueError('DATA_BOUNDARY_DONE_HY_HY')
 *                 if self._on_part_complete is not None:             # <<<<<<<<<<<<<<
 *                     self._on_part_complete()
 *                 if self._on_body_complete is not None:
 */
      __pyx_t_2 = (__pyx_v_self->_on_part_complete != Py_None);
      __pyx_t_6 = (__pyx_t_2 != 0);
      if (__pyx_t_6) {

        /* "multifruits.pyx":249
 *                     raise ValueError('DATA_BOUNDARY_DONE_HY_HY')
 *                 if self._on_part_complete is not None:
 *                     self._on_part_complete()             # <<<<<<<<<<<<<<
 *                 if self._on_body_complete is not None:
 *                     self._on_body_complete()
 */
        __Pyx_INCREF(__pyx_v_self->_on_part_complete);
        __pyx_t_5 = __pyx_v_self->_on_part_complete; __pyx_t_9 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_9)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_9);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
          }
        }
        if (__pyx_t_9) {
          __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_9); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 249, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        } else {
          __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 249, __pyx_L1_error)
        }
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

        /* "multifruits.pyx":248
 *                 if c != b'-':
 *                     raise ValueError('DATA_BOUNDARY_DONE_HY_HY')
 *                 if self._on_part_complete is not None:             # <<<<<<<<<<<<<<
 *                     self._on_part_complete()
 *                 if self._on_body_complete is not None:
 */
      }

      /* "multifruits.pyx":250
 *                 if self._on_part_complete is not None:
 *                     self._on_part_complete()
 *                 if self._on_body_complete is not None:             # <<<<<<<<<<<<<<
 *                     self._on_body_complete()
 *                 self._state = EPILOGUE
 */
      __pyx_t_6 = (__pyx_v_self->_on_body_complete != Py_None);
      __pyx_t_2 = (__pyx_t_6 != 0);
      if (__pyx_t_2) {

        /* "multifruits.pyx":251
 *                     self._on_part_complete()
 *                 if self._on_body_complete is not None:
 *                     self._on_body_complete()             # <<<<<<<<<<<<<<
 *                 self._state = EPILOGUE
 *                 i += 1
 */
        __Pyx_INCREF(__pyx_v_self->_on_body_complete);
        __pyx_t_5 = __pyx_v_self->_on_body_complete; __pyx_t_9 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_9)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_9);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
          }
        }
        if (__pyx_t_9) {
          __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_9); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 251, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        } else {
          __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_5); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 251, __pyx_L1_error)
        }
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

        /* "multifruits.pyx":250
 *                 if self._on_part_complete is not None:
 *                     self._on_part_complete()
 *                 if self._on_body_complete is not None:             # <<<<<<<<<<<<<<
 *                     self._on_body_complete()
 *                 self._state = EPILOGUE
 */
      }

      /* "multifruits.pyx":252
 *                 if self._on_body_complete is not None:
 *                     self._on_body_complete()
 *                 self._state = EPILOGUE             # <<<<<<<<<<<<<<
 *                 i += 1
 *             elif self._state == EPILOGUE:
 */
      __pyx_v_self->_state = __pyx_e_11multifruits_EPILOGUE;

      /* "multifruits.pyx":253
 *                     self._on_body_complete()
 *                 self._state = EPILOGUE
 *                 i += 1             # <<<<<<<<<<<<<<
 *             elif self._state == EPILOGUE:
 *                 i += 1
 */
      __pyx_v_i = (__pyx_v_i + 1);

      /* "multifruits.pyx":245
 *                 self._state = HEADER_NAME_START
 *                 i += 1
 *             elif self._state == DATA_BOUNDARY_DONE_HY_HY:             # <<<<<<<<<<<<<<
 *                 if c != b'-':
 *                     raise ValueError('DATA_BOUNDARY_DONE_HY_HY')
 */
      break;

      /* "multifruits.pyx":254
 *                 self._state = EPILOGUE
 *                 i += 1
 *             elif self._state == EPILOGUE:             # <<<<<<<<<<<<<<
 *                 i += 1
 *                 # Must be ignored according to rfc 1341.
 */
      case __pyx_e_11multifruits_EPILOGUE:

      /* "multifruits.pyx":255
 *                 i += 1
 *             elif self._state == EPILOGUE:
 *                 i += 1             # <<<<<<<<<<<<<<
 *                 # Must be ignored according to rfc 1341.
 *                 break
 */
      __pyx_v_i = (__pyx_v_i + 1);

      /* "multifruits.pyx":257
 *                 i += 1
 *                 # Must be ignored according to rfc 1341.
 *                 break             # <<<<<<<<<<<<<<
 * 
 *     def feed_data(self, char* data):
 */
      goto __pyx_L4_break;

      /* "multifruits.pyx":254
 *                 self._state = EPILOGUE
 *                 i += 1
 *             elif self._state == EPILOGUE:             # <<<<<<<<<<<<<<
 *                 i += 1
 *                 # Must be ignored according to rfc 1341.
 */
      break;
      default: break;
    }
  }
  __pyx_L4_break:;

  /* "multifruits.pyx":84
 *             self._on_headers_complete()
 * 
 *     cdef _feed_data(self, char* data):             # <<<<<<<<<<<<<<
 *         cdef:
 *             unsigned int i = 0
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("multifruits.Parser._feed_data", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "multifruits.pyx":259
 *                 break
 * 
 *     def feed_data(self, char* data):             # <<<<<<<<<<<<<<
 *         return self._feed_data(data)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_11multifruits_6Parser_11feed_data(PyObject *__pyx_v_self, PyObject *__pyx_arg_data); /*proto*/
static PyObject *__pyx_pw_11multifruits_6Parser_11feed_data(PyObject *__pyx_v_self, PyObject *__pyx_arg_data) {
  char *__pyx_v_data;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("feed_data (wrapper)", 0);
  assert(__pyx_arg_data); {
    __pyx_v_data = __Pyx_PyObject_AsWritableString(__pyx_arg_data); if (unlikely((!__pyx_v_data) && PyErr_Occurred())) __PYX_ERR(0, 259, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("multifruits.Parser.feed_data", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_11multifruits_6Parser_10feed_data(((struct __pyx_obj_11multifruits_Parser *)__pyx_v_self), ((char *)__pyx_v_data));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11multifruits_6Parser_10feed_data(struct __pyx_obj_11multifruits_Parser *__pyx_v_self, char *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("feed_data", 0);

  /* "multifruits.pyx":260
 * 
 *     def feed_data(self, char* data):
 *         return self._feed_data(data)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_11multifruits_Parser *)__pyx_v_self->__pyx_vtab)->_feed_data(__pyx_v_self, __pyx_v_data); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 260, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "multifruits.pyx":259
 *                 break
 * 
 *     def feed_data(self, char* data):             # <<<<<<<<<<<<<<
 *         return self._feed_data(data)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("multifruits.Parser.feed_data", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = (self._boundary, self._boundary_index, self._boundary_length, self._current_header_name, self._current_header_value, self._on_body_begin, self._on_body_complete, self._on_data, self._on_header, self._on_headers_complete, self._on_part_begin, self._on_part_complete, self._state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_11multifruits_6Parser_13__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_11multifruits_6Parser_13__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11multifruits_6Parser_12__reduce_cython__(((struct __pyx_obj_11multifruits_Parser *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11multifruits_6Parser_12__reduce_cython__(struct __pyx_obj_11multifruits_Parser *__pyx_v_self) {
  int __pyx_v_use_setstate;
  PyObject *__pyx_v_state = NULL;
  PyObject *__pyx_v__dict = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     cdef bint use_setstate
 *     state = (self._boundary, self._boundary_index, self._boundary_length, self._current_header_name, self._current_header_value, self._on_body_begin, self._on_body_complete, self._on_data, self._on_header, self._on_headers_complete, self._on_part_begin, self._on_part_complete, self._state)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->_boundary_index); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_unsigned_int(__pyx_v_self->_boundary_length); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_unsigned_char(__pyx_v_self->_state); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(13); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_INCREF(__pyx_v_self->_boundary);
  __Pyx_GIVEREF(__pyx_v_self->_boundary);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_self->_boundary);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->_current_header_name);
  __Pyx_GIVEREF(__pyx_v_self->_current_header_name);
  PyTuple_SET_ITEM(__pyx_t_4, 3, __pyx_v_self->_current_header_name);
  __Pyx_INCREF(__pyx_v_self->_current_header_value);
  __Pyx_GIVEREF(__pyx_v_self->_current_header_value);
  PyTuple_SET_ITEM(__pyx_t_4, 4, __pyx_v_self->_current_header_value);
  __Pyx_INCREF(__pyx_v_self->_on_body_begin);
  __Pyx_GIVEREF(__pyx_v_self->_on_body_begin);
  PyTuple_SET_ITEM(__pyx_t_4, 5, __pyx_v_self->_on_body_begin);
  __Pyx_INCREF(__pyx_v_self->_on_body_complete);
  __Pyx_GIVEREF(__pyx_v_self->_on_body_complete);
  PyTuple_SET_ITEM(__pyx_t_4, 6, __pyx_v_self->_on_body_complete);
  __Pyx_INCREF(__pyx_v_self->_on_data);
  __Pyx_GIVEREF(__pyx_v_self->_on_data);
  PyTuple_SET_ITEM(__pyx_t_4, 7, __pyx_v_self->_on_data);
  __Pyx_INCREF(__pyx_v_self->_on_header);
  __Pyx_GIVEREF(__pyx_v_self->_on_header);
  PyTuple_SET_ITEM(__pyx_t_4, 8, __pyx_v_self->_on_header);
  __Pyx_INCREF(__pyx_v_self->_on_headers_complete);
  __Pyx_GIVEREF(__pyx_v_self->_on_headers_complete);
  PyTuple_SET_ITEM(__pyx_t_4, 9, __pyx_v_self->_on_headers_complete);
  __Pyx_INCREF(__pyx_v_self->_on_part_begin);
  __Pyx_GIVEREF(__pyx_v_self->_on_part_begin);
  PyTuple_SET_ITEM(__pyx_t_4, 10, __pyx_v_self->_on_part_begin);
  __Pyx_INCREF(__pyx_v_self->_on_part_complete);
  __Pyx_GIVEREF(__pyx_v_self->_on_part_complete);
  PyTuple_SET_ITEM(__pyx_t_4, 11, __pyx_v_self->_on_part_complete);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 12, __pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "(tree fragment)":4
 *     cdef bint use_setstate
 *     state = (self._boundary, self._boundary_index, self._boundary_length, self._current_header_name, self._current_header_value, self._on_body_begin, self._on_body_complete, self._on_data, self._on_header, self._on_headers_complete, self._on_part_begin, self._on_part_complete, self._state)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":5
 *     state = (self._boundary, self._boundary_index, self._boundary_length, self._current_header_name, self._current_header_value, self._on_body_begin, self._on_body_complete, self._on_data, self._on_header, self._on_headers_complete, self._on_part_begin, self._on_part_complete, self._state)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_5 = (__pyx_v__dict != Py_None);
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":6
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":7
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._boundary is not None or self._current_header_name is not None or self._current_header_value is not None or self._on_body_begin is not None or self._on_body_complete is not None or self._on_data is not None or self._on_header is not None or self._on_headers_complete is not None or self._on_part_begin is not None or self._on_part_complete is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":5
 *     state = (self._boundary, self._boundary_index, self._boundary_length, self._current_header_name, self._current_header_value, self._on_body_begin, self._on_body_complete, self._on_data, self._on_header, self._on_headers_complete, self._on_part_begin, self._on_part_complete, self._state)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":9
 *         use_setstate = True
 *     else:
 *         use_setstate = self._boundary is not None or self._current_header_name is not None or self._current_header_value is not None or self._on_body_begin is not None or self._on_body_complete is not None or self._on_data is not None or self._on_header is not None or self._on_headers_complete is not None or self._on_part_begin is not None or self._on_part_complete is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_Parser, (type(self), 0x7a58f64, None), state
 */
  /*else*/ {
    __pyx_t_5 = (__pyx_v_self->_boundary != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->_current_header_name != ((PyObject*)Py_None));
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_self->_current_header_value != ((PyObject*)Py_None));
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->_on_body_begin != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_self->_on_body_complete != Py_None);
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->_on_data != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_self->_on_header != Py_None);
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->_on_headers_complete != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_6 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_self->_on_part_begin != Py_None);
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_self->_on_part_complete != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    __pyx_t_6 = __pyx_t_5;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_6;
  }
  __pyx_L3:;

  /* "(tree fragment)":10
 *     else:
 *         use_setstate = self._boundary is not None or self._current_header_name is not None or self._current_header_value is not None or self._on_body_begin is not None or self._on_body_complete is not None or self._on_data is not None or self._on_header is not None or self._on_headers_complete is not None or self._on_part_begin is not None or self._on_part_complete is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Parser, (type(self), 0x7a58f64, None), state
 *     else:
 */
  __pyx_t_6 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":11
 *         use_setstate = self._boundary is not None or self._current_header_name is not None or self._current_header_value is not None or self._on_body_begin is not None or self._on_body_complete is not None or self._on_data is not None or self._on_header is not None or self._on_headers_complete is not None or self._on_part_begin is not None or self._on_part_complete is not None
 *     if use_setstate:
 *         return __pyx_unpickle_Parser, (type(self), 0x7a58f64, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_Parser, (type(self), 0x7a58f64, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_pyx_unpickle_Parser); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_128290660);
    __Pyx_GIVEREF(__pyx_int_128290660);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_128290660);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 11, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":10
 *     else:
 *         use_setstate = self._boundary is not None or self._current_header_name is not None or self._current_header_value is not None or self._on_body_begin is not None or self._on_body_complete is not None or self._on_data is not None or self._on_header is not None or self._on_headers_complete is not None or self._on_part_begin is not None or self._on_part_complete is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_Parser, (type(self), 0x7a58f64, None), state
 *     else:
 */
  }

  /* "(tree fragment)":13
 *         return __pyx_unpickle_Parser, (type(self), 0x7a58f64, None), state
 *     else:
 *         return __pyx_unpickle_Parser, (type(self), 0x7a58f64, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Parser__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_pyx_unpickle_Parser); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_128290660);
    __Pyx_GIVEREF(__pyx_int_128290660);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_128290660);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef bint use_setstate
 *     state = (self._boundary, self._boundary_index, self._boundary_length, self._current_header_name, self._current_header_value, self._on_body_begin, self._on_body_complete, self._on_data, self._on_header, self._on_headers_complete, self._on_part_begin, self._on_part_complete, self._state)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("multifruits.Parser.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle_Parser, (type(self), 0x7a58f64, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Parser__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_11multifruits_6Parser_15__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static PyObject *__pyx_pw_11multifruits_6Parser_15__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_11multifruits_6Parser_14__setstate_cython__(((struct __pyx_obj_11multifruits_Parser *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11multifruits_6Parser_14__setstate_cython__(struct __pyx_obj_11multifruits_Parser *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":15
 *         return __pyx_unpickle_Parser, (type(self), 0x7a58f64, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_Parser__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 15, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_11multifruits___pyx_unpickle_Parser__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":14
 *     else:
 *         return __pyx_unpickle_Parser, (type(self), 0x7a58f64, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_Parser__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("multifruits.Parser.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "multifruits.pyx":263
 * 
 * 
 * cpdef parse_content_disposition(bytes data):             # <<<<<<<<<<<<<<
 *     cdef:
 *         bytes dtype = None
 */

static PyObject *__pyx_pw_11multifruits_1parse_content_disposition(PyObject *__pyx_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_f_11multifruits_parse_content_disposition(PyObject *__pyx_v_data, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_v_dtype = 0;
  PyObject *__pyx_v_params = 0;
  unsigned int __pyx_v_length;
  unsigned int __pyx_v_start;
  unsigned int __pyx_v_end;
  unsigned int __pyx_v_i;
  PyBoolObject *__pyx_v_quoted = 0;
  char __pyx_v_c;
  PyObject *__pyx_v_previous = 0;
  PyObject *__pyx_v_field = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  int __pyx_t_3;
  char __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  __Pyx_RefNannySetupContext("parse_content_disposition", 0);

  /* "multifruits.pyx":265
 * cpdef parse_content_disposition(bytes data):
 *     cdef:
 *         bytes dtype = None             # <<<<<<<<<<<<<<
 *         dict params = {}
 *         unsigned int length = len(data)
 */
  __Pyx_INCREF(Py_None);
  __pyx_v_dtype = ((PyObject*)Py_None);

  /* "multifruits.pyx":266
 *     cdef:
 *         bytes dtype = None
 *         dict params = {}             # <<<<<<<<<<<<<<
 *         unsigned int length = len(data)
 *         unsigned int start = 0
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 266, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_params = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "multifruits.pyx":267
 *         bytes dtype = None
 *         dict params = {}
 *         unsigned int length = len(data)             # <<<<<<<<<<<<<<
 *         unsigned int start = 0
 *         unsigned int end = 0
 */
  if (unlikely(__pyx_v_data == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 267, __pyx_L1_error)
  }
  __pyx_t_2 = PyBytes_GET_SIZE(__pyx_v_data); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 267, __pyx_L1_error)
  __pyx_v_length = __pyx_t_2;

  /* "multifruits.pyx":268
 *         dict params = {}
 *         unsigned int length = len(data)
 *         unsigned int start = 0             # <<<<<<<<<<<<<<
 *         unsigned int end = 0
 *         unsigned int i = 0
 */
  __pyx_v_start = 0;

  /* "multifruits.pyx":269
 *         unsigned int length = len(data)
 *         unsigned int start = 0
 *         unsigned int end = 0             # <<<<<<<<<<<<<<
 *         unsigned int i = 0
 *         bool quoted = False
 */
  __pyx_v_end = 0;

  /* "multifruits.pyx":270
 *         unsigned int start = 0
 *         unsigned int end = 0
 *         unsigned int i = 0             # <<<<<<<<<<<<<<
 *         bool quoted = False
 *         char c
 */
  __pyx_v_i = 0;

  /* "multifruits.pyx":271
 *         unsigned int end = 0
 *         unsigned int i = 0
 *         bool quoted = False             # <<<<<<<<<<<<<<
 *         char c
 *         bytes previous = b''
 */
  __Pyx_INCREF(Py_False);
  __pyx_v_quoted = ((PyBoolObject *)Py_False);

  /* "multifruits.pyx":273
 *         bool quoted = False
 *         char c
 *         bytes previous = b''             # <<<<<<<<<<<<<<
 *         bytes field = None
 *     while i < length:
 */
  __Pyx_INCREF(__pyx_kp_b__19);
  __pyx_v_previous = __pyx_kp_b__19;

  /* "multifruits.pyx":274
 *         char c
 *         bytes previous = b''
 *         bytes field = None             # <<<<<<<<<<<<<<
 *     while i < length:
 *         c = data[i]
 */
  __Pyx_INCREF(Py_None);
  __pyx_v_field = ((PyObject*)Py_None);

  /* "multifruits.pyx":275
 *         bytes previous = b''
 *         bytes field = None
 *     while i < length:             # <<<<<<<<<<<<<<
 *         c = data[i]
 *         if not quoted and c == b';':
 */
  while (1) {
    __pyx_t_3 = ((__pyx_v_i < __pyx_v_length) != 0);
    if (!__pyx_t_3) break;

    /* "multifruits.pyx":276
 *         bytes field = None
 *     while i < length:
 *         c = data[i]             # <<<<<<<<<<<<<<
 *         if not quoted and c == b';':
 *             if dtype is None:
 */
    if (unlikely(__pyx_v_data == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 276, __pyx_L1_error)
    }
    __pyx_t_4 = __Pyx_PyBytes_GetItemInt(__pyx_v_data, __pyx_v_i, 1); if (unlikely(__pyx_t_4 == ((char)((char)-1)) && PyErr_Occurred())) __PYX_ERR(0, 276, __pyx_L1_error)
    __pyx_v_c = __pyx_t_4;

    /* "multifruits.pyx":277
 *     while i < length:
 *         c = data[i]
 *         if not quoted and c == b';':             # <<<<<<<<<<<<<<
 *             if dtype is None:
 *                 dtype = data[start:end]
 */
    __pyx_t_5 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_quoted)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 277, __pyx_L1_error)
    __pyx_t_6 = ((!__pyx_t_5) != 0);
    if (__pyx_t_6) {
    } else {
      __pyx_t_3 = __pyx_t_6;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_6 = ((__pyx_v_c == ';') != 0);
    __pyx_t_3 = __pyx_t_6;
    __pyx_L6_bool_binop_done:;
    if (__pyx_t_3) {

      /* "multifruits.pyx":278
 *         c = data[i]
 *         if not quoted and c == b';':
 *             if dtype is None:             # <<<<<<<<<<<<<<
 *                 dtype = data[start:end]
 *             elif field is not None:
 */
      __pyx_t_3 = (__pyx_v_dtype == ((PyObject*)Py_None));
      __pyx_t_6 = (__pyx_t_3 != 0);
      if (__pyx_t_6) {

        /* "multifruits.pyx":279
 *         if not quoted and c == b';':
 *             if dtype is None:
 *                 dtype = data[start:end]             # <<<<<<<<<<<<<<
 *             elif field is not None:
 *                 params[field.lower()] = data[start:end].replace(b'\\', b'')
 */
        if (unlikely(__pyx_v_data == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 279, __pyx_L1_error)
        }
        __pyx_t_1 = PySequence_GetSlice(__pyx_v_data, __pyx_v_start, __pyx_v_end); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 279, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF_SET(__pyx_v_dtype, ((PyObject*)__pyx_t_1));
        __pyx_t_1 = 0;

        /* "multifruits.pyx":278
 *         c = data[i]
 *         if not quoted and c == b';':
 *             if dtype is None:             # <<<<<<<<<<<<<<
 *                 dtype = data[start:end]
 *             elif field is not None:
 */
        goto __pyx_L8;
      }

      /* "multifruits.pyx":280
 *             if dtype is None:
 *                 dtype = data[start:end]
 *             elif field is not None:             # <<<<<<<<<<<<<<
 *                 params[field.lower()] = data[start:end].replace(b'\\', b'')
 *                 field = None
 */
      __pyx_t_6 = (__pyx_v_field != ((PyObject*)Py_None));
      __pyx_t_3 = (__pyx_t_6 != 0);
      if (__pyx_t_3) {

        /* "multifruits.pyx":281
 *                 dtype = data[start:end]
 *             elif field is not None:
 *                 params[field.lower()] = data[start:end].replace(b'\\', b'')             # <<<<<<<<<<<<<<
 *                 field = None
 *             i += 1
 */
        if (unlikely(__pyx_v_data == Py_None)) {
          PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
          __PYX_ERR(0, 281, __pyx_L1_error)
        }
        __pyx_t_1 = PySequence_GetSlice(__pyx_v_data, __pyx_v_start, __pyx_v_end); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 281, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_replace); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 281, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_tuple__21, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 281, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_field, __pyx_n_s_lower); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 281, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __pyx_t_9 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
          __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
          if (likely(__pyx_t_9)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
            __Pyx_INCREF(__pyx_t_9);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_8, function);
          }
        }
        if (__pyx_t_9) {
          __pyx_t_7 = __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_9); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 281, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        } else {
          __pyx_t_7 = __Pyx_PyObject_CallNoArg(__pyx_t_8); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 281, __pyx_L1_error)
        }
        __Pyx_GOTREF(__pyx_t_7);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        if (unlikely(PyDict_SetItem(__pyx_v_params, __pyx_t_7, __pyx_t_1) < 0)) __PYX_ERR(0, 281, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "multifruits.pyx":282
 *             elif field is not None:
 *                 params[field.lower()] = data[start:end].replace(b'\\', b'')
 *                 field = None             # <<<<<<<<<<<<<<
 *             i += 1
 *             start = end = i
 */
        __Pyx_INCREF(Py_None);
        __Pyx_DECREF_SET(__pyx_v_field, ((PyObject*)Py_None));

        /* "multifruits.pyx":280
 *             if dtype is None:
 *                 dtype = data[start:end]
 *             elif field is not None:             # <<<<<<<<<<<<<<
 *                 params[field.lower()] = data[start:end].replace(b'\\', b'')
 *                 field = None
 */
      }
      __pyx_L8:;

      /* "multifruits.pyx":283
 *                 params[field.lower()] = data[start:end].replace(b'\\', b'')
 *                 field = None
 *             i += 1             # <<<<<<<<<<<<<<
 *             start = end = i
 *         elif c == b'"':
 */
      __pyx_v_i = (__pyx_v_i + 1);

      /* "multifruits.pyx":284
 *                 field = None
 *             i += 1
 *             start = end = i             # <<<<<<<<<<<<<<
 *         elif c == b'"':
 *             i += 1
 */
      __pyx_v_start = __pyx_v_i;
      __pyx_v_end = __pyx_v_i;

      /* "multifruits.pyx":277
 *     while i < length:
 *         c = data[i]
 *         if not quoted and c == b';':             # <<<<<<<<<<<<<<
 *             if dtype is None:
 *                 dtype = data[start:end]
 */
      goto __pyx_L5;
    }

    /* "multifruits.pyx":285
 *             i += 1
 *             start = end = i
 *         elif c == b'"':             # <<<<<<<<<<<<<<
 *             i += 1
 *             if not previous or previous != b'\\':
 */
    __pyx_t_3 = ((__pyx_v_c == '"') != 0);
    if (__pyx_t_3) {

      /* "multifruits.pyx":286
 *             start = end = i
 *         elif c == b'"':
 *             i += 1             # <<<<<<<<<<<<<<
 *             if not previous or previous != b'\\':
 *                 if not quoted:
 */
      __pyx_v_i = (__pyx_v_i + 1);

      /* "multifruits.pyx":287
 *         elif c == b'"':
 *             i += 1
 *             if not previous or previous != b'\\':             # <<<<<<<<<<<<<<
 *                 if not quoted:
 *                     start = i
 */
      __pyx_t_6 = (__pyx_v_previous != Py_None) && (PyBytes_GET_SIZE(__pyx_v_previous) != 0);
      __pyx_t_5 = ((!__pyx_t_6) != 0);
      if (!__pyx_t_5) {
      } else {
        __pyx_t_3 = __pyx_t_5;
        goto __pyx_L10_bool_binop_done;
      }
      __pyx_t_5 = (__Pyx_PyBytes_Equals(__pyx_v_previous, __pyx_kp_b__20, Py_NE)); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 287, __pyx_L1_error)
      __pyx_t_6 = (__pyx_t_5 != 0);
      __pyx_t_3 = __pyx_t_6;
      __pyx_L10_bool_binop_done:;
      if (__pyx_t_3) {

        /* "multifruits.pyx":288
 *             i += 1
 *             if not previous or previous != b'\\':
 *                 if not quoted:             # <<<<<<<<<<<<<<
 *                     start = i
 *                 quoted = not quoted
 */
        __pyx_t_3 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_quoted)); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 288, __pyx_L1_error)
        __pyx_t_6 = ((!__pyx_t_3) != 0);
        if (__pyx_t_6) {

          /* "multifruits.pyx":289
 *             if not previous or previous != b'\\':
 *                 if not quoted:
 *                     start = i             # <<<<<<<<<<<<<<
 *                 quoted = not quoted
 *             else:
 */
          __pyx_v_start = __pyx_v_i;

          /* "multifruits.pyx":288
 *             i += 1
 *             if not previous or previous != b'\\':
 *                 if not quoted:             # <<<<<<<<<<<<<<
 *                     start = i
 *                 quoted = not quoted
 */
        }

        /* "multifruits.pyx":290
 *                 if not quoted:
 *                     start = i
 *                 quoted = not quoted             # <<<<<<<<<<<<<<
 *             else:
 *                 end = i
 */
        __pyx_t_6 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_quoted)); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 290, __pyx_L1_error)
        __pyx_t_1 = __Pyx_PyBool_FromLong((!__pyx_t_6)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 290, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        if (!(likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_7cpython_4bool_bool)))) __PYX_ERR(0, 290, __pyx_L1_error)
        __Pyx_DECREF_SET(__pyx_v_quoted, ((PyBoolObject *)__pyx_t_1));
        __pyx_t_1 = 0;

        /* "multifruits.pyx":287
 *         elif c == b'"':
 *             i += 1
 *             if not previous or previous != b'\\':             # <<<<<<<<<<<<<<
 *                 if not quoted:
 *                     start = i
 */
        goto __pyx_L9;
      }

      /* "multifruits.pyx":292
 *                 quoted = not quoted
 *             else:
 *                 end = i             # <<<<<<<<<<<<<<
 *         elif c == b'=':
 *             field = data[start:end]
 */
      /*else*/ {
        __pyx_v_end = __pyx_v_i;
      }
      __pyx_L9:;

      /* "multifruits.pyx":285
 *             i += 1
 *             start = end = i
 *         elif c == b'"':             # <<<<<<<<<<<<<<
 *             i += 1
 *             if not previous or previous != b'\\':
 */
      goto __pyx_L5;
    }

    /* "multifruits.pyx":293
 *             else:
 *                 end = i
 *         elif c == b'=':             # <<<<<<<<<<<<<<
 *             field = data[start:end]
 *             i += 1
 */
    __pyx_t_6 = ((__pyx_v_c == '=') != 0);
    if (__pyx_t_6) {

      /* "multifruits.pyx":294
 *                 end = i
 *         elif c == b'=':
 *             field = data[start:end]             # <<<<<<<<<<<<<<
 *             i += 1
 *             start = end = i
 */
      if (unlikely(__pyx_v_data == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 294, __pyx_L1_error)
      }
      __pyx_t_1 = PySequence_GetSlice(__pyx_v_data, __pyx_v_start, __pyx_v_end); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 294, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF_SET(__pyx_v_field, ((PyObject*)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "multifruits.pyx":295
 *         elif c == b'=':
 *             field = data[start:end]
 *             i += 1             # <<<<<<<<<<<<<<
 *             start = end = i
 *         elif c == b' ':
 */
      __pyx_v_i = (__pyx_v_i + 1);

      /* "multifruits.pyx":296
 *             field = data[start:end]
 *             i += 1
 *             start = end = i             # <<<<<<<<<<<<<<
 *         elif c == b' ':
 *             i += 1
 */
      __pyx_v_start = __pyx_v_i;
      __pyx_v_end = __pyx_v_i;

      /* "multifruits.pyx":293
 *             else:
 *                 end = i
 *         elif c == b'=':             # <<<<<<<<<<<<<<
 *             field = data[start:end]
 *             i += 1
 */
      goto __pyx_L5;
    }

    /* "multifruits.pyx":297
 *             i += 1
 *             start = end = i
 *         elif c == b' ':             # <<<<<<<<<<<<<<
 *             i += 1
 *             if not quoted and start == end:  # Leading spaces.
 */
    __pyx_t_6 = ((__pyx_v_c == ' ') != 0);
    if (__pyx_t_6) {

      /* "multifruits.pyx":298
 *             start = end = i
 *         elif c == b' ':
 *             i += 1             # <<<<<<<<<<<<<<
 *             if not quoted and start == end:  # Leading spaces.
 *                 start = end = i
 */
      __pyx_v_i = (__pyx_v_i + 1);

      /* "multifruits.pyx":299
 *         elif c == b' ':
 *             i += 1
 *             if not quoted and start == end:  # Leading spaces.             # <<<<<<<<<<<<<<
 *                 start = end = i
 *         else:
 */
      __pyx_t_3 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_quoted)); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 299, __pyx_L1_error)
      __pyx_t_5 = ((!__pyx_t_3) != 0);
      if (__pyx_t_5) {
      } else {
        __pyx_t_6 = __pyx_t_5;
        goto __pyx_L14_bool_binop_done;
      }
      __pyx_t_5 = ((__pyx_v_start == __pyx_v_end) != 0);
      __pyx_t_6 = __pyx_t_5;
      __pyx_L14_bool_binop_done:;
      if (__pyx_t_6) {

        /* "multifruits.pyx":300
 *             i += 1
 *             if not quoted and start == end:  # Leading spaces.
 *                 start = end = i             # <<<<<<<<<<<<<<
 *         else:
 *             i += 1
 */
        __pyx_v_start = __pyx_v_i;
        __pyx_v_end = __pyx_v_i;

        /* "multifruits.pyx":299
 *         elif c == b' ':
 *             i += 1
 *             if not quoted and start == end:  # Leading spaces.             # <<<<<<<<<<<<<<
 *                 start = end = i
 *         else:
 */
      }

      /* "multifruits.pyx":297
 *             i += 1
 *             start = end = i
 *         elif c == b' ':             # <<<<<<<<<<<<<<
 *             i += 1
 *             if not quoted and start == end:  # Leading spaces.
 */
      goto __pyx_L5;
    }

    /* "multifruits.pyx":302
 *                 start = end = i
 *         else:
 *             i += 1             # <<<<<<<<<<<<<<
 *             end = i
 *         previous = c
 */
    /*else*/ {
      __pyx_v_i = (__pyx_v_i + 1);

      /* "multifruits.pyx":303
 *         else:
 *             i += 1
 *             end = i             # <<<<<<<<<<<<<<
 *         previous = c
 *     if i:
 */
      __pyx_v_end = __pyx_v_i;
    }
    __pyx_L5:;

    /* "multifruits.pyx":304
 *             i += 1
 *             end = i
 *         previous = c             # <<<<<<<<<<<<<<
 *     if i:
 *         if dtype is None:
 */
    __pyx_t_1 = PyBytes_FromStringAndSize(&__pyx_v_c, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 304, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_previous, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;
  }

  /* "multifruits.pyx":305
 *             end = i
 *         previous = c
 *     if i:             # <<<<<<<<<<<<<<
 *         if dtype is None:
 *             dtype = data[start:end].lower()
 */
  __pyx_t_6 = (__pyx_v_i != 0);
  if (__pyx_t_6) {

    /* "multifruits.pyx":306
 *         previous = c
 *     if i:
 *         if dtype is None:             # <<<<<<<<<<<<<<
 *             dtype = data[start:end].lower()
 *         elif field is not None:
 */
    __pyx_t_6 = (__pyx_v_dtype == ((PyObject*)Py_None));
    __pyx_t_5 = (__pyx_t_6 != 0);
    if (__pyx_t_5) {

      /* "multifruits.pyx":307
 *     if i:
 *         if dtype is None:
 *             dtype = data[start:end].lower()             # <<<<<<<<<<<<<<
 *         elif field is not None:
 *             params[field.lower()] = data[start:end].replace(b'\\', b'')
 */
      if (unlikely(__pyx_v_data == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 307, __pyx_L1_error)
      }
      __pyx_t_7 = PySequence_GetSlice(__pyx_v_data, __pyx_v_start, __pyx_v_end); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 307, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_lower); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 307, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_8);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_8, function);
        }
      }
      if (__pyx_t_7) {
        __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 307, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      } else {
        __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 307, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 307, __pyx_L1_error)
      __Pyx_DECREF_SET(__pyx_v_dtype, ((PyObject*)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "multifruits.pyx":306
 *         previous = c
 *     if i:
 *         if dtype is None:             # <<<<<<<<<<<<<<
 *             dtype = data[start:end].lower()
 *         elif field is not None:
 */
      goto __pyx_L17;
    }

    /* "multifruits.pyx":308
 *         if dtype is None:
 *             dtype = data[start:end].lower()
 *         elif field is not None:             # <<<<<<<<<<<<<<
 *             params[field.lower()] = data[start:end].replace(b'\\', b'')
 *     return dtype, params
 */
    __pyx_t_5 = (__pyx_v_field != ((PyObject*)Py_None));
    __pyx_t_6 = (__pyx_t_5 != 0);
    if (__pyx_t_6) {

      /* "multifruits.pyx":309
 *             dtype = data[start:end].lower()
 *         elif field is not None:
 *             params[field.lower()] = data[start:end].replace(b'\\', b'')             # <<<<<<<<<<<<<<
 *     return dtype, params
 * 
 */
      if (unlikely(__pyx_v_data == Py_None)) {
        PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
        __PYX_ERR(0, 309, __pyx_L1_error)
      }
      __pyx_t_1 = PySequence_GetSlice(__pyx_v_data, __pyx_v_start, __pyx_v_end); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 309, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_replace); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 309, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_8, __pyx_tuple__22, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 309, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_field, __pyx_n_s_lower); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 309, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_9 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_9)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_9);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
        }
      }
      if (__pyx_t_9) {
        __pyx_t_8 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_9); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 309, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      } else {
        __pyx_t_8 = __Pyx_PyObject_CallNoArg(__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 309, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_8);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      if (unlikely(PyDict_SetItem(__pyx_v_params, __pyx_t_8, __pyx_t_1) < 0)) __PYX_ERR(0, 309, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "multifruits.pyx":308
 *         if dtype is None:
 *             dtype = data[start:end].lower()
 *         elif field is not None:             # <<<<<<<<<<<<<<
 *             params[field.lower()] = data[start:end].replace(b'\\', b'')
 *     return dtype, params
 */
    }
    __pyx_L17:;

    /* "multifruits.pyx":305
 *             end = i
 *         previous = c
 *     if i:             # <<<<<<<<<<<<<<
 *         if dtype is None:
 *             dtype = data[start:end].lower()
 */
  }

  /* "multifruits.pyx":310
 *         elif field is not None:
 *             params[field.lower()] = data[start:end].replace(b'\\', b'')
 *     return dtype, params             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 310, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_dtype);
  __Pyx_GIVEREF(__pyx_v_dtype);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_dtype);
  __Pyx_INCREF(__pyx_v_params);
  __Pyx_GIVEREF(__pyx_v_params);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_params);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "multifruits.pyx":263
 * 
 * 
 * cpdef parse_content_disposition(bytes data):             # <<<<<<<<<<<<<<
 *     cdef:
 *         bytes dtype = None
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("multifruits.parse_content_disposition", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_params);
  __Pyx_XDECREF((PyObject *)__pyx_v_quoted);
  __Pyx_XDECREF(__pyx_v_previous);
  __Pyx_XDECREF(__pyx_v_field);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_11multifruits_1parse_content_disposition(PyObject *__pyx_self, PyObject *__pyx_v_data); /*proto*/
static PyObject *__pyx_pw_11multifruits_1parse_content_disposition(PyObject *__pyx_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("parse_content_disposition (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_data), (&PyBytes_Type), 1, "data", 1))) __PYX_ERR(0, 263, __pyx_L1_error)
  __pyx_r = __pyx_pf_11multifruits_parse_content_disposition(__pyx_self, ((PyObject*)__pyx_v_data));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11multifruits_parse_content_disposition(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_data) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("parse_content_disposition", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_11multifruits_parse_content_disposition(__pyx_v_data, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("multifruits.parse_content_disposition", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "multifruits.pyx":313
 * 
 * 
 * def extract_filename(params: dict):             # <<<<<<<<<<<<<<
 *     if b'filename*' in params:
 *         filename = params.get(b'filename*')
 */

/* Python wrapper */
static PyObject *__pyx_pw_11multifruits_3extract_filename(PyObject *__pyx_self, PyObject *__pyx_v_params); /*proto*/
static PyMethodDef __pyx_mdef_11multifruits_3extract_filename = {"extract_filename", (PyCFunction)__pyx_pw_11multifruits_3extract_filename, METH_O, 0};
static PyObject *__pyx_pw_11multifruits_3extract_filename(PyObject *__pyx_self, PyObject *__pyx_v_params) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("extract_filename (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_params), (&PyDict_Type), 1, "params", 1))) __PYX_ERR(0, 313, __pyx_L1_error)
  __pyx_r = __pyx_pf_11multifruits_2extract_filename(__pyx_self, ((PyObject*)__pyx_v_params));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11multifruits_2extract_filename(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_params) {
  PyObject *__pyx_v_filename = NULL;
  PyObject *__pyx_v_encoding = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *(*__pyx_t_7)(PyObject *);
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  __Pyx_RefNannySetupContext("extract_filename", 0);

  /* "multifruits.pyx":314
 * 
 * def extract_filename(params: dict):
 *     if b'filename*' in params:             # <<<<<<<<<<<<<<
 *         filename = params.get(b'filename*')
 *         if b"''" in filename:
 */
  if (unlikely(__pyx_v_params == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
    __PYX_ERR(0, 314, __pyx_L1_error)
  }
  __pyx_t_1 = (__Pyx_PyDict_ContainsTF(__pyx_kp_b_filename, __pyx_v_params, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 314, __pyx_L1_error)
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "multifruits.pyx":315
 * def extract_filename(params: dict):
 *     if b'filename*' in params:
 *         filename = params.get(b'filename*')             # <<<<<<<<<<<<<<
 *         if b"''" in filename:
 *             encoding, filename = filename.split(b"''")
 */
    if (unlikely(__pyx_v_params == Py_None)) {
      PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "get");
      __PYX_ERR(0, 315, __pyx_L1_error)
    }
    __pyx_t_3 = __Pyx_PyDict_GetItemDefault(__pyx_v_params, __pyx_kp_b_filename, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 315, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_filename = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "multifruits.pyx":316
 *     if b'filename*' in params:
 *         filename = params.get(b'filename*')
 *         if b"''" in filename:             # <<<<<<<<<<<<<<
 *             encoding, filename = filename.split(b"''")
 *             try:
 */
    __pyx_t_2 = (__Pyx_PySequence_ContainsTF(__pyx_kp_b__23, __pyx_v_filename, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 316, __pyx_L1_error)
    __pyx_t_1 = (__pyx_t_2 != 0);
    if (__pyx_t_1) {

      /* "multifruits.pyx":317
 *         filename = params.get(b'filename*')
 *         if b"''" in filename:
 *             encoding, filename = filename.split(b"''")             # <<<<<<<<<<<<<<
 *             try:
 *                 return filename.decode(encoding.decode())
 */
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_filename, __pyx_n_s_split); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 317, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_tuple__24, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 317, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      if ((likely(PyTuple_CheckExact(__pyx_t_4))) || (PyList_CheckExact(__pyx_t_4))) {
        PyObject* sequence = __pyx_t_4;
        #if !CYTHON_COMPILING_IN_PYPY
        Py_ssize_t size = Py_SIZE(sequence);
        #else
        Py_ssize_t size = PySequence_Size(sequence);
        #endif
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 317, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_5 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_3 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_5 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        #else
        __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 317, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_5 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 317, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        #endif
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_6 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 317, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_7 = Py_TYPE(__pyx_t_6)->tp_iternext;
        index = 0; __pyx_t_3 = __pyx_t_7(__pyx_t_6); if (unlikely(!__pyx_t_3)) goto __pyx_L5_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_3);
        index = 1; __pyx_t_5 = __pyx_t_7(__pyx_t_6); if (unlikely(!__pyx_t_5)) goto __pyx_L5_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_5);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_7(__pyx_t_6), 2) < 0) __PYX_ERR(0, 317, __pyx_L1_error)
        __pyx_t_7 = NULL;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        goto __pyx_L6_unpacking_done;
        __pyx_L5_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __pyx_t_7 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 317, __pyx_L1_error)
        __pyx_L6_unpacking_done:;
      }
      __pyx_v_encoding = __pyx_t_3;
      __pyx_t_3 = 0;
      __Pyx_DECREF_SET(__pyx_v_filename, __pyx_t_5);
      __pyx_t_5 = 0;

      /* "multifruits.pyx":318
 *         if b"''" in filename:
 *             encoding, filename = filename.split(b"''")
 *             try:             # <<<<<<<<<<<<<<
 *                 return filename.decode(encoding.decode())
 *             except (LookupError, UnicodeDecodeError):
 */
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_8, &__pyx_t_9, &__pyx_t_10);
        __Pyx_XGOTREF(__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_9);
        __Pyx_XGOTREF(__pyx_t_10);
        /*try:*/ {

          /* "multifruits.pyx":319
 *             encoding, filename = filename.split(b"''")
 *             try:
 *                 return filename.decode(encoding.decode())             # <<<<<<<<<<<<<<
 *             except (LookupError, UnicodeDecodeError):
 *                 pass
 */
          __Pyx_XDECREF(__pyx_r);
          __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_filename, __pyx_n_s_decode); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 319, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_5);
          __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_encoding, __pyx_n_s_decode); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 319, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_6);
          __pyx_t_11 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
            __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_6);
            if (likely(__pyx_t_11)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
              __Pyx_INCREF(__pyx_t_11);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_6, function);
            }
          }
          if (__pyx_t_11) {
            __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_t_11); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 319, __pyx_L7_error)
            __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
          } else {
            __pyx_t_3 = __Pyx_PyObject_CallNoArg(__pyx_t_6); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 319, __pyx_L7_error)
          }
          __Pyx_GOTREF(__pyx_t_3);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
          __pyx_t_6 = NULL;
          if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
            __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
            if (likely(__pyx_t_6)) {
              PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
              __Pyx_INCREF(__pyx_t_6);
              __Pyx_INCREF(function);
              __Pyx_DECREF_SET(__pyx_t_5, function);
            }
          }
          if (!__pyx_t_6) {
            __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 319, __pyx_L7_error)
            __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            __Pyx_GOTREF(__pyx_t_4);
          } else {
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(__pyx_t_5)) {
              PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_3};
              __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 319, __pyx_L7_error)
              __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
              __Pyx_GOTREF(__pyx_t_4);
              __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            } else
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
              PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_3};
              __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 319, __pyx_L7_error)
              __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
              __Pyx_GOTREF(__pyx_t_4);
              __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
            } else
            #endif
            {
              __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 319, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_11);
              __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_6); __pyx_t_6 = NULL;
              __Pyx_GIVEREF(__pyx_t_3);
              PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_3);
              __pyx_t_3 = 0;
              __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_11, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 319, __pyx_L7_error)
              __Pyx_GOTREF(__pyx_t_4);
              __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
            }
          }
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __pyx_r = __pyx_t_4;
          __pyx_t_4 = 0;
          goto __pyx_L11_try_return;

          /* "multifruits.pyx":318
 *         if b"''" in filename:
 *             encoding, filename = filename.split(b"''")
 *             try:             # <<<<<<<<<<<<<<
 *                 return filename.decode(encoding.decode())
 *             except (LookupError, UnicodeDecodeError):
 */
        }
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;

        /* "multifruits.pyx":320
 *             try:
 *                 return filename.decode(encoding.decode())
 *             except (LookupError, UnicodeDecodeError):             # <<<<<<<<<<<<<<
 *                 pass
 *         if b'filename' in params:
 */
        __pyx_t_12 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_LookupError) || __Pyx_PyErr_ExceptionMatches(__pyx_builtin_UnicodeDecodeError);
        if (__pyx_t_12) {
          __Pyx_ErrRestore(0,0,0);
          goto __pyx_L8_exception_handled;
        }
        goto __pyx_L9_except_error;
        __pyx_L9_except_error:;

        /* "multifruits.pyx":318
 *         if b"''" in filename:
 *             encoding, filename = filename.split(b"''")
 *             try:             # <<<<<<<<<<<<<<
 *                 return filename.decode(encoding.decode())
 *             except (LookupError, UnicodeDecodeError):
 */
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_ExceptionReset(__pyx_t_8, __pyx_t_9, __pyx_t_10);
        goto __pyx_L1_error;
        __pyx_L11_try_return:;
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_ExceptionReset(__pyx_t_8, __pyx_t_9, __pyx_t_10);
        goto __pyx_L0;
        __pyx_L8_exception_handled:;
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_XGIVEREF(__pyx_t_10);
        __Pyx_ExceptionReset(__pyx_t_8, __pyx_t_9, __pyx_t_10);
      }

      /* "multifruits.pyx":316
 *     if b'filename*' in params:
 *         filename = params.get(b'filename*')
 *         if b"''" in filename:             # <<<<<<<<<<<<<<
 *             encoding, filename = filename.split(b"''")
 *             try:
 */
    }

    /* "multifruits.pyx":322
 *             except (LookupError, UnicodeDecodeError):
 *                 pass
 *         if b'filename' in params:             # <<<<<<<<<<<<<<
 *             return params.get(b'filename').decode()
 *         return filename.decode(errors='ignore')
 */
    if (unlikely(__pyx_v_params == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      __PYX_ERR(0, 322, __pyx_L1_error)
    }
    __pyx_t_1 = (__Pyx_PyDict_ContainsTF(__pyx_n_b_filename_2, __pyx_v_params, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 322, __pyx_L1_error)
    __pyx_t_2 = (__pyx_t_1 != 0);
    if (__pyx_t_2) {

      /* "multifruits.pyx":323
 *                 pass
 *         if b'filename' in params:
 *             return params.get(b'filename').decode()             # <<<<<<<<<<<<<<
 *         return filename.decode(errors='ignore')
 *     return params.get(b'filename').decode()
 */
      __Pyx_XDECREF(__pyx_r);
      if (unlikely(__pyx_v_params == Py_None)) {
        PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "get");
        __PYX_ERR(0, 323, __pyx_L1_error)
      }
      __pyx_t_5 = __Pyx_PyDict_GetItemDefault(__pyx_v_params, __pyx_n_b_filename_2, Py_None); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 323, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_decode); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 323, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_11);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_11))) {
        __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_11);
        if (likely(__pyx_t_5)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_11);
          __Pyx_INCREF(__pyx_t_5);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_11, function);
        }
      }
      if (__pyx_t_5) {
        __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_11, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 323, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      } else {
        __pyx_t_4 = __Pyx_PyObject_CallNoArg(__pyx_t_11); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 323, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __pyx_r = __pyx_t_4;
      __pyx_t_4 = 0;
      goto __pyx_L0;

      /* "multifruits.pyx":322
 *             except (LookupError, UnicodeDecodeError):
 *                 pass
 *         if b'filename' in params:             # <<<<<<<<<<<<<<
 *             return params.get(b'filename').decode()
 *         return filename.decode(errors='ignore')
 */
    }

    /* "multifruits.pyx":324
 *         if b'filename' in params:
 *             return params.get(b'filename').decode()
 *         return filename.decode(errors='ignore')             # <<<<<<<<<<<<<<
 *     return params.get(b'filename').decode()
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_filename, __pyx_n_s_decode); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 324, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_11 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 324, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    if (PyDict_SetItem(__pyx_t_11, __pyx_n_s_errors, __pyx_n_u_ignore) < 0) __PYX_ERR(0, 324, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_empty_tuple, __pyx_t_11); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 324, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "multifruits.pyx":314
 * 
 * def extract_filename(params: dict):
 *     if b'filename*' in params:             # <<<<<<<<<<<<<<
 *         filename = params.get(b'filename*')
 *         if b"''" in filename:
 */
  }

  /* "multifruits.pyx":325
 *             return params.get(b'filename').decode()
 *         return filename.decode(errors='ignore')
 *     return params.get(b'filename').decode()             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  if (unlikely(__pyx_v_params == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "get");
    __PYX_ERR(0, 325, __pyx_L1_error)
  }
  __pyx_t_11 = __Pyx_PyDict_GetItemDefault(__pyx_v_params, __pyx_n_b_filename_2, Py_None); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 325, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_11);
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_11, __pyx_n_s_decode); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 325, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  __pyx_t_11 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_11)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_11);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  if (__pyx_t_11) {
    __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_11); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 325, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
  } else {
    __pyx_t_5 = __Pyx_PyObject_CallNoArg(__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 325, __pyx_L1_error)
  }
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "multifruits.pyx":313
 * 
 * 
 * def extract_filename(params: dict):             # <<<<<<<<<<<<<<
 *     if b'filename*' in params:
 *         filename = params.get(b'filename*')
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("multifruits.extract_filename", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_filename);
  __Pyx_XDECREF(__pyx_v_encoding);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_Parser(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x7a58f64:
 *         from pickle import PickleError as __pyx_PickleError
 */

/* Python wrapper */
static PyObject *__pyx_pw_11multifruits_5__pyx_unpickle_Parser(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_11multifruits_5__pyx_unpickle_Parser = {"__pyx_unpickle_Parser", (PyCFunction)__pyx_pw_11multifruits_5__pyx_unpickle_Parser, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_11multifruits_5__pyx_unpickle_Parser(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_Parser (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Parser", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Parser", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_Parser") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_Parser", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("multifruits.__pyx_unpickle_Parser", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_11multifruits_4__pyx_unpickle_Parser(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_11multifruits_4__pyx_unpickle_Parser(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = NULL;
  PyObject *__pyx_v___pyx_result = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("__pyx_unpickle_Parser", 0);

  /* "(tree fragment)":2
 * def __pyx_unpickle_Parser(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0x7a58f64:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7a58f64 = (_boundary, _boundary_index, _boundary_length, _current_header_name, _current_header_value, _on_body_begin, _on_body_complete, _on_data, _on_header, _on_headers_complete, _on_part_begin, _on_part_complete, _state))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x7a58f64) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":3
 * def __pyx_unpickle_Parser(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0x7a58f64:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7a58f64 = (_boundary, _boundary_index, _boundary_length, _current_header_name, _current_header_value, _on_body_begin, _on_body_complete, _on_data, _on_header, _on_headers_complete, _on_part_begin, _on_part_complete, _state))" % __pyx_checksum)
 *     __pyx_result = Parser.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":4
 *     if __pyx_checksum != 0x7a58f64:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7a58f64 = (_boundary, _boundary_index, _boundary_length, _current_header_name, _current_header_value, _on_body_begin, _on_body_complete, _on_data, _on_header, _on_headers_complete, _on_part_begin, _on_part_complete, _state))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = Parser.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x7a, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 4, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 4, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_4);
        __pyx_t_4 = 0;
        __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 4, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 4, __pyx_L1_error)

    /* "(tree fragment)":2
 * def __pyx_unpickle_Parser(__pyx_type, long __pyx_checksum, __pyx_state):
 *     if __pyx_checksum != 0x7a58f64:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7a58f64 = (_boundary, _boundary_index, _boundary_length, _current_header_name, _current_header_value, _on_body_begin, _on_body_complete, _on_data, _on_header, _on_headers_complete, _on_part_begin, _on_part_complete, _state))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":5
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7a58f64 = (_boundary, _boundary_index, _boundary_length, _current_header_name, _current_header_value, _on_body_begin, _on_body_complete, _on_data, _on_header, _on_headers_complete, _on_part_begin, _on_part_complete, _state))" % __pyx_checksum)
 *     __pyx_result = Parser.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Parser__set_state(<Parser> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_11multifruits_Parser), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_6) {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v___pyx_type};
      __pyx_t_3 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_v___pyx_type};
      __pyx_t_3 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_GOTREF(__pyx_t_3);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_6); __pyx_t_6 = NULL;
      __Pyx_INCREF(__pyx_v___pyx_type);
      __Pyx_GIVEREF(__pyx_v___pyx_type);
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v___pyx_type);
      __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7a58f64 = (_boundary, _boundary_index, _boundary_length, _current_header_name, _current_header_value, _on_body_begin, _on_body_complete, _on_data, _on_header, _on_headers_complete, _on_part_begin, _on_part_complete, _state))" % __pyx_checksum)
 *     __pyx_result = Parser.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Parser__set_state(<Parser> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_7 = (__pyx_t_1 != 0);
  if (__pyx_t_7) {

    /* "(tree fragment)":7
 *     __pyx_result = Parser.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Parser__set_state(<Parser> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_Parser__set_state(Parser __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 7, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_11multifruits___pyx_unpickle_Parser__set_state(((struct __pyx_obj_11multifruits_Parser *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x7a58f64 = (_boundary, _boundary_index, _boundary_length, _current_header_name, _current_header_value, _on_body_begin, _on_body_complete, _on_data, _on_header, _on_headers_complete, _on_part_begin, _on_part_complete, _state))" % __pyx_checksum)
 *     __pyx_result = Parser.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_Parser__set_state(<Parser> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":8
 *     if __pyx_state is not None:
 *         __pyx_unpickle_Parser__set_state(<Parser> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_Parser__set_state(Parser __pyx_result, tuple __pyx_state):
 *     __pyx_result._boundary = __pyx_state[0]; __pyx_result._boundary_index = __pyx_state[1]; __pyx_result._boundary_length = __pyx_state[2]; __pyx_result._current_header_name = __pyx_state[3]; __pyx_result._current_header_value = __pyx_state[4]; __pyx_result._on_body_begin = __pyx_state[5]; __pyx_result._on_body_complete = __pyx_state[6]; __pyx_result._on_data = __pyx_state[7]; __pyx_result._on_header = __pyx_state[8]; __pyx_result._on_headers_complete = __pyx_state[9]; __pyx_result._on_part_begin = __pyx_state[10]; __pyx_result._on_part_complete = __pyx_state[11]; __pyx_result._state = __pyx_state[12]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_Parser(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x7a58f64:
 *         from pickle import PickleError as __pyx_PickleError
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("multifruits.__pyx_unpickle_Parser", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":9
 *         __pyx_unpickle_Parser__set_state(<Parser> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Parser__set_state(Parser __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._boundary = __pyx_state[0]; __pyx_result._boundary_index = __pyx_state[1]; __pyx_result._boundary_length = __pyx_state[2]; __pyx_result._current_header_name = __pyx_state[3]; __pyx_result._current_header_value = __pyx_state[4]; __pyx_result._on_body_begin = __pyx_state[5]; __pyx_result._on_body_complete = __pyx_state[6]; __pyx_result._on_data = __pyx_state[7]; __pyx_result._on_header = __pyx_state[8]; __pyx_result._on_headers_complete = __pyx_state[9]; __pyx_result._on_part_begin = __pyx_state[10]; __pyx_result._on_part_complete = __pyx_state[11]; __pyx_result._state = __pyx_state[12]
 *     if len(__pyx_state) > 13 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_11multifruits___pyx_unpickle_Parser__set_state(struct __pyx_obj_11multifruits_Parser *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  unsigned int __pyx_t_2;
  unsigned char __pyx_t_3;
  int __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  __Pyx_RefNannySetupContext("__pyx_unpickle_Parser__set_state", 0);

  /* "(tree fragment)":10
 *     return __pyx_result
 * cdef __pyx_unpickle_Parser__set_state(Parser __pyx_result, tuple __pyx_state):
 *     __pyx_result._boundary = __pyx_state[0]; __pyx_result._boundary_index = __pyx_state[1]; __pyx_result._boundary_length = __pyx_state[2]; __pyx_result._current_header_name = __pyx_state[3]; __pyx_result._current_header_value = __pyx_state[4]; __pyx_result._on_body_begin = __pyx_state[5]; __pyx_result._on_body_complete = __pyx_state[6]; __pyx_result._on_data = __pyx_state[7]; __pyx_result._on_header = __pyx_state[8]; __pyx_result._on_headers_complete = __pyx_state[9]; __pyx_result._on_part_begin = __pyx_state[10]; __pyx_result._on_part_complete = __pyx_state[11]; __pyx_result._state = __pyx_state[12]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 13 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[13])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_boundary);
  __Pyx_DECREF(__pyx_v___pyx_result->_boundary);
  __pyx_v___pyx_result->_boundary = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_unsigned_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_boundary_index = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_As_unsigned_int(__pyx_t_1); if (unlikely((__pyx_t_2 == (unsigned int)-1) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_boundary_length = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_current_header_name);
  __Pyx_DECREF(__pyx_v___pyx_result->_current_header_name);
  __pyx_v___pyx_result->_current_header_name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "bytes", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_current_header_value);
  __Pyx_DECREF(__pyx_v___pyx_result->_current_header_value);
  __pyx_v___pyx_result->_current_header_value = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_on_body_begin);
  __Pyx_DECREF(__pyx_v___pyx_result->_on_body_begin);
  __pyx_v___pyx_result->_on_body_begin = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_on_body_complete);
  __Pyx_DECREF(__pyx_v___pyx_result->_on_body_complete);
  __pyx_v___pyx_result->_on_body_complete = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_on_data);
  __Pyx_DECREF(__pyx_v___pyx_result->_on_data);
  __pyx_v___pyx_result->_on_data = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 8, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_on_header);
  __Pyx_DECREF(__pyx_v___pyx_result->_on_header);
  __pyx_v___pyx_result->_on_header = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 9, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_on_headers_complete);
  __Pyx_DECREF(__pyx_v___pyx_result->_on_headers_complete);
  __pyx_v___pyx_result->_on_headers_complete = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 10, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_on_part_begin);
  __Pyx_DECREF(__pyx_v___pyx_result->_on_part_begin);
  __pyx_v___pyx_result->_on_part_begin = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 11, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_on_part_complete);
  __Pyx_DECREF(__pyx_v___pyx_result->_on_part_complete);
  __pyx_v___pyx_result->_on_part_complete = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 10, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 12, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_As_unsigned_char(__pyx_t_1); if (unlikely((__pyx_t_3 == (unsigned char)-1) && PyErr_Occurred())) __PYX_ERR(1, 10, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_state = __pyx_t_3;

  /* "(tree fragment)":11
 * cdef __pyx_unpickle_Parser__set_state(Parser __pyx_result, tuple __pyx_state):
 *     __pyx_result._boundary = __pyx_state[0]; __pyx_result._boundary_index = __pyx_state[1]; __pyx_result._boundary_length = __pyx_state[2]; __pyx_result._current_header_name = __pyx_state[3]; __pyx_result._current_header_value = __pyx_state[4]; __pyx_result._on_body_begin = __pyx_state[5]; __pyx_result._on_body_complete = __pyx_state[6]; __pyx_result._on_data = __pyx_state[7]; __pyx_result._on_header = __pyx_state[8]; __pyx_result._on_headers_complete = __pyx_state[9]; __pyx_result._on_part_begin = __pyx_state[10]; __pyx_result._on_part_complete = __pyx_state[11]; __pyx_result._state = __pyx_state[12]
 *     if len(__pyx_state) > 13 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[13])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 11, __pyx_L1_error)
  }
  __pyx_t_5 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(1, 11, __pyx_L1_error)
  __pyx_t_6 = ((__pyx_t_5 > 13) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_4 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(1, 11, __pyx_L1_error)
  __pyx_t_7 = (__pyx_t_6 != 0);
  __pyx_t_4 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_4) {

    /* "(tree fragment)":12
 *     __pyx_result._boundary = __pyx_state[0]; __pyx_result._boundary_index = __pyx_state[1]; __pyx_result._boundary_length = __pyx_state[2]; __pyx_result._current_header_name = __pyx_state[3]; __pyx_result._current_header_value = __pyx_state[4]; __pyx_result._on_body_begin = __pyx_state[5]; __pyx_result._on_body_complete = __pyx_state[6]; __pyx_result._on_data = __pyx_state[7]; __pyx_result._on_header = __pyx_state[8]; __pyx_result._on_headers_complete = __pyx_state[9]; __pyx_result._on_part_begin = __pyx_state[10]; __pyx_result._on_part_complete = __pyx_state[11]; __pyx_result._state = __pyx_state[12]
 *     if len(__pyx_state) > 13 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[13])             # <<<<<<<<<<<<<<
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_update); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 12, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 13, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 12, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    if (!__pyx_t_10) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_9)) {
        PyObject *__pyx_temp[2] = {__pyx_t_10, __pyx_t_8};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_9, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_9)) {
        PyObject *__pyx_temp[2] = {__pyx_t_10, __pyx_t_8};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_9, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      } else
      #endif
      {
        __pyx_t_11 = PyTuple_New(1+1); if (unlikely(!__pyx_t_11)) __PYX_ERR(1, 12, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_GIVEREF(__pyx_t_10); PyTuple_SET_ITEM(__pyx_t_11, 0, __pyx_t_10); __pyx_t_10 = NULL;
        __Pyx_GIVEREF(__pyx_t_8);
        PyTuple_SET_ITEM(__pyx_t_11, 0+1, __pyx_t_8);
        __pyx_t_8 = 0;
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_9, __pyx_t_11, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":11
 * cdef __pyx_unpickle_Parser__set_state(Parser __pyx_result, tuple __pyx_state):
 *     __pyx_result._boundary = __pyx_state[0]; __pyx_result._boundary_index = __pyx_state[1]; __pyx_result._boundary_length = __pyx_state[2]; __pyx_result._current_header_name = __pyx_state[3]; __pyx_result._current_header_value = __pyx_state[4]; __pyx_result._on_body_begin = __pyx_state[5]; __pyx_result._on_body_complete = __pyx_state[6]; __pyx_result._on_data = __pyx_state[7]; __pyx_result._on_header = __pyx_state[8]; __pyx_result._on_headers_complete = __pyx_state[9]; __pyx_result._on_part_begin = __pyx_state[10]; __pyx_result._on_part_complete = __pyx_state[11]; __pyx_result._state = __pyx_state[12]
 *     if len(__pyx_state) > 13 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[13])
 */
  }

  /* "(tree fragment)":9
 *         __pyx_unpickle_Parser__set_state(<Parser> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_Parser__set_state(Parser __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._boundary = __pyx_state[0]; __pyx_result._boundary_index = __pyx_state[1]; __pyx_result._boundary_length = __pyx_state[2]; __pyx_result._current_header_name = __pyx_state[3]; __pyx_result._current_header_value = __pyx_state[4]; __pyx_result._on_body_begin = __pyx_state[5]; __pyx_result._on_body_complete = __pyx_state[6]; __pyx_result._on_data = __pyx_state[7]; __pyx_result._on_header = __pyx_state[8]; __pyx_result._on_headers_complete = __pyx_state[9]; __pyx_result._on_part_begin = __pyx_state[10]; __pyx_result._on_part_complete = __pyx_state[11]; __pyx_result._state = __pyx_state[12]
 *     if len(__pyx_state) > 13 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("multifruits.__pyx_unpickle_Parser__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static struct __pyx_vtabstruct_11multifruits_Parser __pyx_vtable_11multifruits_Parser;

static PyObject *__pyx_tp_new_11multifruits_Parser(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_11multifruits_Parser *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_11multifruits_Parser *)o);
  p->__pyx_vtab = __pyx_vtabptr_11multifruits_Parser;
  p->_boundary = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_current_header_name = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_current_header_value = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_on_body_begin = Py_None; Py_INCREF(Py_None);
  p->_on_part_begin = Py_None; Py_INCREF(Py_None);
  p->_on_header = Py_None; Py_INCREF(Py_None);
  p->_on_headers_complete = Py_None; Py_INCREF(Py_None);
  p->_on_data = Py_None; Py_INCREF(Py_None);
  p->_on_part_complete = Py_None; Py_INCREF(Py_None);
  p->_on_body_complete = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_11multifruits_Parser(PyObject *o) {
  struct __pyx_obj_11multifruits_Parser *p = (struct __pyx_obj_11multifruits_Parser *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_boundary);
  Py_CLEAR(p->_current_header_name);
  Py_CLEAR(p->_current_header_value);
  Py_CLEAR(p->_on_body_begin);
  Py_CLEAR(p->_on_part_begin);
  Py_CLEAR(p->_on_header);
  Py_CLEAR(p->_on_headers_complete);
  Py_CLEAR(p->_on_data);
  Py_CLEAR(p->_on_part_complete);
  Py_CLEAR(p->_on_body_complete);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_11multifruits_Parser(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_11multifruits_Parser *p = (struct __pyx_obj_11multifruits_Parser *)o;
  if (p->_on_body_begin) {
    e = (*v)(p->_on_body_begin, a); if (e) return e;
  }
  if (p->_on_part_begin) {
    e = (*v)(p->_on_part_begin, a); if (e) return e;
  }
  if (p->_on_header) {
    e = (*v)(p->_on_header, a); if (e) return e;
  }
  if (p->_on_headers_complete) {
    e = (*v)(p->_on_headers_complete, a); if (e) return e;
  }
  if (p->_on_data) {
    e = (*v)(p->_on_data, a); if (e) return e;
  }
  if (p->_on_part_complete) {
    e = (*v)(p->_on_part_complete, a); if (e) return e;
  }
  if (p->_on_body_complete) {
    e = (*v)(p->_on_body_complete, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_11multifruits_Parser(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_11multifruits_Parser *p = (struct __pyx_obj_11multifruits_Parser *)o;
  tmp = ((PyObject*)p->_on_body_begin);
  p->_on_body_begin = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_on_part_begin);
  p->_on_part_begin = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_on_header);
  p->_on_header = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_on_headers_complete);
  p->_on_headers_complete = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_on_data);
  p->_on_data = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_on_part_complete);
  p->_on_part_complete = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_on_body_complete);
  p->_on_body_complete = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_11multifruits_Parser[] = {
  {"_maybe_call_on_header", (PyCFunction)__pyx_pw_11multifruits_6Parser_3_maybe_call_on_header, METH_NOARGS, 0},
  {"on_header_name", (PyCFunction)__pyx_pw_11multifruits_6Parser_5on_header_name, METH_O, 0},
  {"on_header_value", (PyCFunction)__pyx_pw_11multifruits_6Parser_7on_header_value, METH_O, 0},
  {"on_headers_complete", (PyCFunction)__pyx_pw_11multifruits_6Parser_9on_headers_complete, METH_NOARGS, 0},
  {"feed_data", (PyCFunction)__pyx_pw_11multifruits_6Parser_11feed_data, METH_O, 0},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_11multifruits_6Parser_13__reduce_cython__, METH_NOARGS, 0},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_11multifruits_6Parser_15__setstate_cython__, METH_O, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_11multifruits_Parser = {
  PyVarObject_HEAD_INIT(0, 0)
  "multifruits.Parser", /*tp_name*/
  sizeof(struct __pyx_obj_11multifruits_Parser), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_11multifruits_Parser, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_11multifruits_Parser, /*tp_traverse*/
  __pyx_tp_clear_11multifruits_Parser, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_11multifruits_Parser, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_11multifruits_6Parser_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_11multifruits_Parser, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {"parse_content_disposition", (PyCFunction)__pyx_pw_11multifruits_1parse_content_disposition, METH_O, 0},
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_multifruits(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_multifruits},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "multifruits",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_u_DATA_BOUNDARY_DONE, __pyx_k_DATA_BOUNDARY_DONE, sizeof(__pyx_k_DATA_BOUNDARY_DONE), 0, 1, 0, 1},
  {&__pyx_n_u_DATA_BOUNDARY_DONE_CR_LF, __pyx_k_DATA_BOUNDARY_DONE_CR_LF, sizeof(__pyx_k_DATA_BOUNDARY_DONE_CR_LF), 0, 1, 0, 1},
  {&__pyx_n_u_DATA_BOUNDARY_DONE_HY_HY, __pyx_k_DATA_BOUNDARY_DONE_HY_HY, sizeof(__pyx_k_DATA_BOUNDARY_DONE_HY_HY), 0, 1, 0, 1},
  {&__pyx_n_u_FIRST_BOUNDARY, __pyx_k_FIRST_BOUNDARY, sizeof(__pyx_k_FIRST_BOUNDARY), 0, 1, 0, 1},
  {&__pyx_n_u_FIRST_BOUNDARY_DONE, __pyx_k_FIRST_BOUNDARY_DONE, sizeof(__pyx_k_FIRST_BOUNDARY_DONE), 0, 1, 0, 1},
  {&__pyx_kp_u_FIRST_BOUNDARY_r, __pyx_k_FIRST_BOUNDARY_r, sizeof(__pyx_k_FIRST_BOUNDARY_r), 0, 1, 0, 0},
  {&__pyx_n_u_HEADERS_DONE, __pyx_k_HEADERS_DONE, sizeof(__pyx_k_HEADERS_DONE), 0, 1, 0, 1},
  {&__pyx_n_u_HEADER_NAME, __pyx_k_HEADER_NAME, sizeof(__pyx_k_HEADER_NAME), 0, 1, 0, 1},
  {&__pyx_n_u_HEADER_VALUE_CR, __pyx_k_HEADER_VALUE_CR, sizeof(__pyx_k_HEADER_VALUE_CR), 0, 1, 0, 1},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x7a, __pyx_k_Incompatible_checksums_s_vs_0x7a, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x7a), 0, 0, 1, 0},
  {&__pyx_n_s_KeyError, __pyx_k_KeyError, sizeof(__pyx_k_KeyError), 0, 0, 1, 1},
  {&__pyx_n_s_LookupError, __pyx_k_LookupError, sizeof(__pyx_k_LookupError), 0, 0, 1, 1},
  {&__pyx_kp_u_Missing_boundary_in_Content_Type, __pyx_k_Missing_boundary_in_Content_Type, sizeof(__pyx_k_Missing_boundary_in_Content_Type), 0, 1, 0, 0},
  {&__pyx_n_s_PickleError, __pyx_k_PickleError, sizeof(__pyx_k_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_UnicodeDecodeError, __pyx_k_UnicodeDecodeError, sizeof(__pyx_k_UnicodeDecodeError), 0, 0, 1, 1},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_kp_b__10, __pyx_k__10, sizeof(__pyx_k__10), 0, 0, 0, 0},
  {&__pyx_kp_b__12, __pyx_k__12, sizeof(__pyx_k__12), 0, 0, 0, 0},
  {&__pyx_kp_b__14, __pyx_k__14, sizeof(__pyx_k__14), 0, 0, 0, 0},
  {&__pyx_kp_b__19, __pyx_k__19, sizeof(__pyx_k__19), 0, 0, 0, 0},
  {&__pyx_kp_b__20, __pyx_k__20, sizeof(__pyx_k__20), 0, 0, 0, 0},
  {&__pyx_kp_b__23, __pyx_k__23, sizeof(__pyx_k__23), 0, 0, 0, 0},
  {&__pyx_kp_b__8, __pyx_k__8, sizeof(__pyx_k__8), 0, 0, 0, 0},
  {&__pyx_n_b_boundary, __pyx_k_boundary, sizeof(__pyx_k_boundary), 0, 0, 0, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_content_type, __pyx_k_content_type, sizeof(__pyx_k_content_type), 0, 0, 1, 1},
  {&__pyx_n_s_decode, __pyx_k_decode, sizeof(__pyx_k_decode), 0, 0, 1, 1},
  {&__pyx_n_s_dict, __pyx_k_dict, sizeof(__pyx_k_dict), 0, 0, 1, 1},
  {&__pyx_n_s_encoding, __pyx_k_encoding, sizeof(__pyx_k_encoding), 0, 0, 1, 1},
  {&__pyx_n_s_errors, __pyx_k_errors, sizeof(__pyx_k_errors), 0, 0, 1, 1},
  {&__pyx_n_s_extract_filename, __pyx_k_extract_filename, sizeof(__pyx_k_extract_filename), 0, 0, 1, 1},
  {&__pyx_kp_b_filename, __pyx_k_filename, sizeof(__pyx_k_filename), 0, 0, 0, 0},
  {&__pyx_n_b_filename_2, __pyx_k_filename_2, sizeof(__pyx_k_filename_2), 0, 0, 0, 1},
  {&__pyx_n_s_filename_2, __pyx_k_filename_2, sizeof(__pyx_k_filename_2), 0, 0, 1, 1},
  {&__pyx_n_s_get, __pyx_k_get, sizeof(__pyx_k_get), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_handler, __pyx_k_handler, sizeof(__pyx_k_handler), 0, 0, 1, 1},
  {&__pyx_n_u_ignore, __pyx_k_ignore, sizeof(__pyx_k_ignore), 0, 1, 0, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_lower, __pyx_k_lower, sizeof(__pyx_k_lower), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_maybe_call_on_header, __pyx_k_maybe_call_on_header, sizeof(__pyx_k_maybe_call_on_header), 0, 0, 1, 1},
  {&__pyx_n_s_multifruits, __pyx_k_multifruits, sizeof(__pyx_k_multifruits), 0, 0, 1, 1},
  {&__pyx_kp_s_multifruits_pyx, __pyx_k_multifruits_pyx, sizeof(__pyx_k_multifruits_pyx), 0, 0, 1, 0},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_new, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
  {&__pyx_n_u_on_body_begin, __pyx_k_on_body_begin, sizeof(__pyx_k_on_body_begin), 0, 1, 0, 1},
  {&__pyx_n_u_on_body_complete, __pyx_k_on_body_complete, sizeof(__pyx_k_on_body_complete), 0, 1, 0, 1},
  {&__pyx_n_u_on_data, __pyx_k_on_data, sizeof(__pyx_k_on_data), 0, 1, 0, 1},
  {&__pyx_n_u_on_header, __pyx_k_on_header, sizeof(__pyx_k_on_header), 0, 1, 0, 1},
  {&__pyx_n_s_on_header_name, __pyx_k_on_header_name, sizeof(__pyx_k_on_header_name), 0, 0, 1, 1},
  {&__pyx_n_s_on_header_value, __pyx_k_on_header_value, sizeof(__pyx_k_on_header_value), 0, 0, 1, 1},
  {&__pyx_n_s_on_headers_complete, __pyx_k_on_headers_complete, sizeof(__pyx_k_on_headers_complete), 0, 0, 1, 1},
  {&__pyx_n_u_on_headers_complete, __pyx_k_on_headers_complete, sizeof(__pyx_k_on_headers_complete), 0, 1, 0, 1},
  {&__pyx_n_u_on_part_begin, __pyx_k_on_part_begin, sizeof(__pyx_k_on_part_begin), 0, 1, 0, 1},
  {&__pyx_n_u_on_part_complete, __pyx_k_on_part_complete, sizeof(__pyx_k_on_part_complete), 0, 1, 0, 1},
  {&__pyx_n_s_params, __pyx_k_params, sizeof(__pyx_k_params), 0, 0, 1, 1},
  {&__pyx_n_s_pickle, __pyx_k_pickle, sizeof(__pyx_k_pickle), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_PickleError, __pyx_k_pyx_PickleError, sizeof(__pyx_k_pyx_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_checksum, __pyx_k_pyx_checksum, sizeof(__pyx_k_pyx_checksum), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_result, __pyx_k_pyx_result, sizeof(__pyx_k_pyx_result), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_state, __pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_type, __pyx_k_pyx_type, sizeof(__pyx_k_pyx_type), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_Parser, __pyx_k_pyx_unpickle_Parser, sizeof(__pyx_k_pyx_unpickle_Parser), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_replace, __pyx_k_replace, sizeof(__pyx_k_replace), 0, 0, 1, 1},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_split, __pyx_k_split, sizeof(__pyx_k_split), 0, 0, 1, 1},
  {&__pyx_kp_s_stringsource, __pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 0, 1, 0},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_KeyError = __Pyx_GetBuiltinName(__pyx_n_s_KeyError); if (!__pyx_builtin_KeyError) __PYX_ERR(0, 45, __pyx_L1_error)
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(0, 46, __pyx_L1_error)
  __pyx_builtin_LookupError = __Pyx_GetBuiltinName(__pyx_n_s_LookupError); if (!__pyx_builtin_LookupError) __PYX_ERR(0, 320, __pyx_L1_error)
  __pyx_builtin_UnicodeDecodeError = __Pyx_GetBuiltinName(__pyx_n_s_UnicodeDecodeError); if (!__pyx_builtin_UnicodeDecodeError) __PYX_ERR(0, 320, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "multifruits.pyx":46
 *             self._boundary = params[b'boundary']
 *         except KeyError:
 *             raise ValueError('Missing boundary in Content-Type.')             # <<<<<<<<<<<<<<
 *         self._boundary_length = len(self._boundary)
 *         self._current_header_name = None
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_u_Missing_boundary_in_Content_Type); if (unlikely(!__pyx_tuple_)) __PYX_ERR(0, 46, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "multifruits.pyx":109
 *                     if self._boundary_index == self._boundary_length:
 *                         if c != b'\r':
 *                             raise ValueError('FIRST_BOUNDARY: \\r')             # <<<<<<<<<<<<<<
 *                         self._state = FIRST_BOUNDARY_DONE
 *                         i += 1
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_u_FIRST_BOUNDARY_r); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(0, 109, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "multifruits.pyx":116
 *                         self._boundary_index += 1
 *                     else:
 *                         raise ValueError('FIRST_BOUNDARY')             # <<<<<<<<<<<<<<
 *                     i += 1
 *             elif self._state == FIRST_BOUNDARY_DONE:
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_n_u_FIRST_BOUNDARY); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(0, 116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "multifruits.pyx":120
 *             elif self._state == FIRST_BOUNDARY_DONE:
 *                 if c != b'\n':
 *                     raise ValueError('FIRST_BOUNDARY_DONE')             # <<<<<<<<<<<<<<
 *                 if self._on_body_begin is not None:
 *                     self._on_body_begin()
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_n_u_FIRST_BOUNDARY_DONE); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "multifruits.pyx":144
 *                                b'\\', b'"', b'/', b'[', b']', b'?', b'=', b'{',
 *                                b'}', b' ', b'\t'):
 *                         raise ValueError('HEADER_NAME')             # <<<<<<<<<<<<<<
 *                     i += 1
 *                 if i > mark:
 */
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_n_u_HEADER_NAME); if (unlikely(!__pyx_tuple__5)) __PYX_ERR(0, 144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "multifruits.pyx":167
 *             elif self._state == HEADER_VALUE_CR:
 *                 if c != b'\n':
 *                     raise ValueError('HEADER_VALUE_CR')             # <<<<<<<<<<<<<<
 *                 self._state = HEADER_NAME_START
 *                 i += 1
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_n_u_HEADER_VALUE_CR); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(0, 167, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "multifruits.pyx":172
 *             elif self._state == HEADERS_DONE:
 *                 if c != b'\n':
 *                     raise ValueError('HEADERS_DONE')             # <<<<<<<<<<<<<<
 *                 self.on_headers_complete()
 *                 self._state = DATA
 */
  __pyx_tuple__7 = PyTuple_Pack(1, __pyx_n_u_HEADERS_DONE); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);

  /* "multifruits.pyx":194
 *                 else:
 *                     if self._on_data is not None:
 *                         self._on_data(b'\r')             # <<<<<<<<<<<<<<
 *                     self._state = DATA
 *             elif self._state == DATA_CR_LF:
 */
  __pyx_tuple__9 = PyTuple_Pack(1, __pyx_kp_b__8); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(0, 194, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);

  /* "multifruits.pyx":202
 *                 else:
 *                     if self._on_data is not None:
 *                         self._on_data(b'\r\n')             # <<<<<<<<<<<<<<
 *                     self._state = DATA
 *             elif self._state == DATA_CR_LF_HY:
 */
  __pyx_tuple__11 = PyTuple_Pack(1, __pyx_kp_b__10); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(0, 202, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);

  /* "multifruits.pyx":211
 *                 else:
 *                     if self._on_data is not None:
 *                         self._on_data(b'\r\n-')             # <<<<<<<<<<<<<<
 *                     self._state = DATA
 *             elif self._state == DATA_BOUNDARY:
 */
  __pyx_tuple__13 = PyTuple_Pack(1, __pyx_kp_b__12); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);

  /* "multifruits.pyx":224
 *                     else:
 *                         if self._on_data is not None:
 *                             self._on_data(b'\r\n--')             # <<<<<<<<<<<<<<
 *                             self._on_data(self._boundary[:self._boundary_index])
 *                         self._state = DATA
 */
  __pyx_tuple__15 = PyTuple_Pack(1, __pyx_kp_b__14); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(0, 224, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);

  /* "multifruits.pyx":234
 *                     self._state = DATA_BOUNDARY_DONE_HY_HY
 *                 else:
 *                     raise ValueError('DATA_BOUNDARY_DONE')             # <<<<<<<<<<<<<<
 *                 i += 1
 *             elif self._state == DATA_BOUNDARY_DONE_CR_LF:
 */
  __pyx_tuple__16 = PyTuple_Pack(1, __pyx_n_u_DATA_BOUNDARY_DONE); if (unlikely(!__pyx_tuple__16)) __PYX_ERR(0, 234, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__16);
  __Pyx_GIVEREF(__pyx_tuple__16);

  /* "multifruits.pyx":238
 *             elif self._state == DATA_BOUNDARY_DONE_CR_LF:
 *                 if c != b'\n':
 *                     raise ValueError('DATA_BOUNDARY_DONE_CR_LF')             # <<<<<<<<<<<<<<
 *                 if self._on_part_complete is not None:
 *                     self._on_part_complete()
 */
  __pyx_tuple__17 = PyTuple_Pack(1, __pyx_n_u_DATA_BOUNDARY_DONE_CR_LF); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(0, 238, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);

  /* "multifruits.pyx":247
 *             elif self._state == DATA_BOUNDARY_DONE_HY_HY:
 *                 if c != b'-':
 *                     raise ValueError('DATA_BOUNDARY_DONE_HY_HY')             # <<<<<<<<<<<<<<
 *                 if self._on_part_complete is not None:
 *                     self._on_part_complete()
 */
  __pyx_tuple__18 = PyTuple_Pack(1, __pyx_n_u_DATA_BOUNDARY_DONE_HY_HY); if (unlikely(!__pyx_tuple__18)) __PYX_ERR(0, 247, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__18);
  __Pyx_GIVEREF(__pyx_tuple__18);

  /* "multifruits.pyx":281
 *                 dtype = data[start:end]
 *             elif field is not None:
 *                 params[field.lower()] = data[start:end].replace(b'\\', b'')             # <<<<<<<<<<<<<<
 *                 field = None
 *             i += 1
 */
  __pyx_tuple__21 = PyTuple_Pack(2, __pyx_kp_b__20, __pyx_kp_b__19); if (unlikely(!__pyx_tuple__21)) __PYX_ERR(0, 281, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__21);
  __Pyx_GIVEREF(__pyx_tuple__21);

  /* "multifruits.pyx":309
 *             dtype = data[start:end].lower()
 *         elif field is not None:
 *             params[field.lower()] = data[start:end].replace(b'\\', b'')             # <<<<<<<<<<<<<<
 *     return dtype, params
 * 
 */
  __pyx_tuple__22 = PyTuple_Pack(2, __pyx_kp_b__20, __pyx_kp_b__19); if (unlikely(!__pyx_tuple__22)) __PYX_ERR(0, 309, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__22);
  __Pyx_GIVEREF(__pyx_tuple__22);

  /* "multifruits.pyx":317
 *         filename = params.get(b'filename*')
 *         if b"''" in filename:
 *             encoding, filename = filename.split(b"''")             # <<<<<<<<<<<<<<
 *             try:
 *                 return filename.decode(encoding.decode())
 */
  __pyx_tuple__24 = PyTuple_Pack(1, __pyx_kp_b__23); if (unlikely(!__pyx_tuple__24)) __PYX_ERR(0, 317, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__24);
  __Pyx_GIVEREF(__pyx_tuple__24);

  /* "multifruits.pyx":313
 * 
 * 
 * def extract_filename(params: dict):             # <<<<<<<<<<<<<<
 *     if b'filename*' in params:
 *         filename = params.get(b'filename*')
 */
  __pyx_tuple__25 = PyTuple_Pack(3, __pyx_n_s_params, __pyx_n_s_filename_2, __pyx_n_s_encoding); if (unlikely(!__pyx_tuple__25)) __PYX_ERR(0, 313, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__25);
  __Pyx_GIVEREF(__pyx_tuple__25);
  __pyx_codeobj__26 = (PyObject*)__Pyx_PyCode_New(1, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__25, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_multifruits_pyx, __pyx_n_s_extract_filename, 313, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__26)) __PYX_ERR(0, 313, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __pyx_unpickle_Parser(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x7a58f64:
 *         from pickle import PickleError as __pyx_PickleError
 */
  __pyx_tuple__27 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__27)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__27);
  __Pyx_GIVEREF(__pyx_tuple__27);
  __pyx_codeobj__28 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__27, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_Parser, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__28)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_int_128290660 = PyInt_FromLong(128290660L); if (unlikely(!__pyx_int_128290660)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC initmultifruits(void); /*proto*/
PyMODINIT_FUNC initmultifruits(void)
#else
PyMODINIT_FUNC PyInit_multifruits(void); /*proto*/
PyMODINIT_FUNC PyInit_multifruits(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        result = PyDict_SetItemString(moddict, to_name, value);
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__") < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__") < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static int __pyx_pymod_exec_multifruits(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m && __pyx_m == __pyx_pyinit_module) return 0;
  #endif
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit_multifruits(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("multifruits", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_multifruits) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "multifruits")) {
      if (unlikely(PyDict_SetItemString(modules, "multifruits", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global init code ---*/
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  /*--- Type init code ---*/
  __pyx_vtabptr_11multifruits_Parser = &__pyx_vtable_11multifruits_Parser;
  __pyx_vtable_11multifruits_Parser._feed_data = (PyObject *(*)(struct __pyx_obj_11multifruits_Parser *, char *))__pyx_f_11multifruits_6Parser__feed_data;
  if (PyType_Ready(&__pyx_type_11multifruits_Parser) < 0) __PYX_ERR(0, 27, __pyx_L1_error)
  __pyx_type_11multifruits_Parser.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_11multifruits_Parser.tp_dict, __pyx_vtabptr_11multifruits_Parser) < 0) __PYX_ERR(0, 27, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "Parser", (PyObject *)&__pyx_type_11multifruits_Parser) < 0) __PYX_ERR(0, 27, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_11multifruits_Parser) < 0) __PYX_ERR(0, 27, __pyx_L1_error)
  __pyx_ptype_11multifruits_Parser = &__pyx_type_11multifruits_Parser;
  /*--- Type import code ---*/
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__Pyx_BUILTIN_MODULE_NAME, "type", 
  #if CYTHON_COMPILING_IN_PYPY
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  0); if (unlikely(!__pyx_ptype_7cpython_4type_type)) __PYX_ERR(2, 9, __pyx_L1_error)
  __pyx_ptype_7cpython_4bool_bool = __Pyx_ImportType(__Pyx_BUILTIN_MODULE_NAME, "bool", sizeof(PyBoolObject), 0); if (unlikely(!__pyx_ptype_7cpython_4bool_bool)) __PYX_ERR(3, 8, __pyx_L1_error)
  __pyx_ptype_7cpython_7complex_complex = __Pyx_ImportType(__Pyx_BUILTIN_MODULE_NAME, "complex", sizeof(PyComplexObject), 0); if (unlikely(!__pyx_ptype_7cpython_7complex_complex)) __PYX_ERR(4, 15, __pyx_L1_error)
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "multifruits.pyx":313
 * 
 * 
 * def extract_filename(params: dict):             # <<<<<<<<<<<<<<
 *     if b'filename*' in params:
 *         filename = params.get(b'filename*')
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_11multifruits_3extract_filename, NULL, __pyx_n_s_multifruits); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 313, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_extract_filename, __pyx_t_1) < 0) __PYX_ERR(0, 313, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_Parser(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     if __pyx_checksum != 0x7a58f64:
 *         from pickle import PickleError as __pyx_PickleError
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_11multifruits_5__pyx_unpickle_Parser, NULL, __pyx_n_s_multifruits); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_Parser, __pyx_t_1) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "multifruits.pyx":1
 * # cython: language_level=3             # <<<<<<<<<<<<<<
 * 
 * from cpython cimport bool
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_1) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init multifruits", 0, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init multifruits");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* IterFinish */
static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* UnpackItemEndCheck */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if PY_VERSION_HEX >= 0x030700A2
    *type = tstate->exc_state.exc_type;
    *value = tstate->exc_state.exc_value;
    *tb = tstate->exc_state.exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if PY_VERSION_HEX >= 0x030700A2
    tmp_type = tstate->exc_state.exc_type;
    tmp_value = tstate->exc_state.exc_value;
    tmp_tb = tstate->exc_state.exc_traceback;
    tstate->exc_state.exc_type = type;
    tstate->exc_state.exc_value = value;
    tstate->exc_state.exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb) {
#endif
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if PY_VERSION_HEX >= 0x030700A2
    tmp_type = tstate->exc_state.exc_type;
    tmp_value = tstate->exc_state.exc_value;
    tmp_tb = tstate->exc_state.exc_traceback;
    tstate->exc_state.exc_type = local_type;
    tstate->exc_state.exc_value = local_value;
    tstate->exc_state.exc_traceback = local_tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* PyObjectCall */
  #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyErrFetchRestore */
  #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* RaiseException */
  #if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* GetAttr */
  static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* GetAttr3 */
  static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r = __Pyx_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
}

/* PyFunctionFastCall */
  #if CYTHON_FAST_PYCALL
#include "frameobject.h"
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = f->f_localsplus;
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyCFunctionFastCall */
  #if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)meth)) (self, args, nargs);
    }
}
#endif

/* PyObjectCallMethO */
  #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
  #if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyObjectCallNoArg */
  #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* GetItemInt */
    static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely((0 <= wrapped_i) & (wrapped_i < PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely((n >= 0) & (n < PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely((n >= 0) & (n < PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* GetModuleGlobalName */
    static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name) {
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
    result = PyDict_GetItem(__pyx_d, name);
    if (likely(result)) {
        Py_INCREF(result);
    } else {
#else
    result = PyObject_GetItem(__pyx_d, name);
    if (!result) {
        PyErr_Clear();
#endif
        result = __Pyx_GetBuiltinName(name);
    }
    return result;
}

/* bytes_index */
      static CYTHON_INLINE char __Pyx_PyBytes_GetItemInt(PyObject* bytes, Py_ssize_t index, int check_bounds) {
    if (check_bounds) {
        Py_ssize_t size = PyBytes_GET_SIZE(bytes);
        if (unlikely(index >= size) | ((index < 0) & unlikely(index < -size))) {
            PyErr_SetString(PyExc_IndexError, "string index out of range");
            return (char) -1;
        }
    }
    if (index < 0)
        index += PyBytes_GET_SIZE(bytes);
    return PyBytes_AS_STRING(bytes)[index];
}

/* BytesEquals */
      static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* ExtTypeTest */
      static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* dict_getitem_default */
      static PyObject* __Pyx_PyDict_GetItemDefault(PyObject* d, PyObject* key, PyObject* default_value) {
    PyObject* value;
#if PY_MAJOR_VERSION >= 3 && !CYTHON_COMPILING_IN_PYPY
    value = PyDict_GetItemWithError(d, key);
    if (unlikely(!value)) {
        if (unlikely(PyErr_Occurred()))
            return NULL;
        value = default_value;
    }
    Py_INCREF(value);
#else
    if (PyString_CheckExact(key) || PyUnicode_CheckExact(key) || PyInt_CheckExact(key)) {
        value = PyDict_GetItem(d, key);
        if (unlikely(!value)) {
            value = default_value;
        }
        Py_INCREF(value);
    } else {
        if (default_value == Py_None)
            default_value = NULL;
        value = PyObject_CallMethodObjArgs(
            d, __pyx_n_s_get, key, default_value, NULL);
    }
#endif
    return value;
}

/* Import */
      static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
      static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* HasAttr */
      static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (unlikely(!r)) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* SetVTable */
      static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* SetupReduce */
      static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_cython); if (unlikely(!reduce_cython)) goto BAD;
            ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto BAD;
            ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto BAD;
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate_cython); if (unlikely(!setstate_cython)) goto BAD;
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto BAD;
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto GOOD;
BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* CLineInTraceback */
      #ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
      use_cline = PyDict_GetItem(*cython_runtime_dict, __pyx_n_s_cline_in_traceback);
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (PyObject_Not(use_cline) != 0) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
      static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
      #include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntToPy */
      static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_int(unsigned int value) {
    const unsigned int neg_one = (unsigned int) -1, const_zero = (unsigned int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(unsigned int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(unsigned int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(unsigned int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(unsigned int),
                                     little, !is_unsigned);
    }
}

/* CIntFromPyVerify */
      #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
      static CYTHON_INLINE PyObject* __Pyx_PyInt_From_unsigned_char(unsigned char value) {
    const unsigned char neg_one = (unsigned char) -1, const_zero = (unsigned char) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(unsigned char) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(unsigned char) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned char) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(unsigned char) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(unsigned char) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(unsigned char),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
      static CYTHON_INLINE PyObject* __Pyx_PyInt_From_char(char value) {
    const char neg_one = (char) -1, const_zero = (char) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(char) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(char) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(char) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(char) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(char) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(char),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
      static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
      static CYTHON_INLINE unsigned int __Pyx_PyInt_As_unsigned_int(PyObject *x) {
    const unsigned int neg_one = (unsigned int) -1, const_zero = (unsigned int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(unsigned int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(unsigned int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (unsigned int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(unsigned int, digit, digits[0])
                case 2:
                    if (8 * sizeof(unsigned int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) >= 2 * PyLong_SHIFT) {
                            return (unsigned int) (((((unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) >= 3 * PyLong_SHIFT) {
                            return (unsigned int) (((((((unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) >= 4 * PyLong_SHIFT) {
                            return (unsigned int) (((((((((unsigned int)digits[3]) << PyLong_SHIFT) | (unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (unsigned int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(unsigned int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(unsigned int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(unsigned int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(unsigned int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned int) (((unsigned int)-1)*(((((unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(unsigned int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned int) ((((((unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(unsigned int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned int) (((unsigned int)-1)*(((((((unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned int) ((((((((unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(unsigned int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned int) (((unsigned int)-1)*(((((((((unsigned int)digits[3]) << PyLong_SHIFT) | (unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned int) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned int) ((((((((((unsigned int)digits[3]) << PyLong_SHIFT) | (unsigned int)digits[2]) << PyLong_SHIFT) | (unsigned int)digits[1]) << PyLong_SHIFT) | (unsigned int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(unsigned int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            unsigned int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (unsigned int) -1;
        }
    } else {
        unsigned int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (unsigned int) -1;
        val = __Pyx_PyInt_As_unsigned_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to unsigned int");
    return (unsigned int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to unsigned int");
    return (unsigned int) -1;
}

/* CIntFromPy */
      static CYTHON_INLINE unsigned char __Pyx_PyInt_As_unsigned_char(PyObject *x) {
    const unsigned char neg_one = (unsigned char) -1, const_zero = (unsigned char) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(unsigned char) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(unsigned char, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (unsigned char) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned char) 0;
                case  1: __PYX_VERIFY_RETURN_INT(unsigned char, digit, digits[0])
                case 2:
                    if (8 * sizeof(unsigned char) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) >= 2 * PyLong_SHIFT) {
                            return (unsigned char) (((((unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned char) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) >= 3 * PyLong_SHIFT) {
                            return (unsigned char) (((((((unsigned char)digits[2]) << PyLong_SHIFT) | (unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned char) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) >= 4 * PyLong_SHIFT) {
                            return (unsigned char) (((((((((unsigned char)digits[3]) << PyLong_SHIFT) | (unsigned char)digits[2]) << PyLong_SHIFT) | (unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (unsigned char) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(unsigned char) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned char, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned char) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned char, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (unsigned char) 0;
                case -1: __PYX_VERIFY_RETURN_INT(unsigned char, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(unsigned char,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(unsigned char) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned char) (((unsigned char)-1)*(((((unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(unsigned char) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) - 1 > 2 * PyLong_SHIFT) {
                            return (unsigned char) ((((((unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(unsigned char) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned char) (((unsigned char)-1)*(((((((unsigned char)digits[2]) << PyLong_SHIFT) | (unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(unsigned char) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) - 1 > 3 * PyLong_SHIFT) {
                            return (unsigned char) ((((((((unsigned char)digits[2]) << PyLong_SHIFT) | (unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(unsigned char) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned char) (((unsigned char)-1)*(((((((((unsigned char)digits[3]) << PyLong_SHIFT) | (unsigned char)digits[2]) << PyLong_SHIFT) | (unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(unsigned char) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(unsigned char, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(unsigned char) - 1 > 4 * PyLong_SHIFT) {
                            return (unsigned char) ((((((((((unsigned char)digits[3]) << PyLong_SHIFT) | (unsigned char)digits[2]) << PyLong_SHIFT) | (unsigned char)digits[1]) << PyLong_SHIFT) | (unsigned char)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(unsigned char) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned char, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(unsigned char) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(unsigned char, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            unsigned char val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (unsigned char) -1;
        }
    } else {
        unsigned char val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (unsigned char) -1;
        val = __Pyx_PyInt_As_unsigned_char(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to unsigned char");
    return (unsigned char) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to unsigned char");
    return (unsigned char) -1;
}

/* CIntFromPy */
      static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
      static CYTHON_INLINE char __Pyx_PyInt_As_char(PyObject *x) {
    const char neg_one = (char) -1, const_zero = (char) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(char) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(char, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (char) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (char) 0;
                case  1: __PYX_VERIFY_RETURN_INT(char, digit, digits[0])
                case 2:
                    if (8 * sizeof(char) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) >= 2 * PyLong_SHIFT) {
                            return (char) (((((char)digits[1]) << PyLong_SHIFT) | (char)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(char) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) >= 3 * PyLong_SHIFT) {
                            return (char) (((((((char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(char) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) >= 4 * PyLong_SHIFT) {
                            return (char) (((((((((char)digits[3]) << PyLong_SHIFT) | (char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (char) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(char) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(char, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(char) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(char, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (char) 0;
                case -1: __PYX_VERIFY_RETURN_INT(char, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(char,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(char) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) - 1 > 2 * PyLong_SHIFT) {
                            return (char) (((char)-1)*(((((char)digits[1]) << PyLong_SHIFT) | (char)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(char) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) - 1 > 2 * PyLong_SHIFT) {
                            return (char) ((((((char)digits[1]) << PyLong_SHIFT) | (char)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(char) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) - 1 > 3 * PyLong_SHIFT) {
                            return (char) (((char)-1)*(((((((char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(char) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) - 1 > 3 * PyLong_SHIFT) {
                            return (char) ((((((((char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(char) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) - 1 > 4 * PyLong_SHIFT) {
                            return (char) (((char)-1)*(((((((((char)digits[3]) << PyLong_SHIFT) | (char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(char) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(char, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(char) - 1 > 4 * PyLong_SHIFT) {
                            return (char) ((((((((((char)digits[3]) << PyLong_SHIFT) | (char)digits[2]) << PyLong_SHIFT) | (char)digits[1]) << PyLong_SHIFT) | (char)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(char) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(char, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(char) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(char, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            char val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (char) -1;
        }
    } else {
        char val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (char) -1;
        val = __Pyx_PyInt_As_char(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to char");
    return (char) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to char");
    return (char) -1;
}

/* CIntFromPy */
      static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* FastTypeChecks */
      #if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
      static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* ModuleImport */
      #ifndef __PYX_HAVE_RT_ImportModule
#define __PYX_HAVE_RT_ImportModule
static PyObject *__Pyx_ImportModule(const char *name) {
    PyObject *py_name = 0;
    PyObject *py_module = 0;
    py_name = __Pyx_PyIdentifier_FromString(name);
    if (!py_name)
        goto bad;
    py_module = PyImport_Import(py_name);
    Py_DECREF(py_name);
    return py_module;
bad:
    Py_XDECREF(py_name);
    return 0;
}
#endif

/* TypeImport */
      #ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(const char *module_name, const char *class_name,
    size_t size, int strict)
{
    PyObject *py_module = 0;
    PyObject *result = 0;
    PyObject *py_name = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    py_module = __Pyx_ImportModule(module_name);
    if (!py_module)
        goto bad;
    py_name = __Pyx_PyIdentifier_FromString(class_name);
    if (!py_name)
        goto bad;
    result = PyObject_GetAttr(py_module, py_name);
    Py_DECREF(py_name);
    py_name = 0;
    Py_DECREF(py_module);
    py_module = 0;
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (!strict && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    else if ((size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s has the wrong size, try recompiling. Expected %zd, got %zd",
            module_name, class_name, basicsize, size);
        goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(py_module);
    Py_XDECREF(result);
    return NULL;
}
#endif

/* InitStrings */
      static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            PyErr_Clear();
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
